@@ -1,19 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkCellTreeLocator.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
-// VTK_DEPRECATED_IN_9_2_0() warnings for this class.
-#define VTK_DEPRECATION_LEVEL 0
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkCellTreeLocator.h"
 
@@ -30,12 +16,15 @@
 #include <stack>
 #include <vector>
 
+VTK_ABI_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
 vtkStandardNewMacro(vtkCellTreeLocator);
+VTK_ABI_NAMESPACE_END
 
 //------------------------------------------------------------------------------
 namespace detail
 {
+VTK_ABI_NAMESPACE_BEGIN
 enum
 {
   POS_X,
@@ -45,7 +34,7 @@ enum
   POS_Z,
   NEG_Z
 };
-#define CELLTREE_MAX_DEPTH 32
+#define CELLTREE_MAX_DEPTH 64
 
 //------------------------------------------------------------------------------
 // Perform locator operations like FindCell. Uses templated subclasses
@@ -93,33 +82,27 @@ struct vtkCellTree
       return ((dir[2] > 0) ? POS_Z : NEG_Z);
     }
   }
-  inline static double _getMinDistPOS_X(
-    const double origin[3], const double dir[3], const double B[6])
+  static double _getMinDistPOS_X(const double origin[3], const double dir[3], const double B[6])
   {
     return ((B[0] - origin[0]) / dir[0]);
   }
-  inline static double _getMinDistNEG_X(
-    const double origin[3], const double dir[3], const double B[6])
+  static double _getMinDistNEG_X(const double origin[3], const double dir[3], const double B[6])
   {
     return ((B[1] - origin[0]) / dir[0]);
   }
-  inline static double _getMinDistPOS_Y(
-    const double origin[3], const double dir[3], const double B[6])
+  static double _getMinDistPOS_Y(const double origin[3], const double dir[3], const double B[6])
   {
     return ((B[2] - origin[1]) / dir[1]);
   }
-  inline static double _getMinDistNEG_Y(
-    const double origin[3], const double dir[3], const double B[6])
+  static double _getMinDistNEG_Y(const double origin[3], const double dir[3], const double B[6])
   {
     return ((B[3] - origin[1]) / dir[1]);
   }
-  inline static double _getMinDistPOS_Z(
-    const double origin[3], const double dir[3], const double B[6])
+  static double _getMinDistPOS_Z(const double origin[3], const double dir[3], const double B[6])
   {
     return ((B[4] - origin[2]) / dir[2]);
   }
-  inline static double _getMinDistNEG_Z(
-    const double origin[3], const double dir[3], const double B[6])
+  static double _getMinDistNEG_Z(const double origin[3], const double dir[3], const double B[6])
   {
     return ((B[5] - origin[2]) / dir[2]);
   }
@@ -146,31 +129,31 @@ struct CellTreeNode
   /**
    * b is an array containing left max and right min values
    */
-  inline void MakeNode(const T& left, const T& d, const double b[2])
+  void MakeNode(const T& left, const T& d, const double b[2])
   {
     this->Index = (d & 3) | (left << 2);
     this->LeftMax = b[0];
     this->RightMin = b[1];
   }
-  inline void SetChildren(const T& left)
+  void SetChildren(const T& left)
   {
     // In index 2 LSBs (Least Significant Bits) store the dimension. MSBs store the position
     this->Index = this->GetDimension() | (left << 2);
   }
-  inline bool IsNode() const
+  bool IsNode() const
   {
     return (this->Index & 3) != 3; // For a leaf 2 LSBs in index is 3
   }
-  inline T GetLeftChildIndex() const { return (this->Index >> 2); }
-  inline T GetRightChildIndex() const
+  T GetLeftChildIndex() const { return (this->Index >> 2); }
+  T GetRightChildIndex() const
   {
     // Right child node is adjacent to the Left child node in the data structure
     return (this->Index >> 2) + 1;
   }
-  inline T GetDimension() const { return this->Index & 3; }
-  inline const double& GetLeftMaxValue() const { return this->LeftMax; }
-  inline const double& GetRightMinValue() const { return this->RightMin; }
-  inline void MakeLeaf(const T& start, const T& size)
+  T GetDimension() const { return this->Index & 3; }
+  const double& GetLeftMaxValue() const { return this->LeftMax; }
+  const double& GetRightMinValue() const { return this->RightMin; }
+  void MakeLeaf(const T& start, const T& size)
   {
     this->Index = 3;
     this->Sz = size;
@@ -312,23 +295,17 @@ private:
     T Cnt;
 
     Bucket()
-      : Cnt(0)
-      , Min(VTK_DOUBLE_MAX)
+      : Min(VTK_DOUBLE_MAX)
       , Max(-VTK_DOUBLE_MAX)
+      , Cnt(0)
     {
     }
 
-    inline void Add(const double& min, const double& max)
+    void Add(const double& min, const double& max)
     {
       ++this->Cnt;
-      if (min < this->Min)
-      {
-        this->Min = min;
-      }
-      if (max > this->Max)
-      {
-        this->Max = max;
-      }
+      this->Min = std::min(min, this->Min);
+      this->Max = std::max(max, this->Max);
     }
   };
 
@@ -347,7 +324,7 @@ private:
     {
     }
 
-    inline bool operator()(const CellInfo& pc0, const CellInfo& pc1)
+    bool operator()(const CellInfo& pc0, const CellInfo& pc1)
     {
       return (pc0.Min[this->D] + pc0.Max[this->D]) < (pc1.Min[this->D] + pc1.Max[this->D]);
     }
@@ -363,10 +340,7 @@ private:
     {
     }
 
-    inline bool operator()(const CellInfo& pc)
-    {
-      return pc.Min[this->D] + pc.Max[this->D] < this->P;
-    }
+    bool operator()(const CellInfo& pc) { return pc.Min[this->D] + pc.Max[this->D] < this->P; }
   };
 
   struct SplitInfo
@@ -463,30 +437,24 @@ private:
     CellInfo* mid = begin;
 
     const double ext[3] = { max[0] - min[0], max[1] - min[1], max[2] - min[2] };
-    const double iext[3] = { this->NumberOfBuckets / ext[0], this->NumberOfBuckets / ext[1],
-      this->NumberOfBuckets / ext[2] };
+    double iext[3];
+
+    for (uint8_t comp = 0; comp < 3; ++comp)
+    {
+      double const ext_comp = ext[comp];
+      iext[comp] = ext_comp == 0.0 ? this->NumberOfBuckets : this->NumberOfBuckets / ext_comp;
+    }
 
     buckets.Reset();
 
-    double cen;
-    int ind;
-
     for (const CellInfo* pc = begin; pc != end; ++pc)
     {
       for (uint8_t d = 0; d < 3; ++d)
       {
-        cen = (pc->Min[d] + pc->Max[d]) / 2.0;
-        ind = (int)((cen - min[d]) * iext[d]);
-
-        if (ind < 0)
-        {
-          ind = 0;
-        }
-
-        if (ind >= this->NumberOfBuckets)
-        {
-          ind = this->NumberOfBuckets - 1;
-        }
+        double cen = (pc->Min[d] + pc->Max[d]) / 2.0;
+        double dblIdx = (cen - min[d]) * iext[d];
+        dblIdx = vtkMath::ClampValue(dblIdx, 0.0, static_cast<double>(this->NumberOfBuckets - 1));
+        size_t ind = static_cast<size_t>(dblIdx);
 
         buckets[d][ind].Add(pc->Min[d], pc->Max[d]);
       }
@@ -524,8 +492,6 @@ private:
           }
         }
 
-        // JB : added if (...) to stop floating point error if rmin is unset
-        // this happens when some buckets are empty (bad volume calc)
         if (lMaxValue != -VTK_DOUBLE_MAX && rMinValue != VTK_DOUBLE_MAX)
         {
           sum += buckets[d][n].Cnt;
@@ -887,7 +853,7 @@ struct IntersectionInfo
 
 //------------------------------------------------------------------------------
 template <typename T>
-int CellTree<T>::IntersectWithLine(const double p1[3], const double p2[3], const double tol,
+int CellTree<T>::IntersectWithLine(const double p1[3], const double p2[3], double tol,
   vtkPoints* points, vtkIdList* cellIds, vtkGenericCell* cell)
 {
   using TIntersectionInfo = IntersectionInfo<T>;
@@ -1224,11 +1190,11 @@ void CellTree<T>::GenerateRepresentation(int level, vtkPolyData* pd)
     }
   }
   // For each node, add the bbox to our polydata
-  for (auto i = 0; i < bl.size(); i++)
+  for (auto const& b : bl)
   {
     double bounds[6];
-    bl[i].first.GetBounds(bounds);
-    AddBox(pd, bounds, bl[i].second);
+    b.first.GetBounds(bounds);
+    AddBox(pd, bounds, b.second);
   }
 }
 
@@ -1284,8 +1250,10 @@ void CellTree<T>::FindCellsWithinBounds(double* bbox, vtkIdList* cells)
     }
   }
 }
+VTK_ABI_NAMESPACE_END
 } // namespace
 
+VTK_ABI_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
 vtkCellTreeLocator::vtkCellTreeLocator()
 {
@@ -1394,7 +1362,7 @@ void vtkCellTreeLocator::FindCellsWithinBounds(double* bbox, vtkIdList* cells)
   {
     return;
   }
-  return this->Tree->FindCellsWithinBounds(bbox, cells);
+  this->Tree->FindCellsWithinBounds(bbox, cells);
 }
 
 //------------------------------------------------------------------------------
@@ -1445,7 +1413,6 @@ void vtkCellTreeLocator::ShallowCopy(vtkAbstractCellLocator* locator)
   // we only copy what's actually used by vtkCellTreeLocator
 
   // vtkLocator parameters
-  this->SetDataSet(cellLocator->GetDataSet());
   this->SetUseExistingSearchStructure(cellLocator->GetUseExistingSearchStructure());
 
   // vtkAbstractCellLocator parameters
@@ -1462,7 +1429,7 @@ void vtkCellTreeLocator::ShallowCopy(vtkAbstractCellLocator* locator)
     auto cellLocatorTree = static_cast<CellTree<vtkIdType>*>(cellLocator->Tree);
     auto tree = new CellTree<vtkIdType>(this);
     tree->Locator = this;
-    tree->DataSet = cellLocatorTree->DataSet;
+    tree->DataSet = this->DataSet;
     tree->Leaves = cellLocatorTree->Leaves;
     tree->Nodes = cellLocatorTree->Nodes;
     std::copy_n(cellLocatorTree->DataBBox, 6, tree->DataBBox);
@@ -1473,12 +1440,13 @@ void vtkCellTreeLocator::ShallowCopy(vtkAbstractCellLocator* locator)
     auto cellLocatorTree = static_cast<CellTree<int>*>(cellLocator->Tree);
     auto tree = new CellTree<int>(this);
     tree->Locator = this;
-    tree->DataSet = cellLocatorTree->DataSet;
+    tree->DataSet = this->DataSet;
     tree->Leaves = cellLocatorTree->Leaves;
     tree->Nodes = cellLocatorTree->Nodes;
     std::copy_n(cellLocatorTree->DataBBox, 6, tree->DataBBox);
     this->Tree = tree;
   }
+  this->BuildTime.Modified();
 }
 
 //------------------------------------------------------------------------------
@@ -1488,3 +1456,4 @@ void vtkCellTreeLocator::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "NumberOfBuckets: " << this->NumberOfBuckets << "\n";
   os << indent << "LargeIds: " << this->LargeIds << "\n";
 }
+VTK_ABI_NAMESPACE_END
