@@ -3,13 +3,12 @@
 #include "vtkPolygon.h"
 
 #include "vtkArrayDispatch.h"
+#include "vtkArrayDispatchDataSetArrayList.h"
 #include "vtkArrayRange.h"
 #include "vtkBoundingBox.h"
 #include "vtkBox.h"
 #include "vtkCellArray.h"
-#include "vtkDataSet.h"
 #include "vtkDoubleArray.h"
-#include "vtkIncrementalPointLocator.h"
 #include "vtkLine.h"
 #include "vtkLogger.h"
 #include "vtkMath.h"
@@ -23,14 +22,17 @@
 #include "vtkQuad.h"
 #include "vtkSmartPointer.h"
 #include "vtkTriangle.h"
+#include "vtkVector.h"
 
+#include <algorithm>
 #include <limits> // For DBL_MAX
 #include <vector>
 
 VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkPolygon);
 
-constexpr double VTK_POLYGON_TOL = 1.e-08; // Absolute tolerance for testing near polygon boundary
+constexpr double VTK_POLYGON_TOL = 1.e-08; // Absolute tolerance for testing near polygon boundary.
+constexpr double VTK_DEFAULT_PLANARITY_TOLERANCE = 0.1; // dZ / max(dX, dY). See ComputeCentroid.
 
 //------------------------------------------------------------------------------
 // Instantiate polygon.
@@ -134,10 +136,32 @@ struct NormalWorker
     auto points = vtk::DataArrayTupleRange<3>(p);
     using PointType = typename decltype(points)::ConstTupleReferenceType;
     double v1[3], v2[3];
-    PointType p0 = points[::GetPointId<PointIdRedirection>(pts, 0)];
-    vtkMath::Subtract(points[::GetPointId<PointIdRedirection>(pts, 1)], p0, v1);
+    vtkIdType pointId;
+    vtkIdType commonPointId = -1;
+    for (pointId = 0; pointId < (numPts - 2); ++pointId)
+    {
+      PointType p0 = points[::GetPointId<PointIdRedirection>(pts, pointId)];
+      vtkMath::Subtract(points[::GetPointId<PointIdRedirection>(pts, pointId + 1)], p0, v1);
+      if (vtkMath::SquaredNorm(v1) > 0)
+      {
+        commonPointId = pointId;
+        pointId += 2; // consume the two points we just used to obtain a non-zero v1
+        break;
+      }
+    }
 
-    for (vtkIdType pointId = 2; pointId < numPts; ++pointId)
+    if (pointId >= numPts || commonPointId < 0)
+    {
+      // Either all the points in the loop were coincident or we used
+      // all the points to obtain v1 and have nothing left for v2.
+      n[0] = 0;
+      n[1] = 0;
+      n[2] = 0;
+      return;
+    }
+
+    PointType p0 = points[::GetPointId<PointIdRedirection>(pts, commonPointId)];
+    for (; pointId < numPts; ++pointId)
     {
       vtkMath::Subtract(points[::GetPointId<PointIdRedirection>(pts, pointId)], p0, v2);
       vtkMath::Cross(v1, v2, v1);
@@ -151,7 +175,7 @@ struct NormalWorker
 template <bool PointIdRedirection>
 void ComputeNormal(vtkPoints* p, int numPts, const vtkIdType* pts, double* n)
 {
-  using Dispatcher = vtkArrayDispatch::DispatchByValueType<vtkArrayDispatch::Reals>;
+  using Dispatcher = vtkArrayDispatch::DispatchByArray<vtkArrayDispatch::PointArrays>;
   ::NormalWorker<PointIdRedirection> worker;
   if (!Dispatcher::Execute(p->GetData(), worker, numPts, pts, n))
   {
@@ -171,7 +195,7 @@ void ComputeNormal(vtkPoints* p, int numPts, const vtkIdType* pts, double* n)
 // that index into the points list. Parameter pts can be nullptr, indicating that
 // the polygon indexing is {0, 1, ..., numPts-1}. This version will handle
 // non-convex polygons.
-void vtkPolygon::ComputeNormal(vtkPoints* p, int numPts, const vtkIdType* pts, double* n)
+vtkCellStatus vtkPolygon::ComputeNormal(vtkPoints* p, int numPts, const vtkIdType* pts, double* n)
 {
   //
   // Check for special triangle case. Saves extra work.
@@ -179,7 +203,7 @@ void vtkPolygon::ComputeNormal(vtkPoints* p, int numPts, const vtkIdType* pts, d
   n[0] = n[1] = n[2] = 0.0;
   if (numPts < 3)
   {
-    return;
+    return vtkCellStatus::WrongNumberOfPoints;
   }
 
   if (pts)
@@ -191,14 +215,14 @@ void vtkPolygon::ComputeNormal(vtkPoints* p, int numPts, const vtkIdType* pts, d
     ::ComputeNormal<false>(p, numPts, pts, n);
   }
 
-  vtkMath::Normalize(n);
+  return (vtkMath::Normalize(n) == 0) ? vtkCellStatus::DegenerateFaces : vtkCellStatus::Valid;
 }
 
 //------------------------------------------------------------------------------
 // Compute the polygon normal from a points list, and a list of point ids
 // that index into the points list. This version will handle non-convex
 // polygons.
-void vtkPolygon::ComputeNormal(vtkIdTypeArray* ids, vtkPoints* p, double n[3])
+vtkCellStatus vtkPolygon::ComputeNormal(vtkIdTypeArray* ids, vtkPoints* p, double n[3])
 {
   return vtkPolygon::ComputeNormal(p, ids->GetNumberOfTuples(), ids->GetPointer(0), n);
 }
@@ -206,7 +230,7 @@ void vtkPolygon::ComputeNormal(vtkIdTypeArray* ids, vtkPoints* p, double n[3])
 //------------------------------------------------------------------------------
 // Compute the polygon normal from a list of doubleing points. This version
 // will handle non-convex polygons.
-void vtkPolygon::ComputeNormal(vtkPoints* p, double* n)
+vtkCellStatus vtkPolygon::ComputeNormal(vtkPoints* p, double* n)
 {
   return vtkPolygon::ComputeNormal(p, p->GetNumberOfPoints(), nullptr, n);
 }
@@ -214,7 +238,7 @@ void vtkPolygon::ComputeNormal(vtkPoints* p, double* n)
 //------------------------------------------------------------------------------
 // Compute the polygon normal from an array of points. This version assumes
 // that the polygon is convex, and looks for the first valid normal.
-void vtkPolygon::ComputeNormal(int numPts, double* pts, double n[3])
+vtkCellStatus vtkPolygon::ComputeNormal(int numPts, double* pts, double n[3])
 {
   int i;
   double *v1, *v2, *v3;
@@ -222,6 +246,11 @@ void vtkPolygon::ComputeNormal(int numPts, double* pts, double n[3])
   double ax, ay, az;
   double bx, by, bz;
 
+  if (numPts < 3)
+  {
+    return vtkCellStatus::WrongNumberOfPoints;
+  }
+
   //  Because some polygon vertices are colinear, need to make sure
   //  first non-zero normal is found.
   //
@@ -248,7 +277,7 @@ void vtkPolygon::ComputeNormal(int numPts, double* pts, double n[3])
       n[0] /= length;
       n[1] /= length;
       n[2] /= length;
-      return;
+      return vtkCellStatus::Valid;
     }
     else
     {
@@ -257,6 +286,7 @@ void vtkPolygon::ComputeNormal(int numPts, double* pts, double n[3])
       v3 += 3;
     }
   } // over all points
+  return vtkCellStatus::DegenerateFaces;
 }
 
 //------------------------------------------------------------------------------
@@ -498,7 +528,7 @@ void vtkPolygon::InterpolateFunctionsUsingMVC(const double x[3], double* weights
   // Begin by initializing weights.
   for (int i = 0; i < numPts; i++)
   {
-    weights[i] = static_cast<double>(0.0);
+    weights[i] = 0.0;
   }
 
   // create local array for storing point-to-vertex vectors and distances
@@ -809,15 +839,6 @@ int vtkPolygon::PointInPolygon(double x[3], int numPts, double* pts, double boun
   return ((wn == 0 ? VTK_POLYGON_OUTSIDE : VTK_POLYGON_INSIDE));
 }
 
-//------------------------------------------------------------------------------
-// Triangulate polygon.
-//
-int vtkPolygon::Triangulate(vtkIdList* outTris)
-{
-  this->SuccessfulTriangulation = 1;
-  return this->EarCutTriangulation(outTris);
-}
-
 //------------------------------------------------------------------------------
 // Split into non-degenerate polygons prior to triangulation
 //
@@ -841,30 +862,12 @@ int vtkPolygon::NonDegenerateTriangulate(vtkIdList* outTris)
   {
     this->Points->GetPoint(i, pt);
 
-    if (pt[0] < bounds[0])
-    {
-      bounds[0] = pt[0];
-    }
-    if (pt[1] < bounds[2])
-    {
-      bounds[2] = pt[1];
-    }
-    if (pt[2] < bounds[4])
-    {
-      bounds[4] = pt[2];
-    }
-    if (pt[0] > bounds[1])
-    {
-      bounds[1] = pt[0];
-    }
-    if (pt[1] > bounds[3])
-    {
-      bounds[3] = pt[1];
-    }
-    if (pt[2] > bounds[5])
-    {
-      bounds[5] = pt[2];
-    }
+    bounds[0] = std::min(pt[0], bounds[0]);
+    bounds[2] = std::min(pt[1], bounds[2]);
+    bounds[4] = std::min(pt[2], bounds[4]);
+    bounds[1] = std::max(pt[0], bounds[1]);
+    bounds[3] = std::max(pt[1], bounds[3]);
+    bounds[5] = std::max(pt[2], bounds[5]);
   }
 
   outTris->Reset();
@@ -1173,7 +1176,7 @@ void vtkPolyVertexList::RemoveVertex(
 // into the priority queue.
 void vtkPolyVertexList::RemoveVertex(int i, vtkIdList* tris, vtkPriorityQueue* queue)
 {
-  return this->RemoveVertex(this->Array + i, tris, queue);
+  this->RemoveVertex(this->Array + i, tris, queue);
 }
 
 //------------------------------------------------------------------------------
@@ -1259,6 +1262,7 @@ double vtkPolyVertexList::ComputeMeasure(vtkLocalPolyVertex* vtx)
     double l1 = vtkMath::Norm(v1);
     double l2 = vtkMath::Norm(v2);
     double l3 = vtkMath::Norm(v3);
+    // NOLINTNEXTLINE(readability-avoid-nested-conditional-operator)
     int longestEdge = (l1 > l2 ? (l1 > l3 ? 1 : 3) : (l2 > l3 ? 2 : 3));
     double shortest, longest;
     if (longestEdge == 1)
@@ -1327,6 +1331,7 @@ int vtkPolyVertexList::CanRemoveVertex(vtkLocalPolyVertex* currentVtx)
   // the split line.
   int oneNegative = 0;
   val = vtkPlane::Evaluate(sN, sPt, next->next->x);
+  // NOLINTNEXTLINE(readability-avoid-nested-conditional-operator)
   currentSign = (val > tolerance ? 1 : (val < -tolerance ? -1 : 0));
   oneNegative = (currentSign < 0 ? 1 : 0); // very important
 
@@ -1334,6 +1339,7 @@ int vtkPolyVertexList::CanRemoveVertex(vtkLocalPolyVertex* currentVtx)
   for (vtx = next->next->next; vtx != previous; vtx = vtx->next)
   {
     val = vtkPlane::Evaluate(sN, sPt, vtx->x);
+    // NOLINTNEXTLINE(readability-avoid-nested-conditional-operator)
     sign = (val > tolerance ? 1 : (val < -tolerance ? -1 : 0));
     if (sign != currentSign)
     {
@@ -1623,6 +1629,9 @@ int vtkPolygon::EarCutTriangulation(vtkIdList* outTris, int measure)
     return (this->SuccessfulTriangulation = 1);
   }
 
+  // If npts is 4 and ::SimpleTriangulation() failed, the quad is poorly shaped.
+  // Triangulate by the shorter diagonal
+
   // Establish a more convenient structure for the triangulation process
   vtkPolyVertexList poly(this->PointIds, this->Points, this->Tol * this->Tol, measure);
   vtkLocalPolyVertex* vtx;
@@ -1920,7 +1929,18 @@ int vtkPolygon::TriangulateLocalIds(int vtkNotUsed(index), vtkIdList* ptIds)
   int success = this->EarCutTriangulation(ptIds);
   if (!success) // Indicate possible failure
   {
-    vtkDebugMacro(<< "Possible triangulation failure");
+    // Non-planar quads can cause the ear cut triangulation algorithm to fail.
+    // If that happens, use vtkQuad specifically to triangulate the quad.
+    if (this->PointIds->GetNumberOfIds() == 4)
+    {
+      vtkNew<vtkQuad> quad;
+      quad->DeepCopy(this);
+      quad->TriangulateLocalIds(0, ptIds);
+    }
+    else
+    {
+      vtkDebugMacro(<< "Possible triangulation failure");
+    }
   }
   return this->SuccessfulTriangulation;
 }
@@ -2166,6 +2186,7 @@ double vtkPolygon::ComputeArea(vtkPoints* p, vtkIdType numPts, const vtkIdType*
     ny = (n[1] > 0.0 ? n[1] : -n[1]); // abs y-coord
     nz = (n[2] > 0.0 ? n[2] : -n[2]); // abs z-coord
 
+    // NOLINTNEXTLINE(readability-avoid-nested-conditional-operator)
     coord = (nx > ny ? (nx > nz ? 0 : 2) : (ny > nz ? 1 : 2));
 
     // compute area of the 2D projection
@@ -2242,71 +2263,86 @@ void vtkPolygon::PrintSelf(ostream& os, vtkIndent indent)
 // Compute the polygon centroid from a points list, the number of points, and an
 // array of point ids that index into the points list. Returns false if the
 // computation is invalid.
-bool vtkPolygon::ComputeCentroid(vtkPoints* p, int numPts, const vtkIdType* ids, double c[3])
+vtkCellStatus vtkPolygon::ComputeCentroid(
+  vtkPoints* p, int numPts, const vtkIdType* ids, double c[3], double tolerance)
 {
-  // Strategy:
-  // - Compute centroid of projected polygon on (x,y) if polygon is projectible, (x,z) otherwise
-  // - Accumulate signed projected area as well, which is needed in the centroid's formula
-  // - Infer 3rd dimension using polygon's normal.
-  vtkIdType i;
-  double p0[6];
-  double normal[3] = { 0.0 };
   if (numPts < 2)
   {
-    return false;
+    return vtkCellStatus::WrongNumberOfPoints;
   }
-  // Handle to the doubled area of the projected polygon on (x,y) or (x,z) if the polygon
-  // projected on (x,y) is degenerate.
-  double a = 0.0;
-  p->GetPoint(ids[0], p0);
-
-  vtkPolygon::ComputeNormal(p, numPts, ids, normal);
-  vtkIdType xOffset = 0, yOffset = 0;
-  // Checking if the polygon is colinear with z axis.
-  // If it is, the centroid computation axis shall be shifted
-  // because the projected polygon on (x,y) is degenerate.
 
+  vtkVector3d normal;
+  auto status = vtkPolygon::ComputeNormal(p, numPts, ids, normal.GetData());
+  if (!status)
   {
-    constexpr double z[3] = { 0.0, 0.0, 1.0 };
-    vtkMath::Cross(normal, z, c);
-    // If the normal is orthogonal with z axis, the projected polygon is then a line...
-    if (std::fabs(c[0] * c[0] + c[1] * c[1] + c[2] * c[2] - 1.0) <= VTK_DBL_EPSILON)
-    {
-      yOffset = 1;
-      constexpr double y[3] = { 0.0, 1.0, 0.0 };
-      vtkMath::Cross(normal, y, c);
-      // If the normal is orthogonal with y axis, the projected polygon is then a line...
-      if (std::fabs(c[0] * c[0] + c[1] * c[1] + c[2] * c[2] - 1.0) <= VTK_DBL_EPSILON)
-      {
-        xOffset = 1;
-      }
-    }
+    return status;
   }
 
-  c[0] = c[1] = c[2] = 0.0;
+  // Set xx to be the average coordinate. This is not necessarily the centroid
+  // but will generally produce accurate triangle areas used to compute the centroid.
+  vtkVector3d xx(0, 0, 0);
+  vtkVector3d pp;
+  vtkVector3d qq;
+  double wt = 1. / numPts;
+  for (int ii = 0; ii < numPts; ++ii)
+  {
+    p->GetPoint(ids[ii], pp.GetData());
+    xx += wt * pp;
+  }
+  // Note that pp now contains the final point in the polygon.
+  // If we start again with the first point, we can track pairs
+  // of points along edges.
 
-  double maxabsdet = 0;
-  for (i = 0; i < numPts; i++)
+  // Now compute the centroid of each triangle formed by xx and
+  // the endpoints of one edge in the polygon. Weight the
+  // centroid by the triangle's signed area (negative if the polygon
+  // winds clockwise) and sum them together.
+  //
+  // This is equivalent to computing (Integral(x_i dA) / Integral(dA))
+  // for each coordinate (x_0, x_1, x_2) using the "geometric decomposition"
+  // method.
+  double totalArea = 0.;
+  double area;
+  vtkVector3d accum(0, 0, 0);
+  vtkVector3d ctr;
+  double outOfPlane = 0;
+  double inPlane2 = 0;
+  for (int ii = 0; ii < numPts; ++ii, pp = qq)
   {
-    p->GetPoint(ids[(i + 1) % numPts], p0 + 3 * !(i % 2));
-    double det = (p0[3 * (i % 2) + xOffset] * p0[3 * !(i % 2) + 1 + yOffset] -
-      p0[3 * !(i % 2) + xOffset] * p0[3 * (i % 2) + 1 + yOffset]);
-    c[xOffset] += (p0[xOffset] + p0[3 + xOffset]) * det;
-    c[1 + yOffset] += (p0[1 + yOffset] + p0[4 + yOffset]) * det;
-    a += det;
-    maxabsdet = std::abs(det) > maxabsdet ? std::abs(det) : maxabsdet;
+    p->GetPoint(ids[ii], qq.GetData());
+    // The centroid of xx-pp-qq is 2/3 of the way from xx to the midpoint of qq-pp
+    auto pq = (pp + qq) * 0.5;
+    ctr = (1. / 3. * xx) + (2. / 3. * pq);
+    auto dqx = qq - xx;
+    area = ((pp - xx).Cross(dqx)).Dot(normal) / 2;
+    accum += area * ctr;
+    totalArea += area;
+    // Compute the in-plane and out-of-plane distance from xx to qq.
+    // Note that because xx is the average coordinate, oop and
+    // ip2 will both be half-distances; their ratio will be correct
+    // for comparison to tolerance.
+    double oop = std::abs(dqx.Dot(normal)); // out-of-plane distance
+    outOfPlane = std::max(oop, outOfPlane);
+    double ip2 = (dqx - oop * normal).SquaredNorm();
+    inPlane2 = std::max(ip2, inPlane2);
   }
-  if (std::abs(a) < VTK_DBL_EPSILON * maxabsdet)
+  // Fail if the polygon is too far from planarity and the tolerance is "active":
+  if (tolerance > 0. && outOfPlane / std::sqrt(inPlane2) > tolerance)
   {
-    // Polygon is degenerate
-    return false;
+    return vtkCellStatus::NonPlanarFaces;
   }
-  c[xOffset] /= 3.0 * a;
-  c[1 + yOffset] /= 3.0 * a;
-  c[2 - xOffset - yOffset] = 1.0 / normal[2 - xOffset - yOffset] *
-    (-normal[xOffset] * c[xOffset] - normal[1 + yOffset] * c[1 + yOffset] +
-      vtkMath::Dot(normal, p0));
-  return true;
+  // Divide the accumulated product of weighted centroids by the total area
+  // of all the triangles. This produces the final centroid.
+  accum = (1. / totalArea) * accum;
+  c[0] = accum[0];
+  c[1] = accum[1];
+  c[2] = accum[2];
+  return vtkCellStatus::Valid;
+}
+
+bool vtkPolygon::ComputeCentroid(vtkPoints* p, int numPts, const vtkIdType* pts, double centroid[3])
+{
+  return !!vtkPolygon::ComputeCentroid(p, numPts, pts, centroid, VTK_DEFAULT_PLANARITY_TOLERANCE);
 }
 
 //------------------------------------------------------------------------------
@@ -2314,7 +2350,8 @@ bool vtkPolygon::ComputeCentroid(vtkPoints* p, int numPts, const vtkIdType* ids,
 // that index into the points list. Returns false if the computation is invalid.
 bool vtkPolygon::ComputeCentroid(vtkIdTypeArray* ids, vtkPoints* p, double c[3])
 {
-  return vtkPolygon::ComputeCentroid(p, ids->GetNumberOfTuples(), ids->GetPointer(0), c);
+  return !!vtkPolygon::ComputeCentroid(
+    p, ids->GetNumberOfTuples(), ids->GetPointer(0), c, VTK_DEFAULT_PLANARITY_TOLERANCE);
 }
 
 //------------------------------------------------------------------------------
