@@ -1,17 +1,5 @@
-/*=========================================================================
-
-Program:   Visualization Toolkit
-Module:    vtkCompositeDataPipeline.cxx
-
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkCompositeDataPipeline.h"
 
 #include "vtkAlgorithm.h"
@@ -41,6 +29,7 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkTrivialProducer.h"
 #include "vtkUniformGrid.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkCompositeDataPipeline);
 
 vtkInformationKeyMacro(vtkCompositeDataPipeline, LOAD_REQUESTED_BLOCKS, Integer);
@@ -219,7 +208,8 @@ bool vtkCompositeDataPipeline::ShouldIterateOverInput(
 
         if (strcmp(inputType, "vtkCompositeDataSet") == 0 ||
           strcmp(inputType, "vtkDataObjectTree") == 0 ||
-          strcmp(inputType, "vtkHierarchicalBoxDataSet") == 0 ||
+          strcmp(inputType, "vtkHierarchicalBoxDataSet") == 0 || // VTK_DEPRECATED_IN_9_5_0
+          strcmp(inputType, "vtkUniformGridAMR") == 0 ||
           strcmp(inputType, "vtkOverlappingAMR") == 0 ||
           strcmp(inputType, "vtkNonOverlappingAMR") == 0 ||
           strcmp(inputType, "vtkMultiBlockDataSet") == 0 ||
@@ -284,6 +274,10 @@ void vtkCompositeDataPipeline::ExecuteEach(vtkCompositeDataIterator* iter,
   auto algo = this->GetAlgorithm();
   for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem(), ++block_index)
   {
+    if (algo->GetAbortOutput())
+    {
+      break;
+    }
     vtkDataObject* dobj = iter->GetCurrentDataObject();
     if (dobj)
     {
@@ -914,7 +908,7 @@ int vtkCompositeDataPipeline::CheckCompositeData(
     // If it is not already, then we need to create a composite data object for the outputs
     // on that port to be placed into.  If the output is already a composite data object, it
     // is assumed that the composite data pipeline is being re-run and the data object from
-    // the last run can be re-used.
+    // the last run can be reused.
     bool needsToCreateCompositeOutput = false;
     for (int port = 0; port < outInfoVec->GetNumberOfInformationObjects(); ++port)
     {
@@ -1011,6 +1005,7 @@ std::vector<vtkSmartPointer<vtkDataObject>> vtkCompositeDataPipeline::CreateOutp
   //      whether to create vtkHierarchicalBoxDataSet or vtkMultiBlockDataSet.
   std::vector<vtkSmartPointer<vtkDataObject>> outputVector;
 
+  // VTK_DEPRECATED_IN_9_5_0, remove vtkHierarchicalBoxDataSet on removal
   if (input->IsA("vtkHierarchicalBoxDataSet") || input->IsA("vtkOverlappingAMR") ||
     input->IsA("vtkNonOverlappingAMR"))
   {
@@ -1018,8 +1013,19 @@ std::vector<vtkSmartPointer<vtkDataObject>> vtkCompositeDataPipeline::CreateOutp
 
     // Check if the algorithm can accept UniformGrid on the input port.
     vtkInformation* inPortInfo = this->Algorithm->GetInputPortInformation(compositePort);
-    const char* inputType = inPortInfo->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE());
-    if (!tempInput->IsA(inputType))
+
+    int nbKeys = inPortInfo->GetNumberOfKeys();
+    bool acceptsUniformGrid = false;
+    for (int k = 0; k < nbKeys; ++k)
+    {
+      const char* inputType = inPortInfo->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), k);
+      if (inputType && tempInput->IsA(inputType))
+      {
+        acceptsUniformGrid = true;
+        break;
+      }
+    }
+    if (!acceptsUniformGrid)
     {
       for (int i = 0; i < numOutputPorts; ++i)
       {
@@ -1122,3 +1128,4 @@ void vtkCompositeDataPipeline::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
 }
+VTK_ABI_NAMESPACE_END
