@@ -1,20 +1,9 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkCookieCutter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkCookieCutter.h"
 
 #include "vtkCellData.h"
+#include "vtkDoubleArray.h"
 #include "vtkExecutive.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
@@ -29,6 +18,7 @@
 #include <set>
 #include <vector>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkCookieCutter);
 vtkCxxSetObjectMacro(vtkCookieCutter, Locator, vtkIncrementalPointLocator);
 
@@ -282,7 +272,7 @@ int CleanSortedPolyline(SortedPointsType& sortedPoints)
     t = sortedPoints[i].T;
     if (end == (num - 1))
     { // last point may require special treatment
-      mergeRange.emplace_back(MergeRange(end, end + 1));
+      mergeRange.emplace_back(end, end + 1);
       break;
     }
     ip = i + 1;
@@ -297,7 +287,7 @@ int CleanSortedPolyline(SortedPointsType& sortedPoints)
     end = ip;
 
     // Add new segment
-    mergeRange.emplace_back(MergeRange(start, end));
+    mergeRange.emplace_back(start, end);
 
     // Move to next segment
     i = start = end;
@@ -346,7 +336,7 @@ int CleanSortedPolyline(SortedPointsType& sortedPoints)
     {
       spc = SortPoint::MULT_INTS;
     }
-    newSortedPoints.emplace_back(SortPoint(minT, spc, minId, onId, sortedPoints[minX].X));
+    newSortedPoints.emplace_back(minT, spc, minId, onId, sortedPoints[minX].X);
   } // across all merge ranges
 
   // Update the sorted points array, now clean!
@@ -407,7 +397,7 @@ void CleanSortedPolygon(vtkIdType npts, SortedPointsType& sortedPoints)
   {
     if (i == (imax - 1))
     {
-      mergeRange.emplace_back(MergeRange(i, imax));
+      mergeRange.emplace_back(i, imax);
       break;
     }
 
@@ -438,7 +428,7 @@ void CleanSortedPolygon(vtkIdType npts, SortedPointsType& sortedPoints)
     end = ip;
 
     // Add new segment
-    mergeRange.emplace_back(MergeRange(start, end));
+    mergeRange.emplace_back(start, end);
 
     // Move to next segment
     i = start = end;
@@ -489,7 +479,7 @@ void CleanSortedPolygon(vtkIdType npts, SortedPointsType& sortedPoints)
     {
       spc = SortPoint::MULT_INTS;
     }
-    newSortedPoints.emplace_back(SortPoint(minT, spc, minId, onId, sortedPoints[minX].X));
+    newSortedPoints.emplace_back(minT, spc, minId, onId, sortedPoints[minX].X);
   } // across all merge ranges
 
   // Update the sorted points array, now clean!
@@ -537,7 +527,6 @@ int ClassifyPolyline(
 
     if (currClass == SortPoint::VERTEX)
     { // propagate forward
-      ;
     }
     else if (currClass == SortPoint::INTERSECTION)
     { // Flip classification
@@ -606,7 +595,6 @@ int ClassifyPolygon(
 
     if (currClass == SortPoint::VERTEX)
     { // Propagate the classification forward
-      ;
     }
     else if (currClass == SortPoint::INTERSECTION)
     { // Flip the classification
@@ -797,18 +785,18 @@ struct vtkAttributeManager
     // See if it's a vertex (CopyData)
     if (attType < PointAttribute::Intersection)
     {
-      this->PointAttributes.emplace_back(PointAttribute(attType, m0, m1, tm));
+      this->PointAttributes.emplace_back(attType, m0, m1, tm);
       return;
     }
 
     // See if it's an edge (InterpolateData)
     if (this->PointInterpolation == vtkCookieCutter::USE_MESH_EDGES)
     {
-      this->PointAttributes.emplace_back(PointAttribute(PointAttribute::MeshEdge, m0, m1, tm));
+      this->PointAttributes.emplace_back(PointAttribute::MeshEdge, m0, m1, tm);
     }
     else // if ( this->PointInterpolation == vtkCookieCutter::USE_LOOP_EDGES )
     {
-      this->PointAttributes.emplace_back(PointAttribute(PointAttribute::LoopEdge, l0, l1, tl));
+      this->PointAttributes.emplace_back(PointAttribute::LoopEdge, l0, l1, tl);
     }
   }
 
@@ -941,7 +929,7 @@ void vtkCookieCutterHelper::CropLine(vtkIdType cellId, vtkIdType cellOffset, vtk
   {
     t = static_cast<double>(i);
     this->InPoints->GetPoint(pts[i], x);
-    sortedPoints.emplace_back(SortPoint(t, SortPoint::VERTEX, -1, -1, x));
+    sortedPoints.emplace_back(t, SortPoint::VERTEX, -1, -1, x);
     attrMgr->AddPointAttribute(PointAttribute::MeshVertex, pts[i]);
   }
 
@@ -969,7 +957,7 @@ void vtkCookieCutterHelper::CropLine(vtkIdType cellId, vtkIdType cellOffset, vtk
         attrMgr->AddPointAttribute(PointAttribute::Intersection, m0, m1, l0, l1, u, v);
         u += static_cast<double>(i);
         v += static_cast<double>(j);
-        sortedPoints.emplace_back(SortPoint(u, SortPoint::INTERSECTION, numInts, -1, x));
+        sortedPoints.emplace_back(u, SortPoint::INTERSECTION, numInts, -1, x);
         numInts++;
       }
       else if (result == 3) // parallel lines
@@ -987,8 +975,7 @@ void vtkCookieCutterHelper::CropLine(vtkIdType cellId, vtkIdType cellOffset, vtk
           vtkLine::DistanceToLine(x0, y0, y1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            sortedPoints.emplace_back(
-              SortPoint(static_cast<double>(i), SortPoint::ON, numInts, onId, c));
+            sortedPoints.emplace_back(static_cast<double>(i), SortPoint::ON, numInts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::MeshVertex, m0);
             numInts++;
           }
@@ -996,23 +983,21 @@ void vtkCookieCutterHelper::CropLine(vtkIdType cellId, vtkIdType cellOffset, vtk
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
             sortedPoints.emplace_back(
-              SortPoint(static_cast<double>(i) + 1.0, SortPoint::ON, numInts, onId, c));
+              static_cast<double>(i) + 1.0, SortPoint::ON, numInts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::MeshVertex, m1);
             numInts++;
           }
           vtkLine::DistanceToLine(y0, x0, x1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            sortedPoints.emplace_back(
-              SortPoint(static_cast<double>(i) + u, SortPoint::ON, numInts, onId, c));
+            sortedPoints.emplace_back(static_cast<double>(i) + u, SortPoint::ON, numInts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::LoopVertex, l0);
             numInts++;
           }
           vtkLine::DistanceToLine(y1, x0, x1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            sortedPoints.emplace_back(
-              SortPoint(static_cast<double>(i) + u, SortPoint::ON, numInts, onId, c));
+            sortedPoints.emplace_back(static_cast<double>(i) + u, SortPoint::ON, numInts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::LoopVertex, l1);
             numInts++;
           }
@@ -1085,7 +1070,7 @@ int vtkCookieCutterHelper::ResolveTopology(vtkPolyData* pd)
     pd->GetPointCells(pid, ncells, cells);
     if (ncells == 0 || ncells == 2)
     {
-      ; // skip is ok
+      // skip is ok
     }
     else if (ncells == 1)
     {
@@ -1163,8 +1148,8 @@ void vtkCookieCutterHelper::CropPoly(vtkIdType cellId, vtkIdType cellOffset, vtk
   // (eventually sorted) arrays. These sorted arrays will alternate inside
   // paths with outside paths, sort of like a "braid". To construct polygon
   // loops, the braid is woven together to form the loops.
-  double* p = static_cast<double*>(poly->Points->GetVoidPointer(0));
-  vtkIdType i, j, newCellId, numPts = 0, numInts = 0;
+  double* p = vtkDoubleArray::FastDownCast(poly->Points->GetData())->GetPointer(0);
+  vtkIdType i, j, newCellId, numPts = 0;
   double t, u, v, x[3], x0[3], x1[3], y0[3], y1[3];
   int result;
   SortedPointsType loopPoints, polyPoints;
@@ -1179,7 +1164,7 @@ void vtkCookieCutterHelper::CropPoly(vtkIdType cellId, vtkIdType cellOffset, vtk
   {
     t = static_cast<double>(i);
     this->InPoints->GetPoint(pts[i], x);
-    polyPoints.emplace_back(SortPoint(t, SortPoint::VERTEX, numPts, -1, x));
+    polyPoints.emplace_back(t, SortPoint::VERTEX, numPts, -1, x);
     attrMgr->AddPointAttribute(PointAttribute::MeshVertex, pts[i]);
     numPts++;
   }
@@ -1189,7 +1174,7 @@ void vtkCookieCutterHelper::CropPoly(vtkIdType cellId, vtkIdType cellOffset, vtk
   {
     t = static_cast<double>(i);
     loop->Points->GetPoint(i, x);
-    loopPoints.emplace_back(SortPoint(t, SortPoint::VERTEX, numPts, -1, x));
+    loopPoints.emplace_back(t, SortPoint::VERTEX, numPts, -1, x);
     attrMgr->AddPointAttribute(PointAttribute::LoopVertex, i);
     numPts++;
   }
@@ -1220,9 +1205,8 @@ void vtkCookieCutterHelper::CropPoly(vtkIdType cellId, vtkIdType cellOffset, vtk
         attrMgr->AddPointAttribute(PointAttribute::Intersection, m0, m1, l0, l1, u, v);
         u += static_cast<double>(i);
         v += static_cast<double>(j);
-        polyPoints.emplace_back(SortPoint(u, SortPoint::INTERSECTION, numPts, -1, x));
-        loopPoints.emplace_back(SortPoint(v, SortPoint::INTERSECTION, numPts, -1, x));
-        numInts++;
+        polyPoints.emplace_back(u, SortPoint::INTERSECTION, numPts, -1, x);
+        loopPoints.emplace_back(v, SortPoint::INTERSECTION, numPts, -1, x);
         numPts++;
       }
       else if (result == 3) // parallel lines
@@ -1240,45 +1224,33 @@ void vtkCookieCutterHelper::CropPoly(vtkIdType cellId, vtkIdType cellOffset, vtk
           vtkLine::DistanceToLine(x0, y0, y1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            polyPoints.emplace_back(
-              SortPoint(static_cast<double>(i), SortPoint::ON, numPts, onId, c));
-            loopPoints.emplace_back(
-              SortPoint(static_cast<double>(j) + u, SortPoint::ON, numPts, onId, c));
+            polyPoints.emplace_back(static_cast<double>(i), SortPoint::ON, numPts, onId, c);
+            loopPoints.emplace_back(static_cast<double>(j) + u, SortPoint::ON, numPts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::MeshVertex, m0);
-            numInts++;
             numPts++;
           }
           vtkLine::DistanceToLine(x1, y0, y1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            polyPoints.emplace_back(
-              SortPoint(static_cast<double>(i) + 1.0, SortPoint::ON, numPts, onId, c));
-            loopPoints.emplace_back(
-              SortPoint(static_cast<double>(j) + u, SortPoint::ON, numPts, onId, c));
+            polyPoints.emplace_back(static_cast<double>(i) + 1.0, SortPoint::ON, numPts, onId, c);
+            loopPoints.emplace_back(static_cast<double>(j) + u, SortPoint::ON, numPts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::MeshVertex, m1);
-            numInts++;
             numPts++;
           }
           vtkLine::DistanceToLine(y0, x0, x1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            polyPoints.emplace_back(
-              SortPoint(static_cast<double>(i) + u, SortPoint::ON, numPts, onId, c));
-            loopPoints.emplace_back(
-              SortPoint(static_cast<double>(j), SortPoint::ON, numPts, onId, c));
+            polyPoints.emplace_back(static_cast<double>(i) + u, SortPoint::ON, numPts, onId, c);
+            loopPoints.emplace_back(static_cast<double>(j), SortPoint::ON, numPts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::LoopVertex, l0);
-            numInts++;
             numPts++;
           }
           vtkLine::DistanceToLine(y1, x0, x1, u, c);
           if (-VTK_DEGENERATE_TOL <= u && u <= (1.0 + VTK_DEGENERATE_TOL))
           {
-            polyPoints.emplace_back(
-              SortPoint(static_cast<double>(i) + u, SortPoint::ON, numPts, onId, c));
-            loopPoints.emplace_back(
-              SortPoint(static_cast<double>(j) + 1.0, SortPoint::ON, numPts, onId, c));
+            polyPoints.emplace_back(static_cast<double>(i) + u, SortPoint::ON, numPts, onId, c);
+            loopPoints.emplace_back(static_cast<double>(j) + 1.0, SortPoint::ON, numPts, onId, c);
             attrMgr->AddPointAttribute(PointAttribute::LoopVertex, l1);
-            numInts++;
             numPts++;
           }
         }
@@ -1312,6 +1284,7 @@ void vtkCookieCutterHelper::CropPoly(vtkIdType cellId, vtkIdType cellOffset, vtk
   vtkNew<vtkPolyData> pData;
   pData->SetPoints(pDataPts);
   pData->SetLines(pDataLines);
+  pData->EditableOn();
 
   SortedPointsType* loops[2];
   loops[0] = &polyPoints;
@@ -1533,6 +1506,10 @@ int vtkCookieCutter::RequestData(vtkInformation* vtkNotUsed(request),
   const vtkIdType* pts;
   for (loopPolys->InitTraversal(); loopPolys->GetNextCell(npts, pts);)
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     vtkIdType cellId = 0;
     vtkIdType numLoopPts = npts;
     loop->Initialize(npts, pts, loopPts);
@@ -1540,7 +1517,7 @@ int vtkCookieCutter::RequestData(vtkInformation* vtkNotUsed(request),
     bbox.Inflate(tol); // Handle exactly aligned x-y plane numerical fuzz
     bbox.GetBounds(bds);
     vtkPolygon::ComputeNormal(loop->Points, n);
-    double* l = static_cast<double*>(loop->Points->GetVoidPointer(0));
+    double* l = vtkDoubleArray::FastDownCast(loop->Points->GetData())->GetPointer(0);
 
     // Start by processing the verts. A simple in/out check.
     if (inVerts->GetNumberOfCells() > 0)
@@ -1683,3 +1660,4 @@ void vtkCookieCutter::PrintSelf(ostream& os, vtkIndent indent)
     os << indent << "Locator: (none)\n";
   }
 }
+VTK_ABI_NAMESPACE_END
