@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPResampleFilter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkPResampleFilter.h"
 
 #include "vtkCellData.h"
@@ -30,11 +18,14 @@
 #include "vtkPointData.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 
+#include <iostream>
+
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkPResampleFilter);
 
 vtkCxxSetObjectMacro(vtkPResampleFilter, Controller, vtkMultiProcessController);
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkPResampleFilter::vtkPResampleFilter()
   : UseInputBounds(0)
 {
@@ -48,13 +39,13 @@ vtkPResampleFilter::vtkPResampleFilter()
   vtkMath::UninitializeBounds(this->Bounds);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkPResampleFilter::~vtkPResampleFilter()
 {
   this->SetController(nullptr);
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 double* vtkPResampleFilter::CalculateBounds(vtkDataSet* input)
 {
   double localBounds[6];
@@ -97,12 +88,12 @@ double* vtkPResampleFilter::CalculateBounds(vtkDataSet* input)
     }
   }
 
-  cout << "Bounds: " << localBounds[0] << " " << localBounds[1] << " " << localBounds[2] << " "
-       << localBounds[3] << " " << localBounds[4] << " " << localBounds[5] << " " << endl;
+  std::cout << "Bounds: " << localBounds[0] << " " << localBounds[1] << " " << localBounds[2] << " "
+            << localBounds[3] << " " << localBounds[4] << " " << localBounds[5] << " " << std::endl;
   return this->Bounds;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkPResampleFilter::RequestInformation(
   vtkInformation*, vtkInformationVector**, vtkInformationVector* outputVector)
 {
@@ -115,7 +106,7 @@ int vtkPResampleFilter::RequestInformation(
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkPResampleFilter::RequestUpdateExtent(
   vtkInformation*, vtkInformationVector** inputVector, vtkInformationVector*)
 {
@@ -129,7 +120,7 @@ int vtkPResampleFilter::RequestUpdateExtent(
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkPResampleFilter::RequestData(vtkInformation* vtkNotUsed(request),
   vtkInformationVector** inputVector, vtkInformationVector* outputVector)
 {
@@ -163,7 +154,7 @@ int vtkPResampleFilter::RequestData(vtkInformation* vtkNotUsed(request),
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 int vtkPResampleFilter::FillInputPortInformation(int port, vtkInformation* info)
 {
   if (!this->Superclass::FillInputPortInformation(port, info))
@@ -174,19 +165,20 @@ int vtkPResampleFilter::FillInputPortInformation(int port, vtkInformation* info)
   return 1;
 }
 
-//----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPResampleFilter::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
-  os << indent << "Controller " << this->Controller << endl;
-  os << indent << "UseInputBounds " << this->UseInputBounds << endl;
+  os << indent << "Controller " << this->Controller << std::endl;
+  os << indent << "UseInputBounds " << this->UseInputBounds << std::endl;
   if (this->UseInputBounds == 0)
   {
     os << indent << "CustomSamplingBounds [" << this->CustomSamplingBounds[0] << ", "
        << this->CustomSamplingBounds[1] << ", " << this->CustomSamplingBounds[2] << ", "
        << this->CustomSamplingBounds[3] << ", " << this->CustomSamplingBounds[4] << ", "
-       << this->CustomSamplingBounds[5] << "]" << endl;
+       << this->CustomSamplingBounds[5] << "]" << std::endl;
   }
   os << indent << "SamplingDimension " << this->SamplingDimension[0] << " x "
-     << this->SamplingDimension[1] << " x " << this->SamplingDimension[2] << endl;
+     << this->SamplingDimension[1] << " x " << this->SamplingDimension[2] << std::endl;
 }
+VTK_ABI_NAMESPACE_END
