@@ -812,12 +812,19 @@ struct vtkProbeLineFilter::vtkInternals
           continue;
         }
 
-        vtkNew<vtkStaticCellLocator> locator;
-        locator->SetDataSet(ds);
-        locator->SetTolerance(tolerance);
-        locator->BuildLocator();
         vtkCellLocatorStrategy* strategy = vtkCellLocatorStrategy::New();
-        strategy->SetCellLocator(locator);
+        if (auto ps = vtkPointSet::SafeDownCast(ds))
+        {
+          strategy->Initialize(ps);
+        }
+        else
+        {
+          vtkNew<vtkStaticCellLocator> locator;
+          locator->SetDataSet(ds);
+          locator->SetTolerance(tolerance);
+          locator->BuildLocator();
+          strategy->SetCellLocator(locator);
+        }
 
         this->Strategies[ds] =
           vtkSmartPointer<vtkFindCellStrategy>::Take(static_cast<vtkFindCellStrategy*>(strategy));
@@ -1263,9 +1270,16 @@ vtkSmartPointer<vtkPolyData> vtkProbeLineFilter::IntersectCells(
     int counter = 0;
     for (const auto& inter : intersectionMap)
     {
-      intersections[counter] = inter.second;
-      counter++;
+      // Append only intersections different from the previous one,
+      // and that are not a single point (in & out of the cell at the same position)
+      if (counter == 0 ||
+        (inter.second != intersections[counter - 1] && inter.second.InT != inter.second.OutT))
+      {
+        intersections[counter] = inter.second;
+        counter++;
+      }
     }
+    intersections.resize(counter);
   }
 
   // Make sure our intersections are sorted
