@@ -76,7 +76,7 @@ vtkBoundingBox GetBounds(vtkDataObject* dobj, diy::mpi::communicator& comm)
 }
 
 /**
- * Clip the dataset by the provided plane using vtkmClip.
+ * Clip the dataset by the provided plane using viskoresClip.
  */
 vtkSmartPointer<vtkUnstructuredGrid> ClipPlane(vtkDataSet* dataset, vtkSmartPointer<vtkPlane> plane)
 {
@@ -215,6 +215,27 @@ void vtkRedistributeDataSetFilter::SetExplicitCuts(const std::vector<vtkBounding
   this->Modified();
 }
 
+//------------------------------------------------------------------------------
+void vtkRedistributeDataSetFilter::SetBoundaryMode(int mode)
+{
+  if (mode < this->GetBoundaryModeMinValue() || mode > this->GetBoundaryModeMaxValue())
+  {
+    // silently ignore invalid values
+    return;
+  }
+
+  this->BoundaryMode = mode;
+  if (!::CheckNativeStrategy(this->Strategy))
+  {
+    return;
+  }
+  auto native = vtkNativePartitioningStrategy::SafeDownCast(this->Strategy);
+  bool isAssignToOneRegion =
+    (this->BoundaryMode == vtkRedistributeDataSetFilter::ASSIGN_TO_ONE_REGION);
+  native->SetAssignBoundaryCellsToSmallestRegionId(isAssignToOneRegion);
+  this->Modified();
+}
+
 //------------------------------------------------------------------------------
 void vtkRedistributeDataSetFilter::RemoveAllExplicitCuts()
 {
@@ -515,7 +536,7 @@ int vtkRedistributeDataSetFilter::RequestData(
   std::vector<vtkDataSet*> resultVector = vtkCompositeDataSet::GetDataSets(result);
   for (vtkDataSet* ds : resultVector)
   {
-    // We cannot keep duplicate ghost points as the partitionning changed, invalidating previous
+    // We cannot keep duplicate ghost points as the partitioning changed, invalidating previous
     // duplicate ghost tagging
     if (vtkUnsignedCharArray* ghostArray = ds->GetPointData()->GetGhostArray())
     {
@@ -528,7 +549,7 @@ int vtkRedistributeDataSetFilter::RequestData(
             ghosts[id] &= ~vtkDataSetAttributes::DUPLICATEPOINT;
           }
         });
-    };
+    }
   }
 
   // ******************************************************
