@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPStreamTracer.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkPStreamTracer.h"
 
 #include "vtkAMRInterpolatedVelocityField.h"
@@ -31,7 +19,6 @@
 #include "vtkMPIController.h"
 #include "vtkMath.h"
 #include "vtkMultiProcessController.h"
-#include "vtkMultiProcessStream.h"
 #include "vtkNew.h"
 #include "vtkObjectFactory.h"
 #include "vtkOverlappingAMR.h"
@@ -51,6 +38,8 @@
 #include <numeric>
 #include <vector>
 
+#include <iostream>
+
 #ifndef NDEBUG
 // #define DEBUGTRACE
 // #define LOGTRACE
@@ -60,20 +49,20 @@
 #ifdef DEBUGTRACE
 #define PRINT(x)                                                                                   \
   {                                                                                                \
-    cout << this->Rank << ")" << x << endl;                                                        \
+    std::cout << this->Rank << ")" << x << endl;                                                   \
   }
 #define ALLPRINT(x)                                                                                \
   for (int rank = 0; rank < NumProcs; rank++)                                                      \
   {                                                                                                \
     Controller->Barrier();                                                                         \
     if (rank == Rank)                                                                              \
-      cout << "(" << this->Rank << ")" << x << endl;                                               \
+      std::cout << "(" << this->Rank << ")" << x << endl;                                          \
   }
 #define Assert(a, msg)                                                                             \
   {                                                                                                \
     if (!a)                                                                                        \
     {                                                                                              \
-      cerr << msg << endl;                                                                         \
+      std::cerr << msg << endl;                                                                    \
       assert(false);                                                                               \
     }                                                                                              \
   }
@@ -82,7 +71,7 @@
   {                                                                                                \
     if (a != b)                                                                                    \
     {                                                                                              \
-      cerr << a << " != " << b << endl;                                                            \
+      std::cerr << a << " != " << b << endl;                                                       \
       assert(false);                                                                               \
     }                                                                                              \
   }
@@ -91,7 +80,7 @@
   {                                                                                                \
     if (a == b)                                                                                    \
     {                                                                                              \
-      cerr << a << " == " << b << endl;                                                            \
+      std::cerr << a << " == " << b << endl;                                                       \
       assert(false);                                                                               \
     }                                                                                              \
   }
@@ -100,7 +89,7 @@
   {                                                                                                \
     if (a < b)                                                                                     \
     {                                                                                              \
-      cerr << a << " < " << b << endl;                                                             \
+      std::cerr << a << " < " << b << endl;                                                        \
       assert(false);                                                                               \
     }                                                                                              \
   }
@@ -109,11 +98,11 @@
   {                                                                                                \
     if (a <= b)                                                                                    \
     {                                                                                              \
-      cerr << a << " < " << b << endl;                                                             \
+      std::cerr << a << " < " << b << endl;                                                        \
       assert(false);                                                                               \
     }                                                                                              \
   }
-//#define PRINT(id, x)
+// #define PRINT(id, x)
 #else
 #define PRINT(x)
 #define ALLPRINT(x)
@@ -125,6 +114,7 @@
 #define AssertNe(a, b)
 #endif
 
+VTK_ABI_NAMESPACE_BEGIN
 namespace
 {
 inline int CNext(int i, int n)
@@ -201,17 +191,11 @@ inline void UpdateBB(double* a, const double* b)
 {
   for (int i = 0; i <= 4; i += 2)
   {
-    if (b[i] < a[i])
-    {
-      a[i] = b[i];
-    }
+    a[i] = std::min(a[i], b[i]);
   }
   for (int i = 1; i <= 5; i += 2)
   {
-    if (b[i] > a[i])
-    {
-      a[i] = b[i];
-    }
+    a[i] = std::max(a[i], b[i]);
   }
 }
 }
@@ -337,7 +321,7 @@ public:
           double& xj(xi[j]);
           stream >> xj;
         }
-        pointData->GetArray(i)->InsertNextTuple(&xi[0]);
+        pointData->GetArray(i)->InsertNextTuple(xi.data());
       }
     }
     else
@@ -471,17 +455,17 @@ public:
     }
 
     PRINT(bb[0] << " " << bb[1] << " " << bb[2] << " " << bb[3] << " " << bb[4] << " " << bb[5]);
-    this->Controller->AllGather(bb, &this->BoundingBoxes[0], 6);
+    this->Controller->AllGather(bb, this->BoundingBoxes.data(), 6);
 
 #ifdef DEBUGTRACE
-    cout << "(" << Rank << ") BoundingBoxes: ";
+    std::cout << "(" << Rank << ") BoundingBoxes: ";
     for (int i = 0; i < NumProcs; i++)
     {
       double* box = this->GetBoundingBox(i);
-      cout << box[0] << " " << box[1] << " " << box[2] << " " << box[3] << " " << box[4] << " "
-           << box[5] << ";  ";
+      std::cout << box[0] << " " << box[1] << " " << box[2] << " " << box[3] << " " << box[4] << " "
+                << box[5] << ";  ";
     }
-    cout << endl;
+    std::cout << endl;
 #endif
   }
 
@@ -744,7 +728,8 @@ public:
     {
       amrFunc->SetLastDataSet(amrPoint->GetLevel(), amrPoint->GetGridId());
 #ifdef DEBUGTRACE
-      vtkUniformGrid* grid = this->AMR->GetDataSet(amrPoint->GetLevel(), amrPoint->GetGridId());
+      vtkCartesianGrid* grid =
+        this->AMR->GetDataSetAsCartesianGrid(amrPoint->GetLevel(), amrPoint->GetGridId());
       if (!grid || !InBB(amrPoint->GetSeed(), grid->GetBounds()))
       {
         PRINT("WARNING: Bad AMR Point "
@@ -764,7 +749,7 @@ public:
     {
       amrPoint->SetLevel(level);
       amrPoint->SetId(id);
-      int blockIndex = this->AMR->GetCompositeIndex(level, id);
+      int blockIndex = this->AMR->GetAbsoluteBlockIndex(level, id);
       amrPoint->SetRank(this->BlockProcess[blockIndex]);
       return true;
     }
@@ -790,7 +775,8 @@ public:
       return false;
     }
     AssertNe(amrp, nullptr);
-    vtkUniformGrid* grid = this->AMR->GetDataSet(amrp->GetLevel(), amrp->GetGridId());
+    vtkCartesianGrid* grid =
+      this->AMR->GetDataSetAsCartesianGrid(amrp->GetLevel(), amrp->GetGridId());
     return grid != nullptr;
   }
 
@@ -813,7 +799,7 @@ public:
       {
         amrp->SetLevel((int)level);
         amrp->SetGridId((int)gridId);
-        int blockIndex = this->AMR->GetCompositeIndex(level, gridId);
+        int blockIndex = this->AMR->GetAbsoluteBlockIndex(level, gridId);
         int process = this->BlockProcess[blockIndex];
         AssertGe(process, 0);
         amrp->SetRank(process);
@@ -848,12 +834,6 @@ vtkStandardNewMacro(AMRPStreamTracerUtils);
 //------------------------------------------------------------------------------
 namespace
 {
-inline double normvec3(double* x, double* y)
-{
-  return sqrt(
-    (x[0] - y[0]) * (x[0] - y[0]) + (x[1] - y[1]) * (x[1] - y[1]) + (x[2] - y[2]) * (x[2] - y[2]));
-}
-
 inline vtkIdType LastPointIndex(vtkPolyData* pathPoly)
 {
   vtkCellArray* pathCells = pathPoly->GetLines();
@@ -905,8 +885,8 @@ inline bool SameShape(vtkPointData* a, vtkPointData* b)
 
   if (a->GetNumberOfArrays() != b->GetNumberOfArrays())
   {
-    PrintNames(cerr, a);
-    PrintNames(cerr, b);
+    PrintNames(std::cerr, a);
+    PrintNames(std::cerr, b);
     return false;
   }
 
@@ -1028,7 +1008,7 @@ public:
     std::fill(this->HasData.begin(), this->HasData.end(), 0);
     {
       const int self_hasdata = hasData ? 1 : 0;
-      this->Controller->AllGather(&self_hasdata, &this->HasData[0], 1);
+      this->Controller->AllGather(&self_hasdata, this->HasData.data(), 1);
     }
 
     for (int i = 0; i < NumProcs; i++)
@@ -1054,7 +1034,7 @@ public:
 
     std::vector<int> processMap(MaxId + 1);
     this->Controller->AllReduce(
-      &processMap0[0], &processMap[0], MaxId + 1, vtkCommunicator::MAX_OP);
+      processMap0.data(), processMap.data(), MaxId + 1, vtkCommunicator::MAX_OP);
 
     int totalNumTasks = std::accumulate(processMap.begin(), processMap.end(), 0,
       [](int accumlatedSum, int b) { return accumlatedSum + (b >= 0 ? 1 : 0); });
@@ -1660,7 +1640,8 @@ int vtkPStreamTracer::RequestData(
   if (auto seedIds = vtkIntArray::SafeDownCast(output->GetCellData()->GetArray("SeedIds")))
   {
     vtkSMPTools::For(0, seedIds->GetNumberOfTuples(),
-      [&originalSeedIds, &seedIds](vtkIdType start, vtkIdType end) {
+      [&originalSeedIds, &seedIds](vtkIdType start, vtkIdType end)
+      {
         for (vtkIdType cc = start; cc < end; ++cc)
         {
           const auto seedIdx = seedIds->GetTypedComponent(cc, 0);
@@ -1695,7 +1676,8 @@ int vtkPStreamTracer::RequestData(
     lengths[id] += length;
   }
   std::vector<double> totalLengths(maxSeeds);
-  this->Controller->AllReduce(&lengths[0], &totalLengths[0], maxSeeds, vtkCommunicator::SUM_OP);
+  this->Controller->AllReduce(
+    lengths.data(), totalLengths.data(), maxSeeds, vtkCommunicator::SUM_OP);
 
   int numNonZeros(0);
   double totalLength(0);
@@ -1903,3 +1885,4 @@ void vtkPStreamTracer::Prepend(vtkPolyData* pathPoly, vtkPolyData* headPoly)
   AssertEq(newNumPoints, nPoints + 1);
   AssertEq(newNumPoints, pathPoly->GetNumberOfPoints());
 }
+VTK_ABI_NAMESPACE_END
