@@ -1,28 +1,26 @@
-/*
- * Copyright 2008 Sandia Corporation.
- * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
- * license for use of this work by or on behalf of the
- * U.S. Government. Redistribution and use in source and binary forms, with
- * or without modification, are permitted provided that this Notice and any
- * statement of authorship are reproduced on all copies.
- */
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright 2008 Sandia Corporation
+// SPDX-License-Identifier: LicenseRef-BSD-3-Clause-Sandia-USGov
 // .SECTION Thanks
 // Thanks to Philippe Pebay and David Thompson from Sandia National Laboratories
 // for implementing this test.
 
+#include "vtkDataAssembly.h"
 #include "vtkDataObjectCollection.h"
 #include "vtkDataSetAttributes.h"
 #include "vtkDescriptiveStatistics.h"
 #include "vtkDoubleArray.h"
 #include "vtkLogger.h"
 #include "vtkMath.h"
-#include "vtkMultiBlockDataSet.h"
 #include "vtkNew.h"
+#include "vtkStatisticalModel.h"
 #include "vtkStringArray.h"
 #include "vtkTable.h"
 #include "vtkTimerLog.h"
 #include "vtkUnsignedCharArray.h"
 
+#include <iostream>
+
 //=============================================================================
 int TestDescriptiveStatistics(int, char*[])
 {
@@ -131,7 +129,7 @@ int TestDescriptiveStatistics(int, char*[])
 
   // Pairs of interest
   int nMetrics = 3;
-  vtkStdString columns[] = { "Metric 1", "Metric 2", "Metric 0" };
+  std::string columns[] = { "Metric 1", "Metric 2", "Metric 0" };
 
   // Reference values
   // Means for metrics 0, 1, and 2, respectively
@@ -144,9 +142,9 @@ int TestDescriptiveStatistics(int, char*[])
   vtkDescriptiveStatistics* ds1 = vtkDescriptiveStatistics::New();
 
   // First verify that absence of input does not cause trouble
-  cout << "\n## Verifying that absence of input does not cause trouble... ";
+  std::cout << "\n## Verifying that absence of input does not cause trouble... ";
   ds1->Update();
-  cout << "done.\n";
+  std::cout << "done.\n";
 
   // Prepare first test with data
   ds1->SetInputData(vtkStatisticsAlgorithm::INPUT_DATA, datasetTable1);
@@ -155,7 +153,7 @@ int TestDescriptiveStatistics(int, char*[])
   // Select Columns of Interest
   for (int i = 0; i < nMetrics; ++i)
   {
-    ds1->AddColumn(columns[i]);
+    ds1->AddColumn(columns[i].c_str());
   }
 
   // Test Learn, Derive, Test, and Assess options
@@ -168,20 +166,20 @@ int TestDescriptiveStatistics(int, char*[])
 
   // Get output data and meta tables
   vtkTable* outputData1 = ds1->GetOutput(vtkStatisticsAlgorithm::OUTPUT_DATA);
-  vtkMultiBlockDataSet* outputMetaDS1 = vtkMultiBlockDataSet::SafeDownCast(
+  auto* outputMetaDS1 = vtkStatisticalModel::SafeDownCast(
     ds1->GetOutputDataObject(vtkStatisticsAlgorithm::OUTPUT_MODEL));
-  vtkTable* outputPrimary1 = vtkTable::SafeDownCast(outputMetaDS1->GetBlock(0));
-  vtkTable* outputDerived1 = vtkTable::SafeDownCast(outputMetaDS1->GetBlock(1));
+  vtkTable* outputPrimary1 = outputMetaDS1->GetTable(vtkStatisticalModel::Learned, 0);
+  vtkTable* outputDerived1 = outputMetaDS1->GetTable(vtkStatisticalModel::Derived, 0);
   vtkTable* outputTest1 = ds1->GetOutput(vtkStatisticsAlgorithm::OUTPUT_TEST);
 
-  cout << "\n## Calculated the following primary statistics for first data set:\n";
+  std::cout << "\n## Calculated the following primary statistics for first data set:\n";
   for (vtkIdType r = 0; r < outputPrimary1->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputPrimary1->GetNumberOfColumns(); ++i)
     {
       double val = outputPrimary1->GetValue(r, i).ToDouble();
-      cout << outputPrimary1->GetColumnName(i) << "=" << val << "  ";
+      std::cout << outputPrimary1->GetColumnName(i) << "=" << val << "  ";
     }
 
     // Verify some of the calculated primary statistics
@@ -190,18 +188,18 @@ int TestDescriptiveStatistics(int, char*[])
       vtkGenericWarningMacro("Incorrect mean");
       testStatus = 1;
     }
-    cout << "\n";
+    std::cout << "\n";
   }
 
-  cout << "\n## Calculated the following derived statistics for first data set:\n";
+  std::cout << "\n## Calculated the following derived statistics for first data set:\n";
   for (vtkIdType r = 0; r < outputDerived1->GetNumberOfRows(); ++r)
   {
     double error = false;
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputDerived1->GetNumberOfColumns(); ++i)
     {
       double val = outputDerived1->GetValue(r, i).ToDouble();
-      cout << outputDerived1->GetColumnName(i) << "=" << val << "  ";
+      std::cout << outputDerived1->GetColumnName(i) << "=" << val << "  ";
       // In this case, there is a zero variance, so skew and kurt should be NaN
       if (r == 2 && (i == 2 || i == 3) && val == val)
       {
@@ -220,27 +218,27 @@ int TestDescriptiveStatistics(int, char*[])
       vtkGenericWarningMacro("Incorrect standard deviation");
       testStatus = 1;
     }
-    cout << "\n";
+    std::cout << "\n";
   }
 
   // Check some results of the Test option
-  cout << "\n## Calculated the following Jarque-Bera statistics:\n";
+  std::cout << "\n## Calculated the following Jarque-Bera statistics:\n";
   for (vtkIdType r = 0; r < outputTest1->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputTest1->GetNumberOfColumns(); ++i)
     {
-      cout << outputTest1->GetColumnName(i) << "=" << outputTest1->GetValue(r, i).ToString()
-           << "  ";
+      std::cout << outputTest1->GetColumnName(i) << "=" << outputTest1->GetValue(r, i).ToString()
+                << "  ";
     }
 
-    cout << "\n";
+    std::cout << "\n";
   }
 
   // Search for outliers to check results of Assess option
   double maxdev = 1.5;
-  cout << "\n## Searching for outliers from mean with relative deviation > " << maxdev
-       << " for metric 1:\n";
+  std::cout << "\n## Searching for outliers from mean with relative deviation > " << maxdev
+            << " for metric 1:\n";
 
   vtkDoubleArray* vals0 =
     vtkArrayDownCast<vtkDoubleArray>(outputData1->GetColumnByName("Metric 0"));
@@ -268,9 +266,9 @@ int TestDescriptiveStatistics(int, char*[])
     if (dev > maxdev)
     {
       ++m0outliers;
-      cout << "   "
-           << " row " << r << ", " << devs0->GetName() << " = " << dev << " > " << maxdev
-           << " (value: " << vals0->GetValue(r) << ")\n";
+      std::cout << "   "
+                << " row " << r << ", " << devs0->GetName() << " = " << dev << " > " << maxdev
+                << " (value: " << vals0->GetValue(r) << ")\n";
     }
   }
   for (vtkIdType r = 0; r < outputData1->GetNumberOfRows(); ++r)
@@ -279,14 +277,14 @@ int TestDescriptiveStatistics(int, char*[])
     if (dev > maxdev)
     {
       ++m1outliers;
-      cout << "   "
-           << " row " << r << ", " << devs1->GetName() << " = " << dev << " > " << maxdev
-           << " (value: " << vals1->GetValue(r) << ")\n";
+      std::cout << "   "
+                << " row " << r << ", " << devs1->GetName() << " = " << dev << " > " << maxdev
+                << " (value: " << vals1->GetValue(r) << ")\n";
     }
   }
 
-  cout << "  Found " << m0outliers << " outliers for Metric 0"
-       << " and " << m1outliers << " outliers for Metric 1.\n";
+  std::cout << "  Found " << m0outliers << " outliers for Metric 0"
+            << " and " << m1outliers << " outliers for Metric 1.\n";
 
   if (m0outliers != 4 || m1outliers != 6)
   {
@@ -295,7 +293,7 @@ int TestDescriptiveStatistics(int, char*[])
   }
 
   // Now, used modified output 1 as input 1 to test 0-deviation
-  cout << "\n## Searching for values not equal to 50 for metric 1:\n";
+  std::cout << "\n## Searching for values not equal to 50 for metric 1:\n";
 
   vtkTable* modifiedPrimary = vtkTable::New();
   modifiedPrimary->ShallowCopy(outputPrimary1);
@@ -305,10 +303,13 @@ int TestDescriptiveStatistics(int, char*[])
   modifiedDerived->ShallowCopy(outputDerived1);
   modifiedDerived->SetValueByName(1, "Standard Deviation", 0.);
 
-  vtkMultiBlockDataSet* modifiedModel = vtkMultiBlockDataSet::New();
-  modifiedModel->SetNumberOfBlocks(2);
-  modifiedModel->SetBlock(0, modifiedPrimary);
-  modifiedModel->SetBlock(1, modifiedDerived);
+  auto* modifiedModel = vtkStatisticalModel::New();
+  modifiedModel->SetNumberOfTables(vtkStatisticalModel::Learned, 1);
+  modifiedModel->SetNumberOfTables(vtkStatisticalModel::Derived, 1);
+  modifiedModel->SetTable(vtkStatisticalModel::Learned, 0, modifiedPrimary,
+    outputMetaDS1->GetTableName(vtkStatisticalModel::Learned, 0));
+  modifiedModel->SetTable(vtkStatisticalModel::Derived, 0, modifiedDerived,
+    outputMetaDS1->GetTableName(vtkStatisticalModel::Derived, 0));
 
   // Run with Assess option only (do not recalculate nor rederive a model)
   ds1->SetInputData(vtkStatisticsAlgorithm::INPUT_MODEL, modifiedModel);
@@ -339,7 +340,7 @@ int TestDescriptiveStatistics(int, char*[])
     }
   }
 
-  cout << "  Found " << m1outliers << " outliers for Metric 1.\n";
+  std::cout << "  Found " << m1outliers << " outliers for Metric 1.\n";
 
   if (m1outliers != 28)
   {
@@ -393,7 +394,7 @@ int TestDescriptiveStatistics(int, char*[])
   // Select Columns of Interest (all of them)
   for (int i = 0; i < nMetrics; ++i)
   {
-    ds2->AddColumn(columns[i]);
+    ds2->AddColumn(columns[i].c_str());
   }
 
   // Update with Learn option only
@@ -404,25 +405,25 @@ int TestDescriptiveStatistics(int, char*[])
   ds2->Update();
 
   // Get output meta tables
-  vtkMultiBlockDataSet* outputMetaDS2 = vtkMultiBlockDataSet::SafeDownCast(
+  auto* outputMetaDS2 = vtkStatisticalModel::SafeDownCast(
     ds2->GetOutputDataObject(vtkStatisticsAlgorithm::OUTPUT_MODEL));
-  vtkTable* outputPrimary2 = vtkTable::SafeDownCast(outputMetaDS2->GetBlock(0));
+  vtkTable* outputPrimary2 = outputMetaDS2->GetTable(vtkStatisticalModel::Learned, 0);
 
-  cout << "\n## Calculated the following primary statistics for second data set:\n";
+  std::cout << "\n## Calculated the following primary statistics for second data set:\n";
   for (vtkIdType r = 0; r < outputPrimary2->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputPrimary2->GetNumberOfColumns(); ++i)
     {
-      cout << outputPrimary2->GetColumnName(i) << "=" << outputPrimary2->GetValue(r, i).ToString()
-           << "  ";
+      std::cout << outputPrimary2->GetColumnName(i) << "="
+                << outputPrimary2->GetValue(r, i).ToString() << "  ";
     }
-    cout << "\n";
+    std::cout << "\n";
   }
 
   // Test model aggregation by adding new data to engine which already has a model
   ds1->SetInputData(vtkStatisticsAlgorithm::INPUT_DATA, datasetTable2);
-  vtkMultiBlockDataSet* model = vtkMultiBlockDataSet::New();
+  auto* model = vtkStatisticalModel::New();
   model->ShallowCopy(outputMetaDS1);
   ds1->SetInputData(vtkStatisticsAlgorithm::INPUT_MODEL, model);
 
@@ -446,20 +447,20 @@ int TestDescriptiveStatistics(int, char*[])
   double stdevs0[] = { sqrt(6.1418651), sqrt(7.548397 * 62. / 63.), sqrt(64. / 63.) };
 
   // Get output data and meta tables
-  outputMetaDS1 = vtkMultiBlockDataSet::SafeDownCast(
+  outputMetaDS1 = vtkStatisticalModel::SafeDownCast(
     ds1->GetOutputDataObject(vtkStatisticsAlgorithm::OUTPUT_MODEL));
-  outputPrimary1 = vtkTable::SafeDownCast(outputMetaDS1->GetBlock(0));
-  outputDerived1 = vtkTable::SafeDownCast(outputMetaDS1->GetBlock(1));
+  outputPrimary1 = outputMetaDS1->GetTable(vtkStatisticalModel::Learned, 0);
+  outputDerived1 = outputMetaDS1->GetTable(vtkStatisticalModel::Derived, 0);
 
-  cout
+  std::cout
     << "\n## Calculated the following primary statistics for updated (first + second) data set:\n";
   for (vtkIdType r = 0; r < outputPrimary1->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputPrimary1->GetNumberOfColumns(); ++i)
     {
-      cout << outputPrimary1->GetColumnName(i) << "=" << outputPrimary1->GetValue(r, i).ToString()
-           << "  ";
+      std::cout << outputPrimary1->GetColumnName(i) << "="
+                << outputPrimary1->GetValue(r, i).ToString() << "  ";
     }
 
     // Verify some of the calculated primary statistics
@@ -468,18 +469,18 @@ int TestDescriptiveStatistics(int, char*[])
       vtkGenericWarningMacro("Incorrect mean");
       testStatus = 1;
     }
-    cout << "\n";
+    std::cout << "\n";
   }
 
-  cout
+  std::cout
     << "\n## Calculated the following derived statistics for updated (first + second) data set:\n";
   for (vtkIdType r = 0; r < outputDerived1->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputDerived1->GetNumberOfColumns(); ++i)
     {
-      cout << outputDerived1->GetColumnName(i) << "=" << outputDerived1->GetValue(r, i).ToString()
-           << "  ";
+      std::cout << outputDerived1->GetColumnName(i) << "="
+                << outputDerived1->GetValue(r, i).ToString() << "  ";
     }
 
     // Verify some of the calculated derived statistics
@@ -489,7 +490,7 @@ int TestDescriptiveStatistics(int, char*[])
       vtkGenericWarningMacro("Incorrect standard deviation");
       testStatus = 1;
     }
-    cout << "\n";
+    std::cout << "\n";
   }
 
   // Clean up
@@ -566,7 +567,7 @@ int TestDescriptiveStatistics(int, char*[])
   ds3->AddColumn("Bogus");
 
   // Warning for non existing column will mess up output
-  cout << "\n";
+  std::cout << "\n";
 
   // Test Learn and Derive options only
   ds3->SetLearnOption(true);
@@ -579,17 +580,17 @@ int TestDescriptiveStatistics(int, char*[])
   vtkLog(INFO, "### Testing Sample Statistics ###");
 
   // Get output data and meta tables
-  vtkMultiBlockDataSet* outputMetaDS3 = vtkMultiBlockDataSet::SafeDownCast(
+  auto* outputMetaDS3 = vtkStatisticalModel::SafeDownCast(
     ds3->GetOutputDataObject(vtkStatisticsAlgorithm::OUTPUT_MODEL));
-  vtkTable* outputPrimary3 = vtkTable::SafeDownCast(outputMetaDS3->GetBlock(0));
-  vtkTable* outputDerived3 = vtkTable::SafeDownCast(outputMetaDS3->GetBlock(1));
+  vtkTable* outputPrimary3 = outputMetaDS3->GetTable(vtkStatisticalModel::Learned, 0);
+  vtkTable* outputDerived3 = outputMetaDS3->GetTable(vtkStatisticalModel::Derived, 0);
 
-  cout << "\n## Calculated the following primary statistics for {0,...9} sequence:\n";
-  cout << "   ";
+  std::cout << "\n## Calculated the following primary statistics for {0,...9} sequence:\n";
+  std::cout << "   ";
   for (int i = 0; i < outputPrimary3->GetNumberOfColumns(); ++i)
   {
-    cout << outputPrimary3->GetColumnName(i) << "=" << outputPrimary3->GetValue(0, i).ToString()
-         << "  ";
+    std::cout << outputPrimary3->GetColumnName(i) << "="
+              << outputPrimary3->GetValue(0, i).ToString() << "  ";
   }
 
   // Verify some of the calculated primary statistics
@@ -598,14 +599,14 @@ int TestDescriptiveStatistics(int, char*[])
     vtkGenericWarningMacro("Incorrect mean");
     testStatus = 1;
   }
-  cout << "\n";
+  std::cout << "\n";
 
-  cout << "\n## Calculated the following derived statistics for {0,...9} sequence:\n";
-  cout << "   ";
+  std::cout << "\n## Calculated the following derived statistics for {0,...9} sequence:\n";
+  std::cout << "   ";
   for (int i = 0; i < outputDerived3->GetNumberOfColumns(); ++i)
   {
-    cout << outputDerived3->GetColumnName(i) << "=" << outputDerived3->GetValue(0, i).ToString()
-         << "  ";
+    std::cout << outputDerived3->GetColumnName(i) << "="
+              << outputDerived3->GetValue(0, i).ToString() << "  ";
   }
 
   // Verify some of the calculated derived statistics
@@ -633,7 +634,7 @@ int TestDescriptiveStatistics(int, char*[])
     vtkGenericWarningMacro("Incorrect kurtosis");
     testStatus = 1;
   }
-  cout << "\n";
+  std::cout << "\n";
 
   vtkLog(INFO, "### Testing Population Statistics ###");
 
@@ -641,9 +642,9 @@ int TestDescriptiveStatistics(int, char*[])
   ds3->SampleEstimateOff();
   ds3->SetGhostsToSkip(1);
   ds3->Update();
-  outputMetaDS3 = vtkMultiBlockDataSet::SafeDownCast(
+  outputMetaDS3 = vtkStatisticalModel::SafeDownCast(
     ds3->GetOutputDataObject(vtkStatisticsAlgorithm::OUTPUT_MODEL));
-  outputDerived3 = vtkTable::SafeDownCast(outputMetaDS3->GetBlock(1));
+  outputDerived3 = outputMetaDS3->GetTable(vtkStatisticalModel::Derived, 0);
 
   variance = 12.96;
   standardDeviation = 3.6;
@@ -677,7 +678,7 @@ int TestDescriptiveStatistics(int, char*[])
     vtkGenericWarningMacro("Incorrect kurtosis");
     testStatus = 1;
   }
-  cout << "\n";
+  std::cout << "\n";
 
   // Clean up
   ds3->Delete();
@@ -750,62 +751,63 @@ int TestDescriptiveStatistics(int, char*[])
   ds4->Update();
 
   // Get output data and meta tables
-  vtkMultiBlockDataSet* outputMetaDS4 = vtkMultiBlockDataSet::SafeDownCast(
+  auto* outputMetaDS4 = vtkStatisticalModel::SafeDownCast(
     ds4->GetOutputDataObject(vtkStatisticsAlgorithm::OUTPUT_MODEL));
-  vtkTable* outputPrimary4 = vtkTable::SafeDownCast(outputMetaDS4->GetBlock(0));
-  vtkTable* outputDerived4 = vtkTable::SafeDownCast(outputMetaDS4->GetBlock(1));
+  vtkTable* outputPrimary4 = outputMetaDS4->GetTable(vtkStatisticalModel::Learned, 0);
+  vtkTable* outputDerived4 = outputMetaDS4->GetTable(vtkStatisticalModel::Derived, 0);
   vtkTable* outputTest4 = ds4->GetOutput(vtkStatisticsAlgorithm::OUTPUT_TEST);
 
-  cout << "\n## Calculated the following primary statistics for pseudo-random variables (n="
-       << nVals << "):\n";
+  std::cout << "\n## Calculated the following primary statistics for pseudo-random variables (n="
+            << nVals << "):\n";
   for (vtkIdType r = 0; r < outputPrimary4->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputPrimary4->GetNumberOfColumns(); ++i)
     {
-      cout << outputPrimary4->GetColumnName(i) << "=" << outputPrimary4->GetValue(r, i).ToString()
-           << "  ";
+      std::cout << outputPrimary4->GetColumnName(i) << "="
+                << outputPrimary4->GetValue(r, i).ToString() << "  ";
     }
 
-    cout << "\n";
+    std::cout << "\n";
   }
 
-  cout << "\n## Calculated the following derived statistics for pseudo-random variables (n="
-       << nVals << "):\n";
+  std::cout << "\n## Calculated the following derived statistics for pseudo-random variables (n="
+            << nVals << "):\n";
   for (vtkIdType r = 0; r < outputDerived4->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int i = 0; i < outputDerived4->GetNumberOfColumns(); ++i)
     {
-      cout << outputDerived4->GetColumnName(i) << "=" << outputDerived4->GetValue(r, i).ToString()
-           << "  ";
+      std::cout << outputDerived4->GetColumnName(i) << "="
+                << outputDerived4->GetValue(r, i).ToString() << "  ";
     }
 
-    cout << "\n";
+    std::cout << "\n";
   }
 
   // Check some results of the Test option
-  cout << "\n## Calculated the following Jarque-Bera statistics for pseudo-random variables (n="
-       << nVals;
+  std::cout
+    << "\n## Calculated the following Jarque-Bera statistics for pseudo-random variables (n="
+    << nVals;
 
 #ifdef USE_GNU_R
   int nNonGaussian = 3;
   int nRejected = 0;
   double alpha = .01;
 
-  cout << ", null hypothesis: normality, significance level=" << alpha;
+  std::cout << ", null hypothesis: normality, significance level=" << alpha;
 #endif // USE_GNU_R
 
-  cout << "):\n";
+  std::cout << "):\n";
 
   // Loop over Test table
   for (vtkIdType r = 0; r < outputTest4->GetNumberOfRows(); ++r)
   {
-    cout << "   ";
+    std::cout << "   ";
     for (int c = 0; c < outputTest4->GetNumberOfColumns(); ++c)
     {
-      cout << outputTest4->GetColumnName(c) << "=" << outputTest4->GetValue(r, c).ToString()
-           << "  ";
+      std::cout << outputTest4->GetColumnName(c) << "=" << outputTest4->GetValue(r, c).ToString()
+                << "  ";
     }
 
 #ifdef USE_GNU_R
@@ -814,13 +816,13 @@ int TestDescriptiveStatistics(int, char*[])
     // Must verify that p value is valid (it is set to -1 if R has failed)
     if (p > -1 && p < alpha)
     {
-      cout << "N.H. rejected";
+      std::cout << "N.H. rejected";
 
       ++nRejected;
     }
 #endif // USE_GNU_R
 
-    cout << "\n";
+    std::cout << "\n";
   }
 
 #ifdef USE_GNU_R
