@@ -1,24 +1,15 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    cgio_helpers.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
-//  Copyright 2013-2014 Mickael Philit.
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright 2013-2014 Mickael Philit
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "cgio_helpers.h"
 
+#include <iostream>
+
 namespace CGNSRead
 {
-//----------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
+//------------------------------------------------------------------------------
 int readNodeStringData(int cgioNum, double nodeId, std::string& data)
 {
   int n;
@@ -51,8 +42,8 @@ int readNodeStringData(int cgioNum, double nodeId, std::string& data)
 
   return 0;
 }
+
 //------------------------------------------------------------------------------
-// Specialize char array
 template <>
 int readNodeData<char>(int cgioNum, double nodeId, std::vector<char>& data)
 {
@@ -80,7 +71,7 @@ int readNodeData<char>(int cgioNum, double nodeId, std::vector<char>& data)
   data.resize(size + 1);
 
   // read data
-  if (cgio_read_all_data_type(cgioNum, nodeId, "C1", &data[0]) != CG_OK)
+  if (cgio_read_all_data_type(cgioNum, nodeId, "C1", data.data()) != CG_OK)
   {
     return 1;
   }
@@ -135,10 +126,13 @@ int readBaseIds(int cgioNum, double rootId, std::vector<double>& baseIds)
 
   for (nbases = 0, nc = 0; nc < baseIds.size(); nc++)
   {
+    // Ignore missing or invalid bases
     if (cgio_get_label(cgioNum, baseIds[nc], nodeLabel) != CG_OK)
     {
-      return 1;
+      vtkWarningWithObjectMacro(nullptr, "Failed to read node label, ignoring current base.");
+      continue;
     }
+
     if (strcmp(nodeLabel, "CGNSBase_t") == 0)
     {
       if (nbases < nc)
@@ -791,4 +785,5 @@ void releaseIds(int cgioNum, const std::vector<double>& ids)
     cgio_release_id(cgioNum, *iter);
   }
 }
+VTK_ABI_NAMESPACE_END
 }
