@@ -1,5 +1,7 @@
 // SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
 // SPDX-License-Identifier: BSD-3-Clause
+#include "vtkLegacy.h"
+
 #include "vtkConduitArrayUtilities.h"
 
 #include "vtkArrayDispatch.h"
@@ -7,7 +9,30 @@
 #include "vtkDataSetAttributes.h"
 #include "vtkLogger.h"
 #include "vtkObjectFactory.h"
-#include "vtkSOADataArrayTemplate.h"
+#include "vtkSOATypeFloat32Array.h"
+#include "vtkSOATypeFloat64Array.h"
+#include "vtkSOATypeInt16Array.h"
+#include "vtkSOATypeInt32Array.h"
+#include "vtkSOATypeInt64Array.h"
+#include "vtkSOATypeInt8Array.h"
+#include "vtkSOATypeUInt16Array.h"
+#include "vtkSOATypeUInt32Array.h"
+#include "vtkSOATypeUInt64Array.h"
+#include "vtkSOATypeUInt8Array.h"
+#include "vtkSetGet.h"
+#include "vtkSmartPointer.h"
+#include "vtkStridedTypeFloat32Array.h"
+#include "vtkStridedTypeFloat64Array.h"
+#include "vtkStridedTypeInt16Array.h"
+#include "vtkStridedTypeInt32Array.h"
+#include "vtkStridedTypeInt64Array.h"
+#include "vtkStridedTypeInt8Array.h"
+#include "vtkStridedTypeUInt16Array.h"
+#include "vtkStridedTypeUInt32Array.h"
+#include "vtkStridedTypeUInt64Array.h"
+#include "vtkStridedTypeUInt8Array.h"
+#include "vtkStringFormatter.h"
+#include "vtkType.h"
 #include "vtkTypeFloat32Array.h"
 #include "vtkTypeFloat64Array.h"
 #include "vtkTypeInt16Array.h"
@@ -19,9 +44,16 @@
 #include "vtkTypeUInt64Array.h"
 #include "vtkTypeUInt8Array.h"
 
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+#include "vtkConduitArrayUtilitiesDevice.h"
+#include "vtkmDataArrayUtilities.h"
+#endif // VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+
 #include <catalyst_conduit.hpp>
 #include <catalyst_conduit_blueprint.hpp>
 
+#include <type_traits>
+#include <typeinfo>
 #include <vector>
 
 namespace internals
@@ -60,26 +92,44 @@ bool is_contiguous(const conduit_cpp::Node& node)
   return true;
 }
 
-template <typename ArrayT>
-vtkSmartPointer<ArrayT> CreateAOSArray(
-  vtkIdType number_of_tuples, int number_of_components, const typename ArrayT::ValueType* raw_ptr)
+template <typename AOSArrayT, typename ValueType = typename AOSArrayT::ValueType>
+vtkSmartPointer<AOSArrayT> CreateAOSArray(
+  vtkIdType number_of_tuples, int number_of_components, const void* raw_ptr)
 {
-  auto array = vtkSmartPointer<ArrayT>::New();
+  auto array = vtkSmartPointer<AOSArrayT>::New();
   array->SetNumberOfComponents(number_of_components);
-  array->SetArray(const_cast<typename ArrayT::ValueType*>(raw_ptr),
+  array->SetArray(static_cast<ValueType*>(const_cast<void*>(raw_ptr)),
     number_of_tuples * number_of_components, /*save=*/1);
   return array;
 }
 
-template <typename ValueT>
-vtkSmartPointer<vtkSOADataArrayTemplate<ValueT>> CreateSOArray(
+template <typename StridedArrayT, typename ValueType = typename StridedArrayT::ValueType>
+vtkSmartPointer<StridedArrayT> CreateStridedArray(const conduit_cpp::Node& arrayNode)
+{
+  const auto& child0 = arrayNode.child(0);
+  const conduit_cpp::DataType dtype0 = child0.dtype();
+  const int number_of_components = static_cast<int>(arrayNode.number_of_children());
+  const vtkIdType number_of_tuples = static_cast<vtkIdType>(dtype0.number_of_elements());
+  const int strideBytes = dtype0.stride();
+  const int index_stride = strideBytes / (sizeof(ValueType));
+
+  auto array = vtkSmartPointer<StridedArrayT>::New();
+  array->SetNumberOfComponents(number_of_components);
+  array->SetNumberOfTuples(number_of_tuples);
+  array->ConstructBackend(
+    static_cast<const ValueType*>(child0.element_ptr(0)), index_stride, number_of_components);
+  return array;
+}
+
+template <typename SOAArrayT, typename ValueType = typename SOAArrayT::ValueType>
+vtkSmartPointer<SOAArrayT> CreateSOArray(
   vtkIdType number_of_tuples, int number_of_components, const std::vector<void*>& raw_ptrs)
 {
-  auto array = vtkSmartPointer<vtkSOADataArrayTemplate<ValueT>>::New();
+  auto array = vtkSmartPointer<SOAArrayT>::New();
   array->SetNumberOfComponents(number_of_components);
   for (int cc = 0; cc < number_of_components; ++cc)
   {
-    array->SetArray(cc, reinterpret_cast<ValueT*>(raw_ptrs.at(cc)), number_of_tuples,
+    array->SetArray(cc, static_cast<ValueType*>(raw_ptrs.at(cc)), number_of_tuples,
       /*updateMaxId=*/true, /*save*/ true);
   }
   return array;
@@ -147,6 +197,7 @@ struct ChangeComponentsSOAImpl
     }
   }
 };
+
 //----------------------------------------------------------------------------
 static vtkSmartPointer<vtkDataArray> ChangeComponentsSOA(vtkDataArray* array, int num_components)
 {
@@ -166,30 +217,32 @@ static vtkSmartPointer<vtkDataArray> ChangeComponentsSOA(vtkDataArray* array, in
 }
 
 //----------------------------------------------------------------------------
-conduit_cpp::DataType::Id GetTypeId(conduit_cpp::DataType::Id type, bool force_signed)
+struct FromHostConduitToMixedCellArray
 {
-  if (!force_signed)
+public:
+  FromHostConduitToMixedCellArray(vtkCellArray* cellArray)
+    : CellArray(cellArray)
   {
-    return type;
   }
-  switch (type)
+
+  template <typename ArrayT1, typename ArrayT2>
+  void operator()(ArrayT1* offsets, ArrayT2* connectivity)
   {
-    case conduit_cpp::DataType::Id::uint8:
-      return conduit_cpp::DataType::Id::int8;
+    // conduit offsets array does not include the last index = connectivity.size() as vtkCellArray
+    auto vtkOffsets = vtk::TakeSmartPointer(offsets->NewInstance());
+    vtkOffsets->SetNumberOfTuples(offsets->GetNumberOfTuples() + 1);
+    vtkOffsets->SetNumberOfComponents(1);
 
-    case conduit_cpp::DataType::Id::uint16:
-      return conduit_cpp::DataType::Id::int16;
-
-    case conduit_cpp::DataType::Id::uint32:
-      return conduit_cpp::DataType::Id::int32;
-
-    case conduit_cpp::DataType::Id::uint64:
-      return conduit_cpp::DataType::Id::int64;
-
-    default:
-      return type;
+    const auto offsetsRange = vtk::DataArrayValueRange<1>(offsets);
+    auto vtkOffsetsRange = vtk::DataArrayValueRange<1>(vtkOffsets);
+    std::copy(offsetsRange.begin(), offsetsRange.end(), vtkOffsetsRange.begin());
+    *(vtkOffsetsRange.end() - 1) = connectivity->GetNumberOfTuples();
+    this->CellArray->SetData(vtkOffsets, connectivity);
   }
-}
+
+private:
+  vtkCellArray* CellArray;
+};
 
 VTK_ABI_NAMESPACE_END
 } // internals
@@ -197,6 +250,7 @@ VTK_ABI_NAMESPACE_END
 VTK_ABI_NAMESPACE_BEGIN
 
 vtkStandardNewMacro(vtkConduitArrayUtilities);
+
 //----------------------------------------------------------------------------
 vtkConduitArrayUtilities::vtkConduitArrayUtilities() = default;
 
@@ -215,13 +269,6 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArray(
   return nullptr;
 }
 
-//----------------------------------------------------------------------------
-vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArray(
-  const conduit_node* mcarray)
-{
-  return vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(mcarray, false);
-}
-
 //----------------------------------------------------------------------------
 vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCGhostArrayToVTKGhostArray(
   const conduit_node* c_mcarray, bool is_cell_data)
@@ -259,8 +306,8 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCGhostArrayToVTKGhostAr
 }
 
 //----------------------------------------------------------------------------
-vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
-  const conduit_node* c_mcarray, bool force_signed)
+vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArray(
+  const conduit_node* c_mcarray)
 {
   // XXX(const-correctness): This should really be `const Node`, but is used
   // non-const in the `set_external` below.
@@ -274,15 +321,13 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
     {
       conduit_cpp::Node temp;
       temp.append().set_external(mcarray);
-      return vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
-        conduit_cpp::c_node(&temp), force_signed);
+      return vtkConduitArrayUtilities::MCArrayToVTKArray(conduit_cpp::c_node(&temp));
     }
     // in some cases, the array is inside a values subnode. handle that
     else if (mcarray.has_path("values"))
     {
       const auto& tmp = mcarray["values"];
-      return vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
-        conduit_cpp::c_node(&tmp), force_signed);
+      return vtkConduitArrayUtilities::MCArrayToVTKArray(conduit_cpp::c_node(&tmp));
     }
     else
     {
@@ -314,20 +359,81 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
     }
   }
 
-  if (conduit_cpp::BlueprintMcArray::is_interleaved(mcarray))
+  int8_t id;
+  bool working;
+  bool isDevicePointer = IsDevicePointer(mcarray.child(0).element_ptr(0), id, working);
+  if (isDevicePointer && !working)
   {
-    return vtkConduitArrayUtilities::MCArrayToVTKAOSArray(
-      conduit_cpp::c_node(&mcarray), force_signed);
+    vtkLog(ERROR, "Viskores does not support device" + vtk::to_string(id));
+    return nullptr;
+  }
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+  auto deviceAdapterId = viskores::cont::make_DeviceAdapterId(id);
+#endif
+
+  auto numTuples = mcarray.child(0).dtype().number_of_elements();
+  if (conduit_cpp::BlueprintMcArray::is_interleaved(mcarray) || numTuples == 0)
+  {
+    if (isDevicePointer)
+    {
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+      return vtkConduitArrayUtilitiesDevice::MCArrayToVTKmAOSArray(
+        conduit_cpp::c_node(&mcarray), deviceAdapterId);
+#else
+      // IsDeviceMemory returns false in this case
+      vtkLogF(ERROR, "VTK was not compiled with AcceleratorsVTKmDataModel");
+      return nullptr;
+#endif
+    }
+    else
+    {
+      const auto& child0 = mcarray.child(0);
+      const conduit_cpp::DataType dtype0 = child0.dtype();
+
+      if (mcarray.number_of_children() * dtype0.element_bytes() != dtype0.stride())
+      {
+        // there is some data interlaced with current array
+        return vtkConduitArrayUtilities::MCArrayToVTKStridedArray(conduit_cpp::c_node(&mcarray));
+      }
+
+      return vtkConduitArrayUtilities::MCArrayToVTKAOSArray(conduit_cpp::c_node(&mcarray));
+    }
   }
   else if (internals::is_contiguous(mcarray))
   {
-    return vtkConduitArrayUtilities::MCArrayToVTKSOAArray(
-      conduit_cpp::c_node(&mcarray), force_signed);
+    if (isDevicePointer)
+    {
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+      return vtkConduitArrayUtilitiesDevice::MCArrayToVTKmSOAArray(
+        conduit_cpp::c_node(&mcarray), deviceAdapterId);
+#else
+      // IsDeviceMemory returns false in this case
+      vtkLogF(ERROR, "VTK was not compiled with AcceleratorsVTKmDataModel");
+      return nullptr;
+#endif
+    }
+    else
+    {
+      return vtkConduitArrayUtilities::MCArrayToVTKSOAArray(conduit_cpp::c_node(&mcarray));
+    }
   }
   else if (mcarray.dtype().number_of_elements() == 1)
   {
-    return vtkConduitArrayUtilities::MCArrayToVTKSOAArray(
-      conduit_cpp::c_node(&mcarray), force_signed);
+    if (isDevicePointer)
+    {
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+      return vtkConduitArrayUtilitiesDevice::MCArrayToVTKmSOAArray(
+        conduit_cpp::c_node(&mcarray), deviceAdapterId);
+#else
+      // IsDeviceMemory returns false in this case
+      vtkLogF(ERROR, "VTK was not compiled with AcceleratorsVTKmDataModel");
+      return nullptr;
+#endif
+    }
+    else
+    {
+      return vtkConduitArrayUtilities::MCArrayToVTKSOAArray(conduit_cpp::c_node(&mcarray));
+    }
   }
   else
   {
@@ -339,9 +445,55 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
   return nullptr;
 }
 
+//----------------------------------------------------------------------------
+vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKStridedArray(
+  const conduit_node* c_mcarray)
+{
+  const conduit_cpp::Node mcarray = conduit_cpp::cpp_node(const_cast<conduit_node*>(c_mcarray));
+  const auto& child0 = mcarray.child(0);
+  const conduit_cpp::DataType dtype0 = child0.dtype();
+
+  switch (dtype0.id())
+  {
+    case conduit_cpp::DataType::Id::int8:
+      return internals::CreateStridedArray<vtkStridedTypeInt8Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::int16:
+      return internals::CreateStridedArray<vtkStridedTypeInt16Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::int32:
+      return internals::CreateStridedArray<vtkStridedTypeInt32Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::int64:
+      return internals::CreateStridedArray<vtkStridedTypeInt64Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::uint8:
+      return internals::CreateStridedArray<vtkStridedTypeUInt8Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::uint16:
+      return internals::CreateStridedArray<vtkStridedTypeUInt16Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::uint32:
+      return internals::CreateStridedArray<vtkStridedTypeUInt32Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::uint64:
+      return internals::CreateStridedArray<vtkStridedTypeUInt64Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::float32:
+      return internals::CreateStridedArray<vtkStridedTypeFloat32Array>(mcarray);
+
+    case conduit_cpp::DataType::Id::float64:
+      return internals::CreateStridedArray<vtkStridedTypeFloat64Array>(mcarray);
+
+    default:
+      vtkLogF(ERROR, "unsupported data type '%s' ", dtype0.name().c_str());
+      return nullptr;
+  }
+}
+
 //----------------------------------------------------------------------------
 vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKAOSArray(
-  const conduit_node* c_mcarray, bool force_signed)
+  const conduit_node* c_mcarray)
 {
   const conduit_cpp::Node mcarray = conduit_cpp::cpp_node(const_cast<conduit_node*>(c_mcarray));
   auto& child0 = mcarray.child(0);
@@ -350,47 +502,47 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKAOSArray(
   const int num_components = static_cast<int>(mcarray.number_of_children());
   const vtkIdType num_tuples = static_cast<vtkIdType>(dtype0.number_of_elements());
 
-  switch (internals::GetTypeId(dtype0.id(), force_signed))
+  switch (dtype0.id())
   {
     case conduit_cpp::DataType::Id::int8:
-      return internals::CreateAOSArray<vtkTypeInt8Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeInt8Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeInt8Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::int16:
-      return internals::CreateAOSArray<vtkTypeInt16Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeInt16Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeInt16Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::int32:
-      return internals::CreateAOSArray<vtkTypeInt32Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeInt32Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeInt32Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::int64:
-      return internals::CreateAOSArray<vtkTypeInt64Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeInt64Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeInt64Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::uint8:
-      return internals::CreateAOSArray<vtkTypeUInt8Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeUInt8Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeUInt8Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::uint16:
-      return internals::CreateAOSArray<vtkTypeUInt16Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeUInt16Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeUInt16Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::uint32:
-      return internals::CreateAOSArray<vtkTypeUInt32Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeUInt32Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeUInt32Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::uint64:
-      return internals::CreateAOSArray<vtkTypeUInt64Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeUInt64Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeUInt64Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::float32:
-      return internals::CreateAOSArray<vtkTypeFloat32Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeFloat32Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeFloat32Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     case conduit_cpp::DataType::Id::float64:
-      return internals::CreateAOSArray<vtkTypeFloat64Array>(num_tuples, num_components,
-        reinterpret_cast<const vtkTypeFloat64Array::ValueType*>(child0.element_ptr(0)));
+      return internals::CreateAOSArray<vtkTypeFloat64Array>(
+        num_tuples, num_components, child0.element_ptr(0));
 
     default:
       vtkLogF(ERROR, "unsupported data type '%s' ", dtype0.name().c_str());
@@ -400,7 +552,7 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKAOSArray(
 
 //----------------------------------------------------------------------------
 vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKSOAArray(
-  const conduit_node* c_mcarray, bool force_signed)
+  const conduit_node* c_mcarray)
 {
   const conduit_cpp::Node mcarray = conduit_cpp::cpp_node(const_cast<conduit_node*>(c_mcarray));
   const conduit_cpp::DataType dtype0 = mcarray.child(0).dtype();
@@ -414,37 +566,37 @@ vtkSmartPointer<vtkDataArray> vtkConduitArrayUtilities::MCArrayToVTKSOAArray(
     ptrs.push_back(const_cast<void*>(mcarray.child(cc).element_ptr(0)));
   }
 
-  switch (internals::GetTypeId(dtype0.id(), force_signed))
+  switch (dtype0.id())
   {
     case conduit_cpp::DataType::Id::int8:
-      return internals::CreateSOArray<vtkTypeInt8>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeInt8Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::int16:
-      return internals::CreateSOArray<vtkTypeInt16>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeInt16Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::int32:
-      return internals::CreateSOArray<vtkTypeInt32>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeInt32Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::int64:
-      return internals::CreateSOArray<vtkTypeInt64>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeInt64Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::uint8:
-      return internals::CreateSOArray<vtkTypeUInt8>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeUInt8Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::uint16:
-      return internals::CreateSOArray<vtkTypeUInt16>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeUInt16Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::uint32:
-      return internals::CreateSOArray<vtkTypeUInt32>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeUInt32Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::uint64:
-      return internals::CreateSOArray<vtkTypeUInt64>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeUInt64Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::float32:
-      return internals::CreateSOArray<vtkTypeFloat32>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeFloat32Array>(num_tuples, num_components, ptrs);
 
     case conduit_cpp::DataType::Id::float64:
-      return internals::CreateSOArray<vtkTypeFloat64>(num_tuples, num_components, ptrs);
+      return internals::CreateSOArray<vtkSOATypeFloat64Array>(num_tuples, num_components, ptrs);
 
     default:
       vtkLogF(ERROR, "unsupported data type '%s' ", dtype0.name().c_str());
@@ -481,73 +633,64 @@ struct NoOp
 
 //----------------------------------------------------------------------------
 vtkSmartPointer<vtkCellArray> vtkConduitArrayUtilities::MCArrayToVTKCellArray(
-  vtkIdType cellSize, const conduit_node* mcarray)
+  vtkIdType numberOfPoints, int cellType, vtkIdType cellSize, const conduit_node* c_mcarray)
 {
-  auto array = vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(mcarray, /*force_signed*/ true);
-  if (!array)
+  auto connectivity = vtkConduitArrayUtilities::MCArrayToVTKArray(c_mcarray);
+  conduit_cpp::Node mcarray = conduit_cpp::cpp_node(const_cast<conduit_node*>(c_mcarray));
+
+  int8_t id;
+  bool working;
+  bool isDevicePointer = IsDevicePointer(mcarray.element_ptr(0), id, working);
+  if (isDevicePointer && !working)
+  {
+    vtkLog(ERROR, "Viskores does not support device" + vtk::to_string(id));
+    return nullptr;
+  }
+  if (!connectivity)
   {
     return nullptr;
   }
-
-  // now the array matches the type accepted by vtkCellArray (in most cases).
   vtkNew<vtkCellArray> cellArray;
-  cellArray->SetData(cellSize, array);
+  if (isDevicePointer)
+  {
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+    if (!vtkConduitArrayUtilitiesDevice::IfVTKmConvertVTKMonoShapedCellArray(
+          numberOfPoints, cellType, cellSize, connectivity, cellArray))
+    {
+      vtkLogF(ERROR, "Cannot convert connectivity to a vtkmArray");
+      return nullptr;
+    }
+#else
+    (void)cellType;
+    (void)numberOfPoints; // avoid unused variable warning
+#endif // VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+  }
+  else
+  {
+    // cell arrays are in host memory
+    cellArray->SetData(cellSize, connectivity);
+  }
   return cellArray;
 }
 
-VTK_ABI_NAMESPACE_END
-
-namespace
-{
-VTK_ABI_NAMESPACE_BEGIN
-
-struct O2MRelationToVTKCellArrayWorker
-{
-  vtkNew<vtkCellArray> Cells;
-
-  template <typename ElementsArray, typename SizesArray, typename OffsetsArray>
-  void operator()(ElementsArray* elements, SizesArray* sizes, OffsetsArray* offsets)
-  {
-    VTK_ASSUME(elements->GetNumberOfComponents() == 1);
-    VTK_ASSUME(sizes->GetNumberOfComponents() == 1);
-    VTK_ASSUME(offsets->GetNumberOfComponents() == 1);
-
-    auto& cellArray = this->Cells;
-    cellArray->AllocateEstimate(offsets->GetNumberOfTuples(),
-      std::max(static_cast<vtkIdType>(sizes->GetRange(0)[1]), vtkIdType(1)));
-
-    vtkDataArrayAccessor<ElementsArray> e(elements);
-    vtkDataArrayAccessor<SizesArray> s(sizes);
-    vtkDataArrayAccessor<OffsetsArray> o(offsets);
-
-    const auto numElements = sizes->GetNumberOfTuples();
-    for (vtkIdType id = 0; id < numElements; ++id)
-    {
-      const auto offset = static_cast<vtkIdType>(o.Get(id, 0));
-      const auto size = static_cast<vtkIdType>(s.Get(id, 0));
-
-      cellArray->InsertNextCell(size);
-      for (vtkIdType cc = 0; cc < size; ++cc)
-      {
-        cellArray->InsertCellPoint(e.Get(offset + cc, 0));
-      }
-    }
-  }
-};
-VTK_ABI_NAMESPACE_END
-}
-
-VTK_ABI_NAMESPACE_BEGIN
-
 //----------------------------------------------------------------------------
 vtkSmartPointer<vtkCellArray> vtkConduitArrayUtilities::O2MRelationToVTKCellArray(
-  const conduit_node* c_o2mrelation, const std::string& leafname)
+  vtkIdType numberOfPoints, const conduit_node* c_o2mrelation)
 {
   const conduit_cpp::Node o2mrelation =
     conduit_cpp::cpp_node(const_cast<conduit_node*>(c_o2mrelation));
-  const auto leaf = o2mrelation[leafname];
-  auto elements = vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
-    conduit_cpp::c_node(&leaf), /*force_signed*/ true);
+  const auto leaf = o2mrelation["connectivity"];
+
+  int8_t id;
+  bool working;
+  bool isDevicePointer = IsDevicePointer(leaf.element_ptr(0), id, working);
+  if (isDevicePointer && !working)
+  {
+    vtkLog(ERROR, "Viskores does not support device" + vtk::to_string(id));
+    return nullptr;
+  }
+
+  auto elements = vtkConduitArrayUtilities::MCArrayToVTKArray(conduit_cpp::c_node(&leaf));
   if (!elements)
   {
     return nullptr;
@@ -558,26 +701,68 @@ vtkSmartPointer<vtkCellArray> vtkConduitArrayUtilities::O2MRelationToVTKCellArra
     vtkLogF(WARNING, "'indices' in a O2MRelation are currently ignored.");
   }
 
-  const auto node_sizes = o2mrelation["sizes"];
-  auto sizes = vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
-    conduit_cpp::c_node(&node_sizes), /*force_signed*/ true);
   const auto node_offsets = o2mrelation["offsets"];
-  auto offsets = vtkConduitArrayUtilities::MCArrayToVTKArrayImpl(
-    conduit_cpp::c_node(&node_offsets), /*force_signed*/ true);
-
-  O2MRelationToVTKCellArrayWorker worker;
-
-  // Using a reduced type list for typical id types.
-  using TypeList =
-    vtkTypeList::Unique<vtkTypeList::Create<vtkTypeInt32, vtkTypeInt64, vtkIdType>>::Result;
-
-  using Dispatcher = vtkArrayDispatch::Dispatch3ByValueType<TypeList, TypeList, TypeList>;
-  if (!Dispatcher::Execute(elements.GetPointer(), sizes.GetPointer(), offsets.GetPointer(), worker))
+  auto offsets = vtkConduitArrayUtilities::MCArrayToVTKArray(conduit_cpp::c_node(&node_offsets));
+  vtkNew<vtkCellArray> cellArray;
+  if (isDevicePointer)
   {
-    worker(elements.GetPointer(), sizes.GetPointer(), offsets.GetPointer());
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+    const auto node_shapes = o2mrelation["shapes"];
+    auto shapes = vtkConduitArrayUtilities::MCArrayToVTKArray(conduit_cpp::c_node(&node_shapes));
+    if (!vtkConduitArrayUtilitiesDevice::IfVTKmConvertVTKMixedCellArray(
+          numberOfPoints, offsets, shapes, elements, cellArray))
+    {
+      vtkLogF(ERROR, "Cannot convert connectivity to a vtkmArray");
+      return nullptr;
+    }
+#else
+    (void)numberOfPoints; // avoid unused variable warning
+#endif
   }
+  else
+  {
+    // offsets and connectivity are in host memory
+    using ConduitDispatcher =
+      vtkArrayDispatch::Dispatch2BySameValueType<vtkArrayDispatch::Integrals>;
+    internals::FromHostConduitToMixedCellArray hostWorker{ cellArray };
+    if (!ConduitDispatcher::Execute(offsets, elements, hostWorker))
+    {
+      vtkLogF(ERROR, "offsets and elements do not have int values.");
+      return nullptr;
+    }
+  }
+  return cellArray;
+}
 
-  return worker.Cells;
+/**
+ * Returns true if the pointer is in device memory. In that case
+ * id is the DeviceAdapterTag
+ */
+bool vtkConduitArrayUtilities::IsDevicePointer(const void* ptr, int8_t& id)
+{
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+  return vtkmDataArrayUtilities::IsDevicePointer(ptr, id);
+#else
+  (void)id;
+  (void)ptr;
+  return false;
+#endif
+}
+
+bool vtkConduitArrayUtilities::IsDevicePointer(const void* ptr, int8_t& id, bool& working)
+{
+  void* pointer = const_cast<void*>(ptr);
+#if VTK_MODULE_ENABLE_VTK_AcceleratorsVTKmDataModel
+  bool isDevicePointer = vtkConduitArrayUtilities::IsDevicePointer(pointer, id);
+  // we process host pointers using VTK which is always available
+  working = isDevicePointer ? vtkmDataArrayUtilities::IsDeviceAdapterAvailable(id) : true;
+#else
+  bool isDevicePointer = vtkConduitArrayUtilities::IsDevicePointer(pointer, id);
+  // no Viskores, so for a device pointer there is no runtime
+  // for host pointer VTK can handle that.
+  working = (isDevicePointer) ? false : true;
+#endif
+  return isDevicePointer;
 }
 
 //----------------------------------------------------------------------------
