@@ -6,7 +6,9 @@
 #include "vtkHDF5ScopedHandle.h"
 #include "vtkHDFVersion.h"
 #include "vtkLogger.h"
+#include "vtkStringFormatter.h"
 #include "vtkType.h"
+
 #include "vtk_hdf5.h"
 
 #include <algorithm>
@@ -473,11 +475,11 @@ vtkHDF::ScopedH5DHandle vtkHDFWriter::Implementation::CreateDatasetFromDataArray
     vtkErrorWithObjectMacro(this->Writer, "Could not create Dataset");
     return H5I_INVALID_HID;
   }
-  // Get the data pointer
-  void* data = dataArray->GetVoidPointer(0);
-  // If there is no data pointer, return either an invalid id or the dataset depending on the number
+
+  auto da = vtkDataArray::FastDownCast(dataArray);
+  // If it is not a vtkDataArray, return either an invalid id or the dataset depending on the number
   // of values in the dataArray
-  if (data == nullptr)
+  if (da == nullptr)
   {
     if (dataArray->GetNumberOfValues() == 0)
     {
@@ -499,7 +501,8 @@ vtkHDF::ScopedH5DHandle vtkHDFWriter::Implementation::CreateDatasetFromDataArray
     return H5I_INVALID_HID;
   }
   // Write vtkAbstractArray data to the HDF dataset
-  if (H5Dwrite(dataset, source_type, H5S_ALL, dataspace, H5P_DEFAULT, data) < 0)
+  auto aos = da->ToAOSDataArray(); // NOLINTNEXTLINE(bugprone-unsafe-functions)
+  if (H5Dwrite(dataset, source_type, H5S_ALL, dataspace, H5P_DEFAULT, aos->GetVoidPointer(0)) < 0)
   {
     vtkErrorWithObjectMacro(this->Writer, "Could not write dataset " << name);
     return H5I_INVALID_HID;
@@ -522,7 +525,7 @@ bool vtkHDFWriter::Implementation::AddSingleValueToDataset(
 {
   vtkDebugWithObjectMacro(this->Writer, "Adding 1 value to " << this->GetGroupName(dataset));
   // Create a new dataspace containing a single value
-  const hsize_t addedDims[1] = { 1 };
+  constexpr hsize_t addedDims[1] = { 1 };
   vtkHDF::ScopedH5SHandle newDataspace = H5Screate_simple(1, addedDims, nullptr);
   if (newDataspace == H5I_INVALID_HID)
   {
@@ -583,7 +586,7 @@ bool vtkHDFWriter::Implementation::AddFieldDataSizeValueToDataset(
   if (size <= 1)
   {
     vtkLog(
-      WARNING, "Size given in this method shouldn't be less than 2, got : " + std::to_string(size));
+      WARNING, "Size given in this method shouldn't be less than 2, got : " + vtk::to_string(size));
     return false;
   }
 
@@ -710,8 +713,8 @@ bool vtkHDFWriter::Implementation::AddOrCreateFieldDataSizeValueDataset(
   {
     // Dataset needs to be created
     std::vector<hsize_t> dimensions{ 2 };
-    return this->CreateAndWriteHdfDataset(
-             group, H5T_STD_I64LE, H5T_STD_I64LE, name, 1, dimensions, &value) != H5I_INVALID_HID;
+    return this->CreateAndWriteHdfDataset(group, H5T_STD_I64LE, H5T_STD_I64LE, name, 1, dimensions,
+             &value) != H5I_INVALID_HID; // NOLINT(bugprone-multi-level-implicit-pointer-conversion)
   }
   else
   {
@@ -735,18 +738,9 @@ bool vtkHDFWriter::Implementation::AddArrayToDataset(
     return false;
   }
 
-  // Get raw array data
-  void* rawArrayData = dataArray ? dataArray->GetVoidPointer(0) : nullptr;
-  if (rawArrayData == nullptr)
+  if (dataArray->GetNumberOfValues() == 0)
   {
-    if (dataArray->GetNumberOfValues() == 0)
-    {
-      return true;
-    }
-    else
-    {
-      return false;
-    }
+    return true;
   }
 
   hid_t source_type =
@@ -812,22 +806,16 @@ bool vtkHDFWriter::Implementation::AddArrayToDataset(
     count.emplace_back(addedDims[1]);
   }
 
-  if (dataArray->GetDataType() == VTK_STRING)
+  if (auto sa = vtkStringArray::SafeDownCast(dataArray))
   {
-    auto vtkStrArray = vtkStringArray::SafeDownCast(dataArray);
-    if (vtkStrArray == nullptr)
-    {
-      return false;
-    }
-
     hid_t datatype = H5Tcopy(H5T_C_S1);
     H5Tset_size(datatype, H5T_VARIABLE);
 
     std::vector<const char*> strArray;
-    strArray.resize(vtkStrArray->GetNumberOfValues());
-    for (vtkIdType i = 0; i < vtkStrArray->GetNumberOfValues(); i++)
+    strArray.resize(sa->GetNumberOfValues());
+    for (vtkIdType i = 0; i < sa->GetNumberOfValues(); i++)
     {
-      strArray[i] = vtkStrArray->GetPointer(i)->c_str();
+      strArray[i] = sa->GetPointer(i)->c_str();
     }
 
     if (strArray.empty())
@@ -839,21 +827,29 @@ bool vtkHDFWriter::Implementation::AddArrayToDataset(
       currentDataspace, H5S_SELECT_SET, start.data(), nullptr, count.data(), nullptr);
 
     // Write new data to the dataset
+    // NOLINTNEXTLINE(bugprone-multi-level-implicit-pointer-conversion)
     if (H5Dwrite(dataset, datatype, dataspace, currentDataspace, H5P_DEFAULT, strArray.data()) < 0)
     {
       return false;
     }
   }
+  else if (auto da = vtkDataArray::SafeDownCast(dataArray))
+  {
+    H5Sselect_hyperslab(
+      currentDataspace, H5S_SELECT_SET, start.data(), nullptr, count.data(), nullptr);
+
+    // Write new data to the dataset
+    auto aos = da->ToAOSDataArray();
+    if (H5Dwrite(dataset, source_type, dataspace, currentDataspace, H5P_DEFAULT,
+          aos->GetVoidPointer(0)) < 0) // NOLINT(bugprone-unsafe-functions)
+    {
+      return false;
+    }
+  }
   else
   {
-    H5Sselect_hyperslab(
-      currentDataspace, H5S_SELECT_SET, start.data(), nullptr, count.data(), nullptr);
-
-    // Write new data to the dataset
-    if (H5Dwrite(dataset, source_type, dataspace, currentDataspace, H5P_DEFAULT, rawArrayData) < 0)
-    {
-      return false;
-    }
+    vtkErrorWithObjectMacro(this->Writer, << "Unsupported array type" << dataArray->GetClassName());
+    return false;
   }
 
   return true;
@@ -1268,7 +1264,6 @@ bool vtkHDFWriter::Implementation::WriteSumSteps(hid_t group, const char* name)
     "Creating steps sum " << name << " in " << this->GetGroupName(group) << "with base "
                           << this->GetBasePath(this->GetGroupName(group)));
 
-  const std::string datasetPath = this->GetGroupName(group) + "/" + name;
   vtkHDF::ScopedH5DHandle dataset = this->OpenDataset(group, name);
   if (dataset == H5I_INVALID_HID)
   {
@@ -1301,7 +1296,6 @@ bool vtkHDFWriter::Implementation::WriteSumStepsPolyData(hid_t group, const char
   vtkDebugWithObjectMacro(
     this->Writer, "Creating polydata steps sum " << name << " in " << this->GetGroupName(group));
 
-  const std::string datasetPath = this->GetGroupName(group) + "/" + name;
   vtkHDF::ScopedH5DHandle dataset = this->OpenDataset(group, name);
   if (dataset == H5I_INVALID_HID)
   {
