@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkDataReader.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkDataReader.h"
 
@@ -37,7 +25,6 @@
 #include "vtkInformationStringKey.h"
 #include "vtkInformationStringVectorKey.h"
 #include "vtkInformationUnsignedLongKey.h"
-#include "vtkInformationVector.h"
 #include "vtkIntArray.h"
 #include "vtkLegacyReaderVersion.h"
 #include "vtkLongArray.h"
@@ -47,8 +34,8 @@
 #include "vtkPointSet.h"
 #include "vtkRectilinearGrid.h"
 #include "vtkShortArray.h"
-#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStringArray.h"
+#include "vtkStringScanner.h"
 #include "vtkTable.h"
 #include "vtkTypeInt64Array.h"
 #include "vtkTypeUInt64Array.h"
@@ -71,7 +58,8 @@
 // myself.
 // This function is also defined in Infovis/vtkDelimitedTextReader.cxx,
 // so it would be nice to put this in a common file.
-static int my_getline(istream& in, vtkStdString& output, char delim = '\n');
+VTK_ABI_NAMESPACE_BEGIN
+static int my_getline(istream& in, std::string& output, char delim = '\n');
 
 vtkStandardNewMacro(vtkDataReader);
 
@@ -291,7 +279,7 @@ int vtkDataReader::ReadLine(char result[256])
 //------------------------------------------------------------------------------
 // Internal function to read in a string up to 256 characters.
 // Returns zero if there was an error.
-int vtkDataReader::ReadString(char result[256])
+int vtkDataReader::ReadString(char (&result)[256])
 {
   // Force the parameter to be seen as a 256-byte array rather than a decayed
   // pointer.
@@ -470,10 +458,6 @@ size_t vtkDataReader::Peek(char* str, size_t n)
 // Open a vtk data file. Returns zero if error.
 int vtkDataReader::OpenVTKFile(const char* fname)
 {
-  // Save current locale settings and set standard one to
-  // avoid locale issues - for instance with the decimal separator.
-  this->CurrentLocale = std::locale::global(std::locale::classic());
-
   if (!fname && this->GetNumberOfFileNames() > 0)
   {
     fname = this->GetFileName(0);
@@ -533,6 +517,12 @@ int vtkDataReader::OpenVTKFile(const char* fname)
       this->SetErrorCode(vtkErrorCode::CannotOpenFileError);
       return 0;
     }
+    // Save current locale settings and set standard one to
+    // avoid locale issues - for instance with the decimal separator.
+    // Assumption made here that each OpenVTKFile is matched with a
+    // CloseVTKFile and that no nested Open/Close calls are made.
+    this->OriginalStreamLocale = this->IS->getloc();
+    this->IS->imbue(std::locale::classic());
     return 1;
   }
 
@@ -560,7 +550,7 @@ int vtkDataReader::ReadHeader(const char* fname)
     this->SetErrorCode(vtkErrorCode::PrematureEndOfFileError);
     return 0;
   }
-  const int VERSION_PREFIX_LENGTH = 22;
+  constexpr int VERSION_PREFIX_LENGTH = 22;
   if (strncmp("# vtk DataFile Version", line, VERSION_PREFIX_LENGTH) != 0)
   {
     vtkErrorMacro(<< "Unrecognized file type: " << line
@@ -569,14 +559,15 @@ int vtkDataReader::ReadHeader(const char* fname)
     this->SetErrorCode(vtkErrorCode::UnrecognizedFileTypeError);
     return 0;
   }
-  if (sscanf(line + VERSION_PREFIX_LENGTH, "%d.%d", &this->FileMajorVersion,
-        &this->FileMinorVersion) != 2)
+  auto result = vtk::scan<int, int>(std::string_view(line + VERSION_PREFIX_LENGTH), "{:d}.{:d}");
+  if (!result)
   {
     vtkWarningMacro(<< "Cannot read file version: " << line
                     << " for file: " << (fname ? fname : "(Null FileName)"));
     this->FileMajorVersion = 0;
     this->FileMinorVersion = 0;
   }
+  std::tie(this->FileMajorVersion, this->FileMinorVersion) = result->values();
   if (this->FileMajorVersion > vtkLegacyReaderMajorVersion ||
     (this->FileMajorVersion == vtkLegacyReaderMajorVersion &&
       this->FileMinorVersion > vtkLegacyReaderMinorVersion))
@@ -1843,7 +1834,7 @@ vtkAbstractArray* vtkDataReader::ReadArray(
         {
           vtkTypeUInt8 firstByte;
           vtkTypeUInt8 headerType;
-          vtkStdString::size_type stringLength;
+          std::string::size_type stringLength;
           firstByte = IS->peek();
           headerType = firstByte >> 6;
           if (headerType == 3)
@@ -1880,7 +1871,7 @@ vtkAbstractArray* vtkDataReader::ReadArray(
           }
           std::vector<char> str(stringLength);
           IS->read(str.data(), stringLength);
-          vtkStdString s(str.data(), stringLength);
+          std::string s(str.data(), stringLength);
           ((vtkStringArray*)array)->InsertNextValue(s);
         }
       }
@@ -1888,7 +1879,7 @@ vtkAbstractArray* vtkDataReader::ReadArray(
     else
     {
       // read in newline
-      vtkStdString s;
+      std::string s;
       my_getline(*(this->IS), s);
 
       for (vtkIdType i = 0; i < numTuples; i++)
@@ -1899,7 +1890,7 @@ vtkAbstractArray* vtkDataReader::ReadArray(
           int length = static_cast<int>(s.length());
           std::vector<char> decoded(length + 1);
           int decodedLength = this->DecodeString(decoded.data(), s.c_str());
-          vtkStdString decodedStr(decoded.data(), decodedLength);
+          std::string decodedStr(decoded.data(), decodedLength);
           ((vtkStringArray*)array)->InsertNextValue(decodedStr);
         }
       }
@@ -1914,11 +1905,11 @@ vtkAbstractArray* vtkDataReader::ReadArray(
       for (vtkIdType j = 0; j < numComp; j++)
       {
         int t;
-        vtkStdString str;
+        std::string str;
         *(this->IS) >> t >> str;
         std::vector<char> decoded(str.length() + 1);
         int decodedLength = this->DecodeString(decoded.data(), str.c_str());
-        vtkStdString decodedStr(decoded.data(), decodedLength);
+        std::string decodedStr(decoded.data(), decodedLength);
         vtkVariant sv(decodedStr);
         vtkVariant v;
         switch (t)
@@ -2050,15 +2041,16 @@ vtkAbstractArray* vtkDataReader::ReadArray(
 
   while (this->ReadLine(line))
   {
+    std::string_view lineView(line);
     this->LowerCase(line, 256);
 
     // Blank line indicates end of metadata:
-    if (strlen(line) == 0)
+    if (lineView.empty())
     {
       break;
     }
 
-    if (strncmp(line, "component_names", 15) == 0)
+    if (lineView.substr(0, 15) == "component_names")
     {
       char decoded[256];
       for (int i = 0; i < numComp; ++i)
@@ -2076,14 +2068,15 @@ vtkAbstractArray* vtkDataReader::ReadArray(
       continue;
     }
 
-    if (strncmp(line, "information", 11) == 0)
+    if (lineView.substr(0, 11) == "information")
     {
-      int numKeys;
-      if (sscanf(line, "information %d", &numKeys) != 1)
+      auto result = vtk::scan<int>(lineView, "information {:d}");
+      if (!result)
       {
         vtkWarningMacro("Invalid information header: " << line);
         continue;
       }
+      const int numKeys = result->value();
 
       vtkInformation* info = array->GetInformation();
       this->ReadInformation(info, numKeys);
@@ -2241,7 +2234,7 @@ int vtkDataReader::ReadScalarData(vtkDataSetAttributes* a, vtkIdType numPts)
   // the next string could be an integer number of components or a lookup table
   if (strcmp(this->LowerCase(key), "lookup_table") != 0)
   {
-    numComp = atoi(key);
+    VTK_FROM_CHARS_IF_ERROR_RETURN(key, numComp, 0);
     if (numComp < 1 || !this->ReadString(key))
     {
       const char* fname = this->CurrentFileName.c_str();
@@ -2763,8 +2756,6 @@ int vtkDataReader::ReadInformation(vtkInformation* info, vtkIdType numKeys)
 {
   // Assuming that the opening INFORMATION line has been read.
   char line[256];
-  char name[256];
-  char location[256];
   for (int keyIdx = 0; keyIdx < numKeys; ++keyIdx)
   {
     do
@@ -2778,11 +2769,14 @@ int vtkDataReader::ReadInformation(vtkInformation* info, vtkIdType numKeys)
 
     if (strncmp("NAME ", line, 5) == 0)
     { // New key
-      if (sscanf(line, "NAME %s LOCATION %s", name, location) != 2)
+      auto result =
+        vtk::scan<std::string, std::string>(std::string_view(line), "NAME {:s} LOCATION {:s}");
+      if (!result)
       {
         vtkWarningMacro("Invalid line in information specification: " << line);
         continue;
       }
+      auto& [name, location] = result->values();
 
       vtkInformationKey* key = vtkInformationKeyLookup::Find(name, location);
       if (!key)
@@ -2844,7 +2838,7 @@ int vtkDataReader::ReadInformation(vtkInformation* info, vtkIdType numKeys)
         }
         if (values.size() == static_cast<size_t>(length))
         {
-          info->Set(dvKey, &values[0], length);
+          info->Set(dvKey, values.data(), length);
         }
 
         // Pop off the trailing newline:
@@ -2911,7 +2905,7 @@ int vtkDataReader::ReadInformation(vtkInformation* info, vtkIdType numKeys)
         }
         if (values.size() == static_cast<size_t>(length))
         {
-          info->Set(ivKey, &values[0], length);
+          info->Set(ivKey, values.data(), length);
         }
 
         // Pop off the trailing newline:
@@ -2927,15 +2921,16 @@ int vtkDataReader::ReadInformation(vtkInformation* info, vtkIdType numKeys)
           continue;
         }
 
-        char value[256];
-        if (sscanf(line, "DATA %s", value) != 1)
+        auto resultData = vtk::scan<std::string>(std::string_view(line), "DATA {:s}");
+        if (!resultData)
         {
           vtkWarningMacro("Malformed data block for key " << location << "::" << name << ".");
           continue;
         }
+        auto& value = resultData->value();
 
         char decoded[256];
-        this->DecodeString(decoded, value);
+        this->DecodeString(decoded, value.c_str());
 
         info->Set(sKey, decoded);
       }
@@ -3420,7 +3415,7 @@ vtkFieldData* vtkDataReader::ReadFieldData(FieldType fieldType)
 }
 
 //------------------------------------------------------------------------------
-char* vtkDataReader::LowerCase(char* str, const size_t len)
+char* vtkDataReader::LowerCase(char* str, size_t len)
 {
   size_t i;
   char* s;
@@ -3438,7 +3433,11 @@ void vtkDataReader::CloseVTKFile()
   vtkDebugMacro(<< "Closing vtk file");
 
   // Restore the previous locale settings
-  std::locale::global(this->CurrentLocale);
+  // The stream is deleted below, so this is done just to be tidy.
+  if (this->IS != nullptr)
+  {
+    this->IS->imbue(this->OriginalStreamLocale);
+  }
 
   delete this->IS;
   this->IS = nullptr;
@@ -3596,12 +3595,17 @@ void vtkDataReader::CheckFor(const char* name, char* line, int& num, char**& arr
     }
 
     // enter the name
-    char nameOfAttribute[256];
-    sscanf(line, "%*s %s", nameOfAttribute);
-    if (*nameOfAttribute)
+    auto result =
+      vtk::scan<std::string_view, std::string_view>(std::string_view(line), "{:s} {:s}");
+    if (result)
     {
-      array[num - 1] = new char[strlen(nameOfAttribute) + 1];
-      strcpy(array[num - 1], nameOfAttribute);
+      auto& [_, nameOfAttribute] = result->values();
+      if (!nameOfAttribute.empty())
+      {
+        array[num] = new char[nameOfAttribute.size() + 1];
+        std::copy_n(nameOfAttribute.data(), nameOfAttribute.size(), array[num - 1]);
+        array[num - 1][nameOfAttribute.size()] = '\0';
+      }
     }
   } // found one
 }
@@ -3835,7 +3839,8 @@ int vtkDataReader::DecodeString(char* resname, const char* name)
         buffer[2] = name[cc + 1];
         buffer[3] = name[cc + 2];
         buffer[4] = 0;
-        sscanf(buffer, "%x", &ch);
+        auto result = vtk::scan<unsigned int>(std::string_view(buffer), "{:x}");
+        ch = result->value();
         str << static_cast<char>(ch);
         cc += 2;
         reslen++;
@@ -3853,9 +3858,9 @@ int vtkDataReader::DecodeString(char* resname, const char* name)
   return static_cast<int>(reslen);
 }
 
-static int my_getline(istream& in, vtkStdString& out, char delimiter)
+static int my_getline(istream& in, std::string& out, char delimiter)
 {
-  out = vtkStdString();
+  out = std::string();
   unsigned int numCharactersRead = 0;
   int nextValue = 0;
 
@@ -3902,3 +3907,4 @@ void vtkDataReader::SetScalarLut(const char* lut)
     } while (--n);
   }
 }
+VTK_ABI_NAMESPACE_END
