@@ -1,16 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkSurfaceLICInterface.h"
 
 #include "vtkFloatArray.h"
@@ -31,7 +20,6 @@
 #include "vtkScalarsToColors.h"
 #include "vtkShaderProgram.h"
 #include "vtkSurfaceLICComposite.h"
-#include "vtkTextureObjectVS.h"
 
 #include "vtkOpenGLIndexBufferObject.h"
 #include "vtkOpenGLVertexArrayObject.h"
@@ -60,16 +48,22 @@ typedef vtkLineIntegralConvolution2D vtkLIC2D;
 #if vtkSurfaceLICInterfaceDEBUG >= 2
 #include "vtkTextureIO.h"
 #include <sstream>
+
+#include <iostream>
+
 using std::ostringstream;
 //------------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
 static std::string mpifn(vtkPainterCommunicator* comm, const char* fn)
 {
   ostringstream oss;
   oss << comm->GetRank() << "_" << fn;
   return oss.str();
 }
+VTK_ABI_NAMESPACE_END
 #endif
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkObjectFactoryNewMacro(vtkSurfaceLICInterface);
 
 //------------------------------------------------------------------------------
@@ -121,7 +115,7 @@ vtkSurfaceLICInterface::vtkSurfaceLICInterface()
 vtkSurfaceLICInterface::~vtkSurfaceLICInterface()
 {
 #if vtkSurfaceLICInterfaceDEBUG >= 1
-  cerr << "=====vtkSurfaceLICInterface::~vtkSurfaceLICInterface" << endl;
+  std::cerr << "=====vtkSurfaceLICInterface::~vtkSurfaceLICInterface" << endl;
 #endif
   this->ReleaseGraphicsResources(this->Internals->Context);
   delete this->Internals;
@@ -917,7 +911,7 @@ bool vtkSurfaceLICInterface::CanRenderSurfaceLIC(vtkActor* actor)
   }
 
 #if vtkSurfaceLICInterfaceDEBUG >= 1
-  cerr << this->Internals->Communicator->GetWorldRank() << " CanRender " << canRender << endl;
+  std::cerr << this->Internals->Communicator->GetWorldRank() << " CanRender " << canRender << endl;
 #endif
 
   return canRender;
@@ -1032,8 +1026,8 @@ bool vtkSurfaceLICInterface::NeedToUpdateCommunicator()
   }
 
 #if vtkSurfaceLICInterfaceDEBUG >= 1
-  cerr << this->Internals->Communicator->GetWorldRank() << " NeedToUpdateCommunicator "
-       << this->Internals->CommunicatorNeedsUpdate << endl;
+  std::cerr << this->Internals->Communicator->GetWorldRank() << " NeedToUpdateCommunicator "
+            << this->Internals->CommunicatorNeedsUpdate << endl;
 #endif
 
   return this->Internals->CommunicatorNeedsUpdate;
@@ -1080,8 +1074,8 @@ void vtkSurfaceLICInterface::ValidateContext(vtkRenderer* renderer)
   }
 
 #if vtkSurfaceLICInterfaceDEBUG >= 1
-  cerr << this->Internals->Communicator->GetWorldRank() << " NeedToUpdatContext " << modified
-       << endl;
+  std::cerr << this->Internals->Communicator->GetWorldRank() << " NeedToUpdatContext " << modified
+            << endl;
 #endif
 }
 
@@ -1124,7 +1118,8 @@ void vtkSurfaceLICInterface::CreateCommunicator(
   this->Internals->Communicator = this->CreateCommunicator(includeRank);
 
 #if vtkSurfaceLICInterfaceDEBUG >= 1
-  cerr << this->Internals->Communicator->GetWorldRank() << " is rendering " << includeRank << endl;
+  std::cerr << this->Internals->Communicator->GetWorldRank() << " is rendering " << includeRank
+            << endl;
 #endif
 }
 
@@ -1173,3 +1168,4 @@ void vtkSurfaceLICInterface::PrintSelf(ostream& os, vtkIndent indent)
      << indent << "AlwaysUpdate=" << this->AlwaysUpdate << endl
      << indent << "CompositeStrategy=" << this->CompositeStrategy << endl;
 }
+VTK_ABI_NAMESPACE_END
