@@ -1,26 +1,15 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkLabeledDataMapper.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkLabeledDataMapper.h"
 
 #include "vtkActor2D.h"
+#include "vtkArrayDispatch.h"
 #include "vtkCompositeDataIterator.h"
 #include "vtkCompositeDataSet.h"
 #include "vtkDataArray.h"
+#include "vtkDataArrayRange.h"
 #include "vtkDataSet.h"
-#include "vtkExecutive.h"
 #include "vtkInformation.h"
 #include "vtkIntArray.h"
 #include "vtkObjectFactory.h"
@@ -29,14 +18,14 @@
 #include "vtkPointSet.h"
 #include "vtkSmartPointer.h"
 #include "vtkStringArray.h"
-#include "vtkTable.h"
+#include "vtkStringFormatter.h"
 #include "vtkTextMapper.h"
 #include "vtkTextProperty.h"
 #include "vtkTransform.h"
-#include "vtkTypeTraits.h"
 
 #include <map>
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkLabeledDataMapper::Internals
 {
 public:
@@ -47,15 +36,6 @@ vtkStandardNewMacro(vtkLabeledDataMapper);
 
 vtkCxxSetObjectMacro(vtkLabeledDataMapper, Transform, vtkTransform);
 
-//------------------------------------------------------------------------------
-
-template <typename T>
-void vtkLabeledDataMapper_PrintComponent(
-  char* output, size_t outputSize, const char* format, int index, const T* array)
-{
-  snprintf(output, outputSize, format, array[index]);
-}
-
 //------------------------------------------------------------------------------
 // Creates a new label mapper
 
@@ -348,10 +328,136 @@ void vtkLabeledDataMapper::BuildLabels()
   this->BuildTime.Modified();
 }
 
+//------------------------------------------------------------------------------
+struct vtkLabeledDataMapper::vtkLabeledDataMapperFunctor
+{
+  vtkLabeledDataMapper* Self;
+  vtkIntArray* TypeArr;
+  vtkDataSet* Input;
+  int NumCurLabels;
+
+  vtkLabeledDataMapperFunctor(
+    vtkLabeledDataMapper* self, vtkIntArray* typeArr, vtkDataSet* input, int numCurLabels)
+    : Self(self)
+    , TypeArr(typeArr)
+    , Input(input)
+    , NumCurLabels(numCurLabels)
+  {
+  }
+
+  void SetFormattedString(int i, const char* resultString)
+  {
+    this->Self->TextMappers[i + this->Self->NumberOfLabels]->SetInput(resultString);
+
+    // Find the correct property type
+    int type = 0;
+    if (this->TypeArr)
+    {
+      type = this->TypeArr->GetValue(i);
+    }
+    vtkTextProperty* prop = this->Self->Implementation->TextProperties[type];
+    if (!prop)
+    {
+      prop = this->Self->Implementation->TextProperties[0];
+    }
+    this->Self->TextMappers[i + this->Self->NumberOfLabels]->SetTextProperty(prop);
+
+    double x[3];
+    this->Input->GetPoint(i, x);
+    this->Self->LabelPositions[3 * (i + this->Self->NumberOfLabels)] = x[0];
+    this->Self->LabelPositions[3 * (i + this->Self->NumberOfLabels) + 1] = x[1];
+    this->Self->LabelPositions[3 * (i + this->Self->NumberOfLabels) + 2] = x[2];
+  }
+
+  void operator()(const std::string& FormatString)
+  {
+    char formatedString[1024];
+    for (int i = 0; i < this->NumCurLabels; i++)
+    {
+      auto result = vtk::format_to_n(formatedString, sizeof(formatedString), FormatString, i);
+      *result.out = '\0';
+      this->SetFormattedString(i, formatedString);
+    }
+  }
+
+  struct NumericComponent
+  {
+  };
+  template <class TArray>
+  void operator()(TArray* array, int activeComp, const std::string& FormatString, NumericComponent)
+  {
+    char formatedString[1024];
+    auto a = vtk::DataArrayTupleRange(array);
+    using ValueType = vtk::GetAPIType<TArray>;
+    for (int i = 0; i < this->NumCurLabels; i++)
+    {
+      auto result = vtk::format_to_n(formatedString, sizeof(formatedString), FormatString,
+        static_cast<ValueType>(a[i][activeComp]));
+      *result.out = '\0';
+      this->SetFormattedString(i, formatedString);
+    }
+  }
+
+  struct NumericVector
+  {
+  };
+  template <class TArray>
+  void operator()(TArray* array, int numComp, const std::string& FormatString, NumericVector)
+  {
+    char formatedString[1024];
+    std::string ResultString;
+    auto a = vtk::DataArrayTupleRange(array);
+    using ValueType = vtk::GetAPIType<TArray>;
+    for (int i = 0; i < this->NumCurLabels; i++)
+    {
+      ResultString = "(";
+
+      // Print each component in turn and add it to the string.
+      for (int j = 0; j < numComp; ++j)
+      {
+        auto result = vtk::format_to_n(
+          formatedString, sizeof(formatedString), FormatString, static_cast<ValueType>(a[i][j]));
+        *result.out = '\0';
+
+        ResultString += formatedString;
+        if (j < (numComp - 1))
+        {
+          ResultString += this->Self->GetComponentSeparator();
+        }
+        else
+        {
+          ResultString += ')';
+        }
+      }
+      this->SetFormattedString(i, ResultString.c_str());
+    }
+  }
+
+  void operator()(vtkStringArray* array, const std::string& FormatString)
+  {
+    char formatedString[1024];
+    for (int i = 0; i < this->NumCurLabels; i++)
+    {
+      // If the user hasn't given us a custom format string then just save the value.
+      if (!this->Self->LabelFormat || std::string_view(this->Self->LabelFormat).empty())
+      {
+        this->SetFormattedString(i, array->GetValue(i).c_str());
+      }
+      else // the user specified a label format
+      {
+        auto result = vtk::format_to_n(formatedString, sizeof(formatedString), FormatString,
+          static_cast<std::string&>(array->GetValue(i)));
+        *result.out = '\0';
+        this->SetFormattedString(i, formatedString);
+      }
+    }
+  }
+};
+
 //------------------------------------------------------------------------------
 void vtkLabeledDataMapper::BuildLabelsInternal(vtkDataSet* input)
 {
-  int i, j, numComp = 0, pointIdLabels = 0, activeComp = 0;
+  int numComp = 0, pointIdLabels = 0, activeComp = 0;
   vtkAbstractArray* abstractData = nullptr;
   vtkDataArray* numericData = nullptr;
   vtkStringArray* stringData = nullptr;
@@ -362,6 +468,10 @@ void vtkLabeledDataMapper::BuildLabelsInternal(vtkDataSet* input)
   }
 
   vtkPointData* pd = input->GetPointData();
+
+  vtkIntArray* typeArr =
+    vtkArrayDownCast<vtkIntArray>(this->GetInputAbstractArrayToProcess(0, input));
+
   // figure out what to label, and if we can label it
   pointIdLabels = 0;
   switch (this->LabelMode)
@@ -369,7 +479,7 @@ void vtkLabeledDataMapper::BuildLabelsInternal(vtkDataSet* input)
     case VTK_LABEL_IDS:
     {
       pointIdLabels = 1;
-    };
+    }
     break;
     case VTK_LABEL_SCALARS:
       if (pd->GetScalars())
@@ -417,7 +527,7 @@ void vtkLabeledDataMapper::BuildLabelsInternal(vtkDataSet* input)
       }
       numericData = vtkArrayDownCast<vtkDataArray>(abstractData);
       stringData = vtkArrayDownCast<vtkStringArray>(abstractData);
-    };
+    }
     break;
   }
 
@@ -460,85 +570,65 @@ void vtkLabeledDataMapper::BuildLabelsInternal(vtkDataSet* input)
     }
   }
 
-  vtkStdString FormatString;
-  if (this->LabelFormat)
+  std::string formatString;
+  if (this->LabelFormat && !std::string_view(this->LabelFormat).empty())
   {
     // The user has specified a format string.
     vtkDebugMacro(<< "Using user-specified format string " << this->LabelFormat);
-    FormatString = this->LabelFormat;
+    formatString = vtk::to_std_format(this->LabelFormat);
   }
   else
   {
     // Try to come up with some sane default.
     if (pointIdLabels)
     {
-      FormatString = "%d";
+      formatString = "{:d}";
     }
     else if (numericData)
     {
       switch (numericData->GetDataType())
       {
         case VTK_VOID:
-          FormatString = "0x%x";
+          formatString = "0x{:x}";
           break;
-
-          // don't use vtkTypeTraits::ParseFormat for character types as parse formats
-          // aren't the same as print formats for these types.
         case VTK_BIT:
-        case VTK_SIGNED_CHAR:
-        case VTK_UNSIGNED_CHAR:
         case VTK_SHORT:
         case VTK_UNSIGNED_SHORT:
         case VTK_INT:
         case VTK_UNSIGNED_INT:
-          FormatString = "%d";
+          formatString = "{:d}";
           break;
-
         case VTK_CHAR:
-          FormatString = "%c";
+        case VTK_SIGNED_CHAR:
+        case VTK_UNSIGNED_CHAR:
+          formatString = "{:c}";
           break;
-
         case VTK_LONG:
-          FormatString = vtkTypeTraits<long>::ParseFormat();
-          break;
         case VTK_UNSIGNED_LONG:
-          FormatString = vtkTypeTraits<unsigned long>::ParseFormat();
-          break;
-
         case VTK_ID_TYPE:
-          FormatString = vtkTypeTraits<vtkIdType>::ParseFormat();
-          break;
-
         case VTK_LONG_LONG:
-          FormatString = vtkTypeTraits<long long>::ParseFormat();
-          break;
         case VTK_UNSIGNED_LONG_LONG:
-          FormatString = vtkTypeTraits<unsigned long long>::ParseFormat();
+          formatString = "{:d}";
           break;
-
         case VTK_FLOAT:
-          FormatString = vtkTypeTraits<float>::ParseFormat();
-          break;
-
         case VTK_DOUBLE:
-          FormatString = vtkTypeTraits<double>::ParseFormat();
+          formatString = "{:f}";
           break;
-
         default:
-          FormatString = "BUG - UNKNOWN DATA FORMAT";
+          formatString = "BUG - UNKNOWN DATA FORMAT";
           break;
       }
     }
     else if (stringData)
     {
-      FormatString = "";
+      formatString = "";
     }
     else
     {
-      FormatString = "BUG - COULDN'T DETECT DATA TYPE";
+      formatString = "BUG - COULDN'T DETECT DATA TYPE";
     }
 
-    vtkDebugMacro(<< "Using default format string " << FormatString.c_str());
+    vtkDebugMacro(<< "Using default format string " << formatString);
 
   } // Done building default format string
 
@@ -555,96 +645,34 @@ void vtkLabeledDataMapper::BuildLabelsInternal(vtkDataSet* input)
   // Now we actually construct the label strings
   //
 
-  const char* LiveFormatString = FormatString.c_str();
-  char TempString[1024];
-
-  vtkIntArray* typeArr =
-    vtkArrayDownCast<vtkIntArray>(this->GetInputAbstractArrayToProcess(0, input));
-  for (i = 0; i < numCurLabels; i++)
+  vtkLabeledDataMapperFunctor functor(this, typeArr, input, numCurLabels);
+  if (pointIdLabels)
   {
-    vtkStdString ResultString;
-
-    if (pointIdLabels)
+    functor(formatString);
+  }
+  else if (numericData)
+  {
+    if (numComp == 1)
     {
-      snprintf(TempString, sizeof(TempString), LiveFormatString, i);
-      ResultString = TempString;
+      if (!vtkArrayDispatch::Dispatch::Execute(numericData, functor, activeComp, formatString,
+            vtkLabeledDataMapperFunctor::NumericComponent()))
+      {
+        functor(
+          numericData, activeComp, formatString, vtkLabeledDataMapperFunctor::NumericComponent());
+      }
     }
     else
     {
-      if (numericData)
+      if (!vtkArrayDispatch::Dispatch::Execute(numericData, functor, numComp, formatString,
+            vtkLabeledDataMapperFunctor::NumericVector()))
       {
-        void* rawData = numericData->GetVoidPointer(i * numComp);
-
-        if (numComp == 1)
-        {
-          switch (numericData->GetDataType())
-          {
-            vtkTemplateMacro(vtkLabeledDataMapper_PrintComponent(TempString, sizeof(TempString),
-              LiveFormatString, activeComp, static_cast<VTK_TT*>(rawData)));
-          }
-          ResultString = TempString;
-        }
-        else // numComp != 1
-        {
-          ResultString = "(";
-
-          // Print each component in turn and add it to the string.
-          for (j = 0; j < numComp; ++j)
-          {
-            switch (numericData->GetDataType())
-            {
-              vtkTemplateMacro(vtkLabeledDataMapper_PrintComponent(TempString, sizeof(TempString),
-                LiveFormatString, j, static_cast<VTK_TT*>(rawData)));
-            }
-            ResultString += TempString;
-
-            if (j < (numComp - 1))
-            {
-              ResultString += this->GetComponentSeparator();
-            }
-            else
-            {
-              ResultString += ')';
-            }
-          }
-        }
+        functor(numericData, numComp, formatString, vtkLabeledDataMapperFunctor::NumericVector());
       }
-      else // rendering string data
-      {
-        // If the user hasn't given us a custom format string then
-        // we'll sidestep a lot of snprintf nonsense.
-        if (this->LabelFormat == nullptr)
-        {
-          ResultString = stringData->GetValue(i);
-        }
-        else // the user specified a label format
-        {
-          snprintf(TempString, 1023, LiveFormatString, stringData->GetValue(i).c_str());
-          ResultString = TempString;
-        } // done printing strings with label format
-      }   // done printing strings
-    }     // done creating string
-
-    this->TextMappers[i + this->NumberOfLabels]->SetInput(ResultString.c_str());
-
-    // Find the correct property type
-    int type = 0;
-    if (typeArr)
-    {
-      type = typeArr->GetValue(i);
     }
-    vtkTextProperty* prop = this->Implementation->TextProperties[type];
-    if (!prop)
-    {
-      prop = this->Implementation->TextProperties[0];
-    }
-    this->TextMappers[i + this->NumberOfLabels]->SetTextProperty(prop);
-
-    double x[3];
-    input->GetPoint(i, x);
-    this->LabelPositions[3 * (i + this->NumberOfLabels)] = x[0];
-    this->LabelPositions[3 * (i + this->NumberOfLabels) + 1] = x[1];
-    this->LabelPositions[3 * (i + this->NumberOfLabels) + 2] = x[2];
+  }
+  else // rendering string data
+  {
+    functor(stringData, formatString);
   }
 
   this->NumberOfLabels += numCurLabels;
@@ -744,24 +772,6 @@ void vtkLabeledDataMapper::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "CoordinateSystem: " << this->CoordinateSystem << endl;
 }
 
-//------------------------------------------------------------------------------
-void vtkLabeledDataMapper::SetFieldDataArray(int arrayIndex)
-{
-  delete[] this->FieldDataName;
-  this->FieldDataName = nullptr;
-
-  vtkDebugMacro(<< this->GetClassName() << " (" << this << "): setting FieldDataArray to "
-                << arrayIndex);
-
-  if (this->FieldDataArray !=
-    (arrayIndex < 0 ? 0 : (arrayIndex > VTK_INT_MAX ? VTK_INT_MAX : arrayIndex)))
-  {
-    this->FieldDataArray =
-      (arrayIndex < 0 ? 0 : (arrayIndex > VTK_INT_MAX ? VTK_INT_MAX : arrayIndex));
-    this->Modified();
-  }
-}
-
 //------------------------------------------------------------------------------
 vtkMTimeType vtkLabeledDataMapper::GetMTime()
 {
@@ -773,10 +783,7 @@ vtkMTimeType vtkLabeledDataMapper::GetMTime()
   {
     vtkTextProperty* p = it->second;
     vtkMTimeType curMTime = p->GetMTime();
-    if (curMTime > mtime)
-    {
-      mtime = curMTime;
-    }
+    mtime = std::max(curMTime, mtime);
   }
   return mtime;
 }
@@ -787,31 +794,4 @@ const char* vtkLabeledDataMapper::GetLabelText(int label)
   assert("label index range" && label >= 0 && label < this->NumberOfLabels);
   return this->TextMappers[label]->GetInput();
 }
-
-//------------------------------------------------------------------------------
-void vtkLabeledDataMapper::SetFieldDataName(const char* arrayName)
-{
-  vtkDebugMacro(<< this->GetClassName() << " (" << this << "): setting "
-                << "FieldDataName"
-                << " to " << (arrayName ? arrayName : "(null)"));
-
-  if (this->FieldDataName == nullptr && arrayName == nullptr)
-  {
-    return;
-  }
-  if (this->FieldDataName && arrayName && (!strcmp(this->FieldDataName, arrayName)))
-  {
-    return;
-  }
-  delete[] this->FieldDataName;
-  if (arrayName)
-  {
-    this->FieldDataName = new char[strlen(arrayName) + 1];
-    strcpy(this->FieldDataName, arrayName);
-  }
-  else
-  {
-    this->FieldDataName = nullptr;
-  }
-  this->Modified();
-}
+VTK_ABI_NAMESPACE_END
