@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    TestGPURayCastVolumeUpdate.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 // This test volume tests whether updating the volume MTime updates the ,
 // geometry in the volume mapper.
 
@@ -35,6 +23,8 @@
 #include <vtkVolumeProperty.h>
 #include <vtkXMLImageDataReader.h>
 
+#include <iostream>
+
 static const char* TestGPURayCastVolumeUpdateLog = "# StreamVersion 1\n"
                                                    "EnterEvent 396 237 0 0 0 0 0\n"
                                                    "LeftButtonPressEvent 223 255 0 0 0 0 0\n"
@@ -208,7 +198,7 @@ static const char* TestGPURayCastVolumeUpdateLog = "# StreamVersion 1\n"
 
 int TestGPURayCastVolumeUpdate(int argc, char* argv[])
 {
-  cout << "CTEST_FULL_OUTPUT (Avoid ctest truncation of output)" << endl;
+  std::cout << "CTEST_FULL_OUTPUT (Avoid ctest truncation of output)" << std::endl;
   bool useOSP = true;
   for (int i = 0; i < argc; i++)
   {
@@ -319,7 +309,7 @@ int TestGPURayCastVolumeUpdate(int argc, char* argv[])
   else
   {
     retVal = vtkTesting::PASSED;
-    cout << "Required extensions not supported" << endl;
+    std::cout << "Required extensions not supported" << std::endl;
   }
 
   return !retVal;
