@@ -5,6 +5,7 @@
 #include "vtkImageData.h"
 #include "vtkRenderWindowInteractor.h"
 #include "vtkRendererCollection.h"
+#include "vtkStringScanner.h"
 #include "vtkWebGPUConfiguration.h"
 #include "vtkWebGPURenderWindow.h"
 #include "vtkXWebGPURenderWindow.h"
@@ -55,13 +56,13 @@ vtkXWebGPURenderWindow::vtkXWebGPURenderWindow()
 // free up memory & close the window
 vtkXWebGPURenderWindow::~vtkXWebGPURenderWindow()
 {
-  vtkRenderer* ren;
+  vtkRenderer* renderer;
   vtkCollectionSimpleIterator rit;
   this->Renderers->InitTraversal(rit);
-  while ((ren = this->Renderers->GetNextRenderer(rit)))
+  while ((renderer = this->Renderers->GetNextRenderer(rit)))
   {
-    ren->ReleaseGraphicsResources(this);
-    ren->SetRenderWindow(nullptr);
+    renderer->ReleaseGraphicsResources(this);
+    renderer->SetRenderWindow(nullptr);
   }
   this->Renderers->RemoveAllItems();
   // Finalize in turn destroys the WGPUInstance. As a result, it must be called after all renderers
@@ -421,11 +422,11 @@ void vtkXWebGPURenderWindow::WindowInitialize()
   this->CreateAWindow();
 
   // tell our renderers about us
-  vtkRenderer* ren;
-  for (this->Renderers->InitTraversal(); (ren = this->Renderers->GetNextItem());)
+  vtkRenderer* renderer;
+  for (this->Renderers->InitTraversal(); (renderer = this->Renderers->GetNextItem());)
   {
-    ren->SetRenderWindow(nullptr);
-    ren->SetRenderWindow(this);
+    renderer->SetRenderWindow(nullptr);
+    renderer->SetRenderWindow(this);
   }
 }
 
@@ -447,7 +448,7 @@ bool vtkXWebGPURenderWindow::WindowSetup()
 
   if (this->WGPUInit())
   {
-    wgpu::SurfaceDescriptorFromXlibWindow x11SurfDesc = {};
+    wgpu::SurfaceSourceXlibWindow x11SurfDesc = {};
     x11SurfDesc.display = this->GetDisplayId();
     x11SurfDesc.window = this->GetWindowId();
     wgpu::SurfaceDescriptor surfDesc = {};
@@ -734,6 +735,7 @@ bool vtkXWebGPURenderWindow::EnsureDisplay()
 {
   if (!this->DisplayId)
   {
+    XInitThreads();
     this->DisplayId = XOpenDisplay(static_cast<char*>(nullptr));
     if (this->DisplayId == nullptr)
     {
@@ -817,7 +819,7 @@ void vtkXWebGPURenderWindow::SetWindowInfo(const char* info)
   this->EnsureDisplay();
 
   int tmp;
-  sscanf(info, "%i", &tmp);
+  VTK_FROM_CHARS_IF_ERROR_BREAK(info, tmp);
 
   this->SetWindowId(static_cast<Window>(tmp));
 }
@@ -827,7 +829,7 @@ void vtkXWebGPURenderWindow::SetWindowInfo(const char* info)
 void vtkXWebGPURenderWindow::SetNextWindowInfo(const char* info)
 {
   int tmp;
-  sscanf(info, "%i", &tmp);
+  VTK_FROM_CHARS_IF_ERROR_BREAK(info, tmp);
 
   this->SetNextWindowId(static_cast<Window>(tmp));
 }
@@ -840,7 +842,7 @@ void vtkXWebGPURenderWindow::SetParentInfo(const char* info)
   this->EnsureDisplay();
 
   int tmp;
-  sscanf(info, "%i", &tmp);
+  VTK_FROM_CHARS_IF_ERROR_BREAK(info, tmp);
 
   this->SetParentId(static_cast<Window>(tmp));
 }
