@@ -14,12 +14,14 @@
 
 #include "vtkRenderingWebGPUModule.h" // For export macro
 #include "vtkWebGPURenderWindow.h"
-#include <X11/Xlib.h> // Needed for X types used in the public interface
+#include "vtkWrappingHints.h" // For VTK_MARSHALAUTO
+#include <X11/Xlib.h>         // Needed for X types used in the public interface
 
 VTK_ABI_NAMESPACE_BEGIN
 // Forward declarations
 
-class VTKRENDERINGWEBGPU_EXPORT vtkXWebGPURenderWindow : public vtkWebGPURenderWindow
+class VTKRENDERINGWEBGPU_EXPORT VTK_MARSHALAUTO vtkXWebGPURenderWindow
+  : public vtkWebGPURenderWindow
 {
 public:
   /**
@@ -136,6 +138,7 @@ public:
   /**
    * Get this RenderWindow's X display id.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   Display* GetDisplayId();
 
   /**
@@ -148,33 +151,41 @@ public:
    * Set the X display id for this RenderWindow to use to a pre-existing
    * X display id.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   void SetDisplayId(Display*);
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   void SetDisplayId(void*) override;
   ///@}
 
   /**
    * Get this RenderWindow's parent X window id.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   Window GetParentId();
 
   ///@{
   /**
    * Sets the parent of the window that WILL BE created.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   void SetParentId(Window);
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   void SetParentId(void*) override;
   ///@}
 
   /**
    * Get this RenderWindow's X window id.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   Window GetWindowId();
 
   ///@{
   /**
    * Set this RenderWindow's X window id to a pre-existing window.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   void SetWindowId(Window);
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_NOT_SUPPORTED)
   void SetWindowId(void*) override;
   ///@}
 
