@@ -1,40 +1,25 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkApplyColors.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
-/*-------------------------------------------------------------------------
-  Copyright 2008 Sandia Corporation.
-  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-  the U.S. Government retains certain rights in this software.
--------------------------------------------------------------------------*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright 2008 Sandia Corporation
+// SPDX-License-Identifier: LicenseRef-BSD-3-Clause-Sandia-USGov
 #include "vtkApplyColors.h"
 
 #include "vtkAnnotation.h"
 #include "vtkAnnotationLayers.h"
 #include "vtkCellData.h"
 #include "vtkConvertSelection.h"
+#include "vtkDataSet.h"
 #include "vtkGraph.h"
 #include "vtkIdTypeArray.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
-#include "vtkDataSet.h"
 #include "vtkScalarsToColors.h"
 #include "vtkSmartPointer.h"
 #include "vtkTable.h"
 #include "vtkUnsignedCharArray.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkApplyColors);
 vtkCxxSetObjectMacro(vtkApplyColors, PointLookupTable, vtkScalarsToColors);
 vtkCxxSetObjectMacro(vtkApplyColors, CellLookupTable, vtkScalarsToColors);
@@ -60,12 +45,10 @@ vtkApplyColors::vtkApplyColors()
   this->SelectedCellColor[2] = 0.0;
   this->SelectedCellOpacity = 1.0;
   this->SetNumberOfInputPorts(2);
-  this->SetInputArrayToProcess(0, 0, 0,
-    vtkDataObject::FIELD_ASSOCIATION_VERTICES,
-    vtkDataSetAttributes::SCALARS);
-  this->SetInputArrayToProcess(1, 0, 0,
-    vtkDataObject::FIELD_ASSOCIATION_EDGES,
-    vtkDataSetAttributes::SCALARS);
+  this->SetInputArrayToProcess(
+    0, 0, 0, vtkDataObject::FIELD_ASSOCIATION_VERTICES, vtkDataSetAttributes::SCALARS);
+  this->SetInputArrayToProcess(
+    1, 0, 0, vtkDataObject::FIELD_ASSOCIATION_EDGES, vtkDataSetAttributes::SCALARS);
   this->ScalePointLookupTable = true;
   this->ScaleCellLookupTable = true;
   this->UsePointLookupTable = false;
@@ -102,10 +85,8 @@ int vtkApplyColors::FillInputPortInformation(int port, vtkInformation* info)
   return 1;
 }
 
-int vtkApplyColors::RequestData(
-  vtkInformation *vtkNotUsed(request),
-  vtkInformationVector **inputVector,
-  vtkInformationVector *outputVector)
+int vtkApplyColors::RequestData(vtkInformation* vtkNotUsed(request),
+  vtkInformationVector** inputVector, vtkInformationVector* outputVector)
 {
   // get the info objects
   vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
@@ -123,8 +104,7 @@ int vtkApplyColors::RequestData(
   vtkAnnotationLayers* layers = nullptr;
   if (layersInfo)
   {
-    layers = vtkAnnotationLayers::SafeDownCast(
-      layersInfo->Get(vtkDataObject::DATA_OBJECT()));
+    layers = vtkAnnotationLayers::SafeDownCast(layersInfo->Get(vtkDataObject::DATA_OBJECT()));
   }
   vtkDataObject* output = outInfo->Get(vtkDataObject::DATA_OBJECT());
 
@@ -135,8 +115,7 @@ int vtkApplyColors::RequestData(
   vtkTable* table = vtkTable::SafeDownCast(output);
 
   // initialize color arrays
-  vtkSmartPointer<vtkUnsignedCharArray> colorArr1 =
-    vtkSmartPointer<vtkUnsignedCharArray>::New();
+  vtkSmartPointer<vtkUnsignedCharArray> colorArr1 = vtkSmartPointer<vtkUnsignedCharArray>::New();
   colorArr1->SetName(this->PointColorOutputArrayName);
   colorArr1->SetNumberOfComponents(4);
   if (graph)
@@ -154,8 +133,7 @@ int vtkApplyColors::RequestData(
     colorArr1->SetNumberOfTuples(table->GetNumberOfRows());
     table->AddColumn(colorArr1);
   }
-  vtkSmartPointer<vtkUnsignedCharArray> colorArr2 =
-    vtkSmartPointer<vtkUnsignedCharArray>::New();
+  vtkSmartPointer<vtkUnsignedCharArray> colorArr2 = vtkSmartPointer<vtkUnsignedCharArray>::New();
   colorArr2->SetName(this->CellColorOutputArrayName);
   colorArr2->SetNumberOfComponents(4);
   if (graph)
@@ -170,45 +148,43 @@ int vtkApplyColors::RequestData(
   }
 
   unsigned char pointColor[4];
-  pointColor[0] = static_cast<unsigned char>(255*this->DefaultPointColor[0]);
-  pointColor[1] = static_cast<unsigned char>(255*this->DefaultPointColor[1]);
-  pointColor[2] = static_cast<unsigned char>(255*this->DefaultPointColor[2]);
-  pointColor[3] = static_cast<unsigned char>(255*this->DefaultPointOpacity);
+  pointColor[0] = static_cast<unsigned char>(255 * this->DefaultPointColor[0]);
+  pointColor[1] = static_cast<unsigned char>(255 * this->DefaultPointColor[1]);
+  pointColor[2] = static_cast<unsigned char>(255 * this->DefaultPointColor[2]);
+  pointColor[3] = static_cast<unsigned char>(255 * this->DefaultPointOpacity);
   vtkAbstractArray* arr1 = nullptr;
   if (this->PointLookupTable && this->UsePointLookupTable)
   {
     arr1 = this->GetInputAbstractArrayToProcess(0, inputVector);
   }
-  this->ProcessColorArray(colorArr1, this->PointLookupTable, arr1,
-    pointColor, this->ScalePointLookupTable);
+  this->ProcessColorArray(
+    colorArr1, this->PointLookupTable, arr1, pointColor, this->ScalePointLookupTable);
 
   unsigned char cellColor[4];
-  cellColor[0] = static_cast<unsigned char>(255*this->DefaultCellColor[0]);
-  cellColor[1] = static_cast<unsigned char>(255*this->DefaultCellColor[1]);
-  cellColor[2] = static_cast<unsigned char>(255*this->DefaultCellColor[2]);
-  cellColor[3] = static_cast<unsigned char>(255*this->DefaultCellOpacity);
+  cellColor[0] = static_cast<unsigned char>(255 * this->DefaultCellColor[0]);
+  cellColor[1] = static_cast<unsigned char>(255 * this->DefaultCellColor[1]);
+  cellColor[2] = static_cast<unsigned char>(255 * this->DefaultCellColor[2]);
+  cellColor[3] = static_cast<unsigned char>(255 * this->DefaultCellOpacity);
   vtkAbstractArray* arr2 = nullptr;
   if (this->CellLookupTable && this->UseCellLookupTable)
   {
     arr2 = this->GetInputAbstractArrayToProcess(1, inputVector);
   }
-  this->ProcessColorArray(colorArr2, this->CellLookupTable, arr2,
-    cellColor, this->ScaleCellLookupTable);
+  this->ProcessColorArray(
+    colorArr2, this->CellLookupTable, arr2, cellColor, this->ScaleCellLookupTable);
 
   if (layers)
   {
-    vtkSmartPointer<vtkIdTypeArray> list1 =
-      vtkSmartPointer<vtkIdTypeArray>::New();
-    vtkSmartPointer<vtkIdTypeArray> list2 =
-      vtkSmartPointer<vtkIdTypeArray>::New();
-    unsigned char annColor[4] = {0, 0, 0, 0};
-    unsigned char prev[4] = {0, 0, 0, 0};
+    vtkSmartPointer<vtkIdTypeArray> list1 = vtkSmartPointer<vtkIdTypeArray>::New();
+    vtkSmartPointer<vtkIdTypeArray> list2 = vtkSmartPointer<vtkIdTypeArray>::New();
+    unsigned char annColor[4] = { 0, 0, 0, 0 };
+    unsigned char prev[4] = { 0, 0, 0, 0 };
     unsigned int numAnnotations = layers->GetNumberOfAnnotations();
     for (unsigned int a = 0; a < numAnnotations; ++a)
     {
       vtkAnnotation* ann = layers->GetAnnotation(a);
       if (ann->GetInformation()->Has(vtkAnnotation::ENABLE()) &&
-          ann->GetInformation()->Get(vtkAnnotation::ENABLE())==0)
+        ann->GetInformation()->Get(vtkAnnotation::ENABLE()) == 0)
       {
         continue;
       }
@@ -221,15 +197,15 @@ int vtkApplyColors::RequestData(
       {
         hasColor = true;
         double* color = ann->GetInformation()->Get(vtkAnnotation::COLOR());
-        annColor[0] = static_cast<unsigned char>(255*color[0]);
-        annColor[1] = static_cast<unsigned char>(255*color[1]);
-        annColor[2] = static_cast<unsigned char>(255*color[2]);
+        annColor[0] = static_cast<unsigned char>(255 * color[0]);
+        annColor[1] = static_cast<unsigned char>(255 * color[1]);
+        annColor[2] = static_cast<unsigned char>(255 * color[2]);
       }
       if (ann->GetInformation()->Has(vtkAnnotation::OPACITY()))
       {
         hasOpacity = true;
         double opacity = ann->GetInformation()->Get(vtkAnnotation::OPACITY());
-        annColor[3] = static_cast<unsigned char>(255*opacity);
+        annColor[3] = static_cast<unsigned char>(255 * opacity);
       }
       if (!hasColor && !hasOpacity)
       {
@@ -273,7 +249,7 @@ int vtkApplyColors::RequestData(
         if (hasOpacity)
         {
           // Combine opacities
-          curColor[3] = static_cast<unsigned char>((prev[3]/255.0)*annColor[3]);
+          curColor[3] = static_cast<unsigned char>((prev[3] / 255.0) * annColor[3]);
         }
         else
         {
@@ -304,7 +280,7 @@ int vtkApplyColors::RequestData(
         if (hasOpacity)
         {
           // Combine opacities
-          curColor[3] = static_cast<unsigned char>((prev[3]/255.0)*annColor[3]);
+          curColor[3] = static_cast<unsigned char>((prev[3] / 255.0) * annColor[3]);
         }
         else
         {
@@ -318,21 +294,21 @@ int vtkApplyColors::RequestData(
       vtkSelection* selection = ann->GetSelection();
       list1 = vtkSmartPointer<vtkIdTypeArray>::New();
       list2 = vtkSmartPointer<vtkIdTypeArray>::New();
-      unsigned char color1[4] = {0, 0, 0, 255};
-      unsigned char color2[4] = {0, 0, 0, 255};
+      unsigned char color1[4] = { 0, 0, 0, 255 };
+      unsigned char color2[4] = { 0, 0, 0, 255 };
       if (this->UseCurrentAnnotationColor)
       {
         if (ann->GetInformation()->Has(vtkAnnotation::COLOR()))
         {
           double* color = ann->GetInformation()->Get(vtkAnnotation::COLOR());
-          color1[0] = static_cast<unsigned char>(255*color[0]);
-          color1[1] = static_cast<unsigned char>(255*color[1]);
-          color1[2] = static_cast<unsigned char>(255*color[2]);
+          color1[0] = static_cast<unsigned char>(255 * color[0]);
+          color1[1] = static_cast<unsigned char>(255 * color[1]);
+          color1[2] = static_cast<unsigned char>(255 * color[2]);
         }
         if (ann->GetInformation()->Has(vtkAnnotation::OPACITY()))
         {
           double opacity = ann->GetInformation()->Get(vtkAnnotation::OPACITY());
-          color1[3] = static_cast<unsigned char>(255*opacity);
+          color1[3] = static_cast<unsigned char>(255 * opacity);
         }
         for (int c = 0; c < 4; ++c)
         {
@@ -341,14 +317,14 @@ int vtkApplyColors::RequestData(
       }
       else
       {
-        color1[0] = static_cast<unsigned char>(255*this->SelectedPointColor[0]);
-        color1[1] = static_cast<unsigned char>(255*this->SelectedPointColor[1]);
-        color1[2] = static_cast<unsigned char>(255*this->SelectedPointColor[2]);
-        color1[3] = static_cast<unsigned char>(255*this->SelectedPointOpacity);
-        color2[0] = static_cast<unsigned char>(255*this->SelectedCellColor[0]);
-        color2[1] = static_cast<unsigned char>(255*this->SelectedCellColor[1]);
-        color2[2] = static_cast<unsigned char>(255*this->SelectedCellColor[2]);
-        color2[3] = static_cast<unsigned char>(255*this->SelectedCellOpacity);
+        color1[0] = static_cast<unsigned char>(255 * this->SelectedPointColor[0]);
+        color1[1] = static_cast<unsigned char>(255 * this->SelectedPointColor[1]);
+        color1[2] = static_cast<unsigned char>(255 * this->SelectedPointColor[2]);
+        color1[3] = static_cast<unsigned char>(255 * this->SelectedPointOpacity);
+        color2[0] = static_cast<unsigned char>(255 * this->SelectedCellColor[0]);
+        color2[1] = static_cast<unsigned char>(255 * this->SelectedCellColor[1]);
+        color2[2] = static_cast<unsigned char>(255 * this->SelectedCellColor[2]);
+        color2[3] = static_cast<unsigned char>(255 * this->SelectedCellOpacity);
       }
       if (graph)
       {
@@ -388,12 +364,8 @@ int vtkApplyColors::RequestData(
   return 1;
 }
 
-void vtkApplyColors::ProcessColorArray(
-  vtkUnsignedCharArray* colorArr,
-  vtkScalarsToColors* lut,
-  vtkAbstractArray* arr,
-  unsigned char color[4],
-  bool scaleToArray)
+void vtkApplyColors::ProcessColorArray(vtkUnsignedCharArray* colorArr, vtkScalarsToColors* lut,
+  vtkAbstractArray* arr, unsigned char color[4], bool scaleToArray)
 {
   if (lut && arr)
   {
@@ -409,14 +381,8 @@ void vtkApplyColors::ProcessColorArray(
       for (vtkIdType i = 0; i < colorArr->GetNumberOfTuples(); ++i)
       {
         double val = arr->GetVariantValue(i).ToDouble();
-        if (val > maxVal)
-        {
-          maxVal = val;
-        }
-        if (val < minVal)
-        {
-          minVal = val;
-        }
+        maxVal = std::max(val, maxVal);
+        minVal = std::min(val, minVal);
       }
     }
 
@@ -424,20 +390,19 @@ void vtkApplyColors::ProcessColorArray(
     double scale = 1.0;
     if (minVal != maxVal)
     {
-      scale = (rng[1]-rng[0])/(maxVal-minVal);
+      scale = (rng[1] - rng[0]) / (maxVal - minVal);
     }
-    unsigned char myColor[4] = {0, 0, 0, 0};
+    unsigned char myColor[4] = { 0, 0, 0, 0 };
     for (vtkIdType i = 0; i < colorArr->GetNumberOfTuples(); ++i)
     {
       double val = arr->GetVariantValue(i).ToDouble();
-      const unsigned char* mappedColor = lut->MapValue(
-        rng[0] + scale*(val - minVal));
+      const unsigned char* mappedColor = lut->MapValue(rng[0] + scale * (val - minVal));
       myColor[0] = mappedColor[0];
       myColor[1] = mappedColor[1];
       myColor[2] = mappedColor[2];
       // Combine the opacity of the lookup table with the
       // default color opacity.
-      myColor[3] = static_cast<unsigned char>((color[3]/255.0)*mappedColor[3]);
+      myColor[3] = static_cast<unsigned char>((color[3] / 255.0) * mappedColor[3]);
       colorArr->SetTypedTuple(i, myColor);
     }
   }
@@ -467,51 +432,38 @@ vtkMTimeType vtkApplyColors::GetMTime()
 
 void vtkApplyColors::PrintSelf(ostream& os, vtkIndent indent)
 {
-  this->Superclass::PrintSelf(os,indent);
-  os << indent << "PointLookupTable: "
-     << (this->PointLookupTable ? "" : "(none)") << endl;
+  this->Superclass::PrintSelf(os, indent);
+  os << indent << "PointLookupTable: " << (this->PointLookupTable ? "" : "(none)") << endl;
   if (this->PointLookupTable)
   {
     this->PointLookupTable->PrintSelf(os, indent.GetNextIndent());
   }
-  os << indent << "CellLookupTable: "
-     << (this->CellLookupTable ? "" : "(none)") << endl;
+  os << indent << "CellLookupTable: " << (this->CellLookupTable ? "" : "(none)") << endl;
   if (this->CellLookupTable)
   {
     this->CellLookupTable->PrintSelf(os, indent.GetNextIndent());
   }
-  os << indent << "DefaultPointColor: "
-    << this->DefaultPointColor[0] << ","
-    << this->DefaultPointColor[1] << ","
-    << this->DefaultPointColor[2] << endl;
+  os << indent << "DefaultPointColor: " << this->DefaultPointColor[0] << ","
+     << this->DefaultPointColor[1] << "," << this->DefaultPointColor[2] << endl;
   os << indent << "DefaultPointOpacity: " << this->DefaultPointOpacity << endl;
-  os << indent << "DefaultCellColor: "
-    << this->DefaultCellColor[0] << ","
-    << this->DefaultCellColor[1] << ","
-    << this->DefaultCellColor[2] << endl;
+  os << indent << "DefaultCellColor: " << this->DefaultCellColor[0] << ","
+     << this->DefaultCellColor[1] << "," << this->DefaultCellColor[2] << endl;
   os << indent << "DefaultCellOpacity: " << this->DefaultCellOpacity << endl;
-  os << indent << "SelectedPointColor: "
-    << this->SelectedPointColor[0] << ","
-    << this->SelectedPointColor[1] << ","
-    << this->SelectedPointColor[2] << endl;
+  os << indent << "SelectedPointColor: " << this->SelectedPointColor[0] << ","
+     << this->SelectedPointColor[1] << "," << this->SelectedPointColor[2] << endl;
   os << indent << "SelectedPointOpacity: " << this->SelectedPointOpacity << endl;
-  os << indent << "SelectedCellColor: "
-    << this->SelectedCellColor[0] << ","
-    << this->SelectedCellColor[1] << ","
-    << this->SelectedCellColor[2] << endl;
+  os << indent << "SelectedCellColor: " << this->SelectedCellColor[0] << ","
+     << this->SelectedCellColor[1] << "," << this->SelectedCellColor[2] << endl;
   os << indent << "SelectedCellOpacity: " << this->SelectedCellOpacity << endl;
-  os << indent << "ScalePointLookupTable: "
-    << (this->ScalePointLookupTable ? "on" : "off") << endl;
-  os << indent << "ScaleCellLookupTable: "
-    << (this->ScaleCellLookupTable ? "on" : "off") << endl;
-  os << indent << "UsePointLookupTable: "
-    << (this->UsePointLookupTable ? "on" : "off") << endl;
-  os << indent << "UseCellLookupTable: "
-    << (this->UseCellLookupTable ? "on" : "off") << endl;
+  os << indent << "ScalePointLookupTable: " << (this->ScalePointLookupTable ? "on" : "off") << endl;
+  os << indent << "ScaleCellLookupTable: " << (this->ScaleCellLookupTable ? "on" : "off") << endl;
+  os << indent << "UsePointLookupTable: " << (this->UsePointLookupTable ? "on" : "off") << endl;
+  os << indent << "UseCellLookupTable: " << (this->UseCellLookupTable ? "on" : "off") << endl;
   os << indent << "PointColorOutputArrayName: "
-    << (this->PointColorOutputArrayName ? this->PointColorOutputArrayName : "(none)") << endl;
+     << (this->PointColorOutputArrayName ? this->PointColorOutputArrayName : "(none)") << endl;
   os << indent << "CellColorOutputArrayName: "
-    << (this->CellColorOutputArrayName ? this->CellColorOutputArrayName : "(none)") << endl;
-  os << indent << "UseCurrentAnnotationColor: "
-    << (this->UseCurrentAnnotationColor ? "on" : "off") << endl;
+     << (this->CellColorOutputArrayName ? this->CellColorOutputArrayName : "(none)") << endl;
+  os << indent << "UseCurrentAnnotationColor: " << (this->UseCurrentAnnotationColor ? "on" : "off")
+     << endl;
 }
+VTK_ABI_NAMESPACE_END
