@@ -1,40 +1,30 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPythonUtil.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkPythonUtil.h"
+#include "vtkABINamespace.h"
 #include "vtkPythonOverload.h"
 
-#include "vtkSystemIncludes.h"
+#include "PyVTKMethodDescriptor.h"
 
 #include "vtkObject.h"
 #include "vtkPythonCommand.h"
-#include "vtkSmartPointerBase.h"
-#include "vtkStdString.h"
-#include "vtkToolkits.h"
-#include "vtkUnicodeString.h"
+#ifdef PY_LIMITED_API
+#include "vtkSmartPyObject.h"
+#endif
+#include "vtkStringFormatter.h"
+#include "vtkStringScanner.h"
+#include "vtkSystemIncludes.h"
 #include "vtkVariant.h"
 #include "vtkWeakPointer.h"
-#include "vtkWindows.h"
 
 #include <algorithm>
 #include <atomic>
 #include <cstring>
+#include <iostream>
 #include <map>
 #include <sstream>
 #include <string>
-#include <utility>
 #include <vector>
 
 // for uintptr_t
@@ -44,14 +34,21 @@
 #include <cstdint>
 #endif
 
-//--------------------------------------------------------------------
+// Py_HashPointer added in 3.13 and _Py_HashPointer deprecated in 3.14.
+#if PY_VERSION_HEX >= 0x030D0000
+#define PY_HASHPOINTER Py_HashPointer
+#else
+#define PY_HASHPOINTER _Py_HashPointer
+#endif
+
+VTK_ABI_NAMESPACE_BEGIN
+//------------------------------------------------------------------------------
 // A ghost object, can be used to recreate a deleted PyVTKObject
 class PyVTKObjectGhost
 {
 public:
   PyVTKObjectGhost()
-    : vtk_ptr()
-    , vtk_class(nullptr)
+    : vtk_class(nullptr)
     , vtk_dict(nullptr)
   {
   }
@@ -61,14 +58,14 @@ public:
   PyObject* vtk_dict;
 };
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // There are six maps associated with the Python wrappers
 
 // Map VTK objects to python objects (this is also the cornerstone
 // of the vtk/python garbage collection system, because it contains
 // exactly one pointer reference for each VTK object known to python)
 class vtkPythonObjectMap
-  : public std::map<vtkObjectBase*, std::pair<PyObject*, std::atomic<int32_t> > >
+  : public std::map<vtkObjectBase*, std::pair<PyObject*, std::atomic<int32_t>>>
 {
 public:
   ~vtkPythonObjectMap();
@@ -142,6 +139,14 @@ class vtkPythonClassMap : public std::map<std::string, PyVTKClass>
 {
 };
 
+// Map the Pythonic class names to the ones given by GetClassName().
+// These differ only for templated classes derived from vtkObjectBase,
+// where GetClassName() returns typeid(T).name() which in general is
+// not a valid Python name.
+class vtkPythonClassNameMap : public std::map<std::string, std::string>
+{
+};
+
 // Like the ClassMap, for types not derived from vtkObjectBase.
 class vtkPythonSpecialTypeMap : public std::map<std::string, PyVTKSpecialType>
 {
@@ -163,7 +168,7 @@ class vtkPythonModuleList : public std::vector<std::string>
 };
 
 // Keep track of all vtkPythonCommand instances.
-class vtkPythonCommandList : public std::vector<vtkWeakPointer<vtkPythonCommand> >
+class vtkPythonCommandList : public std::vector<vtkWeakPointer<vtkPythonCommand>>
 {
 public:
   ~vtkPythonCommandList()
@@ -184,7 +189,7 @@ public:
   }
 };
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // The singleton for vtkPythonUtil
 
 static vtkPythonUtil* vtkPythonMap = nullptr;
@@ -206,12 +211,13 @@ void vtkPythonUtilCreateIfNeeded()
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkPythonUtil::vtkPythonUtil()
 {
   this->ObjectMap = new vtkPythonObjectMap;
   this->GhostMap = new vtkPythonGhostMap;
   this->ClassMap = new vtkPythonClassMap;
+  this->ClassNameMap = new vtkPythonClassNameMap;
   this->SpecialTypeMap = new vtkPythonSpecialTypeMap;
   this->NamespaceMap = new vtkPythonNamespaceMap;
   this->EnumMap = new vtkPythonEnumMap;
@@ -219,12 +225,13 @@ vtkPythonUtil::vtkPythonUtil()
   this->PythonCommandList = new vtkPythonCommandList;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkPythonUtil::~vtkPythonUtil()
 {
   delete this->ObjectMap;
   delete this->GhostMap;
   delete this->ClassMap;
+  delete this->ClassNameMap;
   delete this->SpecialTypeMap;
   delete this->NamespaceMap;
   delete this->EnumMap;
@@ -232,17 +239,31 @@ vtkPythonUtil::~vtkPythonUtil()
   delete this->PythonCommandList;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
+void vtkPythonUtil::Initialize()
+{
+  // create the singleton
+  vtkPythonUtilCreateIfNeeded();
+  // finalize our custom MethodDescriptor type
+  PyType_Ready(&PyVTKMethodDescriptor_Type);
+}
+
+//------------------------------------------------------------------------------
+bool vtkPythonUtil::IsInitialized()
+{
+  return (vtkPythonMap != nullptr);
+}
+
+//------------------------------------------------------------------------------
 void vtkPythonUtil::RegisterPythonCommand(vtkPythonCommand* cmd)
 {
   if (cmd)
   {
-    vtkPythonUtilCreateIfNeeded();
     vtkPythonMap->PythonCommandList->push_back(cmd);
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonUtil::UnRegisterPythonCommand(vtkPythonCommand* cmd)
 {
   if (cmd && vtkPythonMap)
@@ -251,12 +272,11 @@ void vtkPythonUtil::UnRegisterPythonCommand(vtkPythonCommand* cmd)
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyTypeObject* vtkPythonUtil::AddSpecialTypeToMap(
   PyTypeObject* pytype, PyMethodDef* methods, PyMethodDef* constructors, vtkcopyfunc copyfunc)
 {
-  const char* classname = vtkPythonUtil::StripModule(pytype->tp_name);
-  vtkPythonUtilCreateIfNeeded();
+  const char* classname = vtkPythonUtil::StripModuleFromType(pytype);
 
   // lets make sure it isn't already there
   vtkPythonSpecialTypeMap::iterator i = vtkPythonMap->SpecialTypeMap->find(classname);
@@ -270,7 +290,7 @@ PyTypeObject* vtkPythonUtil::AddSpecialTypeToMap(
   return i->second.py_type;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyVTKSpecialType* vtkPythonUtil::FindSpecialType(const char* classname)
 {
   if (vtkPythonMap)
@@ -286,11 +306,9 @@ PyVTKSpecialType* vtkPythonUtil::FindSpecialType(const char* classname)
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonUtil::AddObjectToMap(PyObject* obj, vtkObjectBase* ptr)
 {
-  vtkPythonUtilCreateIfNeeded();
-
 #ifdef VTKPYTHONDEBUG
   vtkGenericWarningMacro("Adding an object to map ptr = " << ptr);
 #endif
@@ -303,7 +321,7 @@ void vtkPythonUtil::AddObjectToMap(PyObject* obj, vtkObjectBase* ptr)
 #endif
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonUtil::RemoveObjectFromMap(PyObject* obj)
 {
   PyVTKObject* pobj = (PyVTKObject*)obj;
@@ -363,7 +381,7 @@ void vtkPythonUtil::RemoveObjectFromMap(PyObject* obj)
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyObject* vtkPythonUtil::FindObject(vtkObjectBase* ptr)
 {
   PyObject* obj = nullptr;
@@ -403,7 +421,7 @@ PyObject* vtkPythonUtil::FindObject(vtkObjectBase* ptr)
   return obj;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyObject* vtkPythonUtil::GetObjectFromPointer(vtkObjectBase* ptr)
 {
   PyObject* obj = vtkPythonUtil::FindObject(ptr);
@@ -437,7 +455,7 @@ PyObject* vtkPythonUtil::GetObjectFromPointer(vtkObjectBase* ptr)
   return obj;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 const char* vtkPythonUtil::PythonicClassName(const char* classname)
 {
   const char* cp = classname;
@@ -454,17 +472,32 @@ const char* vtkPythonUtil::PythonicClassName(const char* classname)
   if (*cp != '\0')
   {
     /* look up class and get its pythonic name */
-    PyTypeObject* pytype = vtkPythonUtil::FindClassTypeObject(classname);
+    PyTypeObject* pytype = vtkPythonUtil::FindBaseTypeObject(classname);
     if (pytype)
     {
-      classname = vtkPythonUtil::StripModule(pytype->tp_name);
+      classname = vtkPythonUtil::StripModuleFromType(pytype);
     }
   }
 
   return classname;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
+const char* vtkPythonUtil::VTKClassName(const char* pyname)
+{
+  if (vtkPythonMap && pyname != nullptr)
+  {
+    vtkPythonClassNameMap::iterator it = vtkPythonMap->ClassNameMap->find(pyname);
+    if (it != vtkPythonMap->ClassNameMap->end())
+    {
+      return it->second.c_str();
+    }
+  }
+
+  return pyname;
+}
+
+//------------------------------------------------------------------------------
 const char* vtkPythonUtil::StripModule(const char* tpname)
 {
   const char* cp = tpname;
@@ -479,12 +512,39 @@ const char* vtkPythonUtil::StripModule(const char* tpname)
   return strippedname;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
+const char* vtkPythonUtil::StripModuleFromType(PyTypeObject* pytype)
+{
+  return vtkPythonUtil::StripModule(vtkPythonUtil::GetTypeName(pytype));
+}
+
+//------------------------------------------------------------------------------
+const char* vtkPythonUtil::StripModuleFromObject(PyObject* ob)
+{
+  return vtkPythonUtil::StripModuleFromType(Py_TYPE(ob));
+}
+
+//------------------------------------------------------------------------------
+const char* vtkPythonUtil::GetTypeName(PyTypeObject* pytype)
+{
+#ifdef PY_LIMITED_API
+  vtkSmartPyObject tname = PyType_GetName(pytype);
+  return PyUnicode_AsUTF8AndSize(tname, nullptr);
+#else
+  return pytype->tp_name;
+#endif
+}
+
+//------------------------------------------------------------------------------
+const char* vtkPythonUtil::GetTypeNameForObject(PyObject* ob)
+{
+  return vtkPythonUtil::GetTypeName(Py_TYPE(ob));
+}
+
+//------------------------------------------------------------------------------
 PyTypeObject* vtkPythonUtil::AddClassToMap(
   PyTypeObject* pytype, PyMethodDef* methods, const char* classname, vtknewfunc constructor)
 {
-  vtkPythonUtilCreateIfNeeded();
-
   // lets make sure it isn't already there
   vtkPythonClassMap::iterator i = vtkPythonMap->ClassMap->find(classname);
   if (i == vtkPythonMap->ClassMap->end())
@@ -492,12 +552,22 @@ PyTypeObject* vtkPythonUtil::AddClassToMap(
     i = vtkPythonMap->ClassMap->insert(i,
       vtkPythonClassMap::value_type(
         classname, PyVTKClass(pytype, methods, classname, constructor)));
+
+    // if Python type name differs from VTK ClassName, store in ClassNameMap
+    // (this only occurs for templated classes, due to their GetClassName()
+    // implementation in their type macro in vtkSetGet.h)
+    const char* pyname = vtkPythonUtil::StripModuleFromType(pytype);
+    if (strcmp(pyname, classname) != 0)
+    {
+      vtkPythonMap->ClassNameMap->insert(
+        vtkPythonMap->ClassNameMap->end(), vtkPythonClassNameMap::value_type(pyname, classname));
+    }
   }
 
   return i->second.py_type;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyVTKClass* vtkPythonUtil::FindClass(const char* classname)
 {
   if (vtkPythonMap)
@@ -512,7 +582,7 @@ PyVTKClass* vtkPythonUtil::FindClass(const char* classname)
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // this is a helper function to find the nearest base class for an
 // object whose class is not in the ClassDict
 PyVTKClass* vtkPythonUtil::FindNearestBaseClass(vtkObjectBase* ptr)
@@ -528,11 +598,21 @@ PyVTKClass* vtkPythonUtil::FindNearestBaseClass(vtkObjectBase* ptr)
 
     if (ptr->IsA(pyclass->vtk_name))
     {
-      PyTypeObject* base = pyclass->py_type->tp_base;
+      PyTypeObject* base =
+#if PY_VERSION_HEX >= 0x030A0000
+        (PyTypeObject*)PyType_GetSlot(pyclass->py_type, Py_tp_base)
+#else
+        pyclass->py_type->tp_base
+#endif
+        ;
       // count the hierarchy depth for this class
       for (depth = 0; base != nullptr; depth++)
       {
+#if PY_VERSION_HEX >= 0x030A0000
+        base = (PyTypeObject*)PyType_GetSlot(base, Py_tp_base);
+#else
         base = base->tp_base;
+#endif
       }
       // we want the class that is furthest from vtkObjectBase
       if (depth > maxdepth)
@@ -546,7 +626,7 @@ PyVTKClass* vtkPythonUtil::FindNearestBaseClass(vtkObjectBase* ptr)
   return nearestbase;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 vtkObjectBase* vtkPythonUtil::GetPointerFromObject(PyObject* obj, const char* result_type)
 {
   vtkObjectBase* ptr;
@@ -614,9 +694,11 @@ vtkObjectBase* vtkPythonUtil::GetPointerFromObject(PyObject* obj, const char* re
 #ifdef VTKPYTHONDEBUG
     vtkGenericWarningMacro("vtk bad argument, type conversion failed.");
 #endif
-    snprintf(error_string, sizeof(error_string), "method requires a %.500s, a %.500s was provided.",
+    auto result = vtk::format_to_n(error_string, sizeof(error_string),
+      "method requires a {:.500s}, a {:.500s} was provided.",
       vtkPythonUtil::PythonicClassName(result_type),
-      vtkPythonUtil::PythonicClassName(((vtkObjectBase*)ptr)->GetClassName()));
+      vtkPythonUtil::PythonicClassName(ptr->GetClassName()));
+    *result.out = '\0';
     PyErr_SetString(PyExc_TypeError, error_string);
     return nullptr;
   }
@@ -624,67 +706,74 @@ vtkObjectBase* vtkPythonUtil::GetPointerFromObject(PyObject* obj, const char* re
 
 //----------------
 // union of long int and pointer
-union vtkPythonUtilPointerUnion {
+union vtkPythonUtilPointerUnion
+{
   void* p;
   uintptr_t l;
 };
 
 //----------------
 // union of long int and pointer
-union vtkPythonUtilConstPointerUnion {
+union vtkPythonUtilConstPointerUnion
+{
   const void* p;
   uintptr_t l;
 };
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyObject* vtkPythonUtil::GetObjectFromObject(PyObject* arg, const char* type)
 {
   union vtkPythonUtilPointerUnion u;
   PyObject* tmp = nullptr;
 
-#ifdef Py_USING_UNICODE
   if (PyUnicode_Check(arg))
   {
     tmp = PyUnicode_AsUTF8String(arg);
     arg = tmp;
   }
-#endif
 
   if (PyBytes_Check(arg))
   {
     vtkObjectBase* ptr;
     char* ptrText = PyBytes_AsString(arg);
-
-    char typeCheck[1024]; // typeCheck is currently not used
-    unsigned long long l;
-    int i = sscanf(ptrText, "_%llx_%s", &l, typeCheck);
-    u.l = static_cast<uintptr_t>(l);
-
-    if (i <= 0)
+    const auto text = std::string_view(ptrText);
+    std::string_view typeCheck;
+    auto lAndTypeResult = vtk::scan<unsigned long long, std::string_view>(text, "_{:x}_{:s}");
+    if (lAndTypeResult)
     {
-      i = sscanf(ptrText, "Addr=0x%llx", &l);
-      u.l = static_cast<uintptr_t>(l);
+      std::tie(u.l, typeCheck) = lAndTypeResult->values();
     }
-    if (i <= 0)
+    else
     {
-      i = sscanf(ptrText, "%p", &u.p);
+      auto lResult = vtk::scan<unsigned long long>(text, "Addr=0x{:x}");
+      if (lResult)
+      {
+        u.l = lResult->value();
+      }
+      else
+      {
+        auto pResult = vtk::scan<void*>(text, "{:p}");
+        if (pResult)
+        {
+          u.p = pResult->value();
+        }
+        else
+        {
+          Py_XDECREF(tmp);
+          PyErr_SetString(PyExc_ValueError, "could not extract address from argument string");
+          return nullptr;
+        }
+      }
     }
-    if (i <= 0)
-    {
-      Py_XDECREF(tmp);
-      PyErr_SetString(
-        PyExc_ValueError, "could not extract hexadecimal address from argument string");
-      return nullptr;
-    }
-
     ptr = static_cast<vtkObjectBase*>(u.p);
 
     if (!ptr->IsA(type))
     {
       char error_string[2048];
-      snprintf(error_string, sizeof(error_string),
-        "method requires a %.500s address, a %.500s address was provided.", type,
+      auto result = vtk::format_to_n(error_string, sizeof(error_string),
+        "method requires a {:.500s} address, a {:.500s} address was provided.", type,
         ptr->GetClassName());
+      *result.out = '\0';
       Py_XDECREF(tmp);
       PyErr_SetString(PyExc_TypeError, error_string);
       return nullptr;
@@ -699,7 +788,7 @@ PyObject* vtkPythonUtil::GetObjectFromObject(PyObject* arg, const char* type)
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void* vtkPythonUtil::GetPointerFromSpecialObject(
   PyObject* obj, const char* result_type, PyObject** newobj)
 {
@@ -709,7 +798,7 @@ void* vtkPythonUtil::GetPointerFromSpecialObject(
     return nullptr;
   }
 
-  const char* object_type = vtkPythonUtil::StripModule(Py_TYPE(obj)->tp_name);
+  const char* object_type = vtkPythonUtil::StripModuleFromObject(obj);
 
   // do a lookup on the desired type
   vtkPythonSpecialTypeMap::iterator it = vtkPythonMap->SpecialTypeMap->find(result_type);
@@ -731,12 +820,13 @@ void* vtkPythonUtil::GetPointerFromSpecialObject(
     // If a constructor signature exists for "obj", call it
     if (meth && meth->ml_meth)
     {
-      PyObject* args = PyTuple_New(1);
-      PyTuple_SET_ITEM(args, 0, obj);
-      Py_INCREF(obj);
-
-      sobj = meth->ml_meth(nullptr, args);
-
+      PyObject* args = PyTuple_Pack(1, obj);
+      PyObject* func = PyCFunction_New(meth, nullptr);
+      if (func)
+      {
+        sobj = PyObject_Call(func, args, nullptr);
+        Py_DECREF(func);
+      }
       Py_DECREF(args);
     }
 
@@ -749,8 +839,9 @@ void* vtkPythonUtil::GetPointerFromSpecialObject(
     {
       char error_text[2048];
       Py_DECREF(sobj);
-      snprintf(error_text, sizeof(error_text), "cannot pass %.500s as a non-const %.500s reference",
-        object_type, result_type);
+      auto result = vtk::format_to_n(error_text, sizeof(error_text),
+        "cannot pass {:.500s} as a non-const {:.500s} reference", object_type, result_type);
+      *result.out = '\0';
       PyErr_SetString(PyExc_TypeError, error_text);
       return nullptr;
     }
@@ -775,14 +866,15 @@ void* vtkPythonUtil::GetPointerFromSpecialObject(
 #endif
 
   char error_string[2048];
-  snprintf(error_string, sizeof(error_string), "method requires a %.500s, a %.500s was provided.",
-    result_type, object_type);
+  auto result = vtk::format_to_n(error_string, sizeof(error_string),
+    "method requires a {:.500s}, a {:.500s} was provided.", result_type, object_type);
+  *result.out = '\0';
   PyErr_SetString(PyExc_TypeError, error_string);
 
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonUtil::AddNamespaceToMap(PyObject* module)
 {
   if (!PyVTKNamespace_Check(module))
@@ -790,8 +882,6 @@ void vtkPythonUtil::AddNamespaceToMap(PyObject* module)
     return;
   }
 
-  vtkPythonUtilCreateIfNeeded();
-
   const char* name = PyVTKNamespace_GetName(module);
   // let's make sure it isn't already there
   vtkPythonNamespaceMap::iterator i = vtkPythonMap->NamespaceMap->find(name);
@@ -803,7 +893,7 @@ void vtkPythonUtil::AddNamespaceToMap(PyObject* module)
   (*vtkPythonMap->NamespaceMap)[name] = module;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // This method is called from PyVTKNamespace_Delete
 void vtkPythonUtil::RemoveNamespaceFromMap(PyObject* obj)
 {
@@ -820,7 +910,7 @@ void vtkPythonUtil::RemoveNamespaceFromMap(PyObject* obj)
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyObject* vtkPythonUtil::FindNamespace(const char* name)
 {
   if (vtkPythonMap)
@@ -835,11 +925,9 @@ PyObject* vtkPythonUtil::FindNamespace(const char* name)
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonUtil::AddEnumToMap(PyTypeObject* enumtype, const char* name)
 {
-  vtkPythonUtilCreateIfNeeded();
-
   // Only add to map if it isn't already there
   vtkPythonEnumMap::iterator i = vtkPythonMap->EnumMap->find(name);
   if (i == vtkPythonMap->EnumMap->end())
@@ -848,7 +936,7 @@ void vtkPythonUtil::AddEnumToMap(PyTypeObject* enumtype, const char* name)
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyTypeObject* vtkPythonUtil::FindEnum(const char* name)
 {
   PyTypeObject* pytype = nullptr;
@@ -865,7 +953,35 @@ PyTypeObject* vtkPythonUtil::FindEnum(const char* name)
   return pytype;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
+PyTypeObject* vtkPythonUtil::FindBaseTypeObject(const char* name)
+{
+  PyVTKClass* info = vtkPythonUtil::FindClass(name);
+  if (info)
+  {
+    // in case of override, drill down to get the original (non-override) type,
+    // that's what we need to use for the base class of other wrapped classes
+    for (PyTypeObject* pytype = info->py_type; pytype != nullptr;
+         pytype =
+#if PY_VERSION_HEX >= 0x030A0000
+           (PyTypeObject*)PyType_GetSlot(pytype, Py_tp_base)
+#else
+           pytype->tp_base
+#endif
+    )
+    {
+      if (strcmp(vtkPythonUtil::StripModuleFromType(pytype), name) == 0)
+      {
+        return pytype;
+      }
+    }
+    return info->py_type;
+  }
+
+  return nullptr;
+}
+
+//------------------------------------------------------------------------------
 PyTypeObject* vtkPythonUtil::FindClassTypeObject(const char* name)
 {
   PyVTKClass* info = vtkPythonUtil::FindClass(name);
@@ -877,7 +993,7 @@ PyTypeObject* vtkPythonUtil::FindClassTypeObject(const char* name)
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 PyTypeObject* vtkPythonUtil::FindSpecialTypeObject(const char* name)
 {
   PyVTKSpecialType* info = vtkPythonUtil::FindSpecialType(name);
@@ -889,7 +1005,7 @@ PyTypeObject* vtkPythonUtil::FindSpecialTypeObject(const char* name)
   return nullptr;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 bool vtkPythonUtil::ImportModule(const char* fullname, PyObject* globals)
 {
   // strip all but the final part of the path
@@ -917,8 +1033,8 @@ bool vtkPythonUtil::ImportModule(const char* fullname, PyObject* globals)
 
   if (fullname == name || (fullname[0] == '.' && &fullname[1] == name))
   {
-    // try relative import (const-cast is needed for Python 2.x only)
-    m = PyImport_ImportModuleLevel(const_cast<char*>(name), globals, nullptr, nullptr, 1);
+    // try relative import
+    m = PyImport_ImportModuleLevel(name, globals, nullptr, nullptr, 1);
     if (!m)
     {
       PyErr_Clear();
@@ -941,15 +1057,24 @@ bool vtkPythonUtil::ImportModule(const char* fullname, PyObject* globals)
   return true;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonUtil::AddModule(const char* name)
 {
-  vtkPythonUtilCreateIfNeeded();
-
   vtkPythonMap->ModuleList->push_back(name);
+
+  // Register module name into pending list for deferred side module loading
+  PyObject* pModule = PyImport_ImportModule("vtkmodules");
+  PyObject* pFunc = PyObject_GetAttrString(pModule, "on_vtk_module_init");
+  PyObject* pArgs = PyTuple_New(1);
+  PyTuple_SetItem(pArgs, 0, PyUnicode_FromString(name));
+  PyObject* execVal = PyObject_CallObject(pFunc, pArgs);
+  Py_DECREF(execVal);
+  Py_DECREF(pArgs);
+  Py_DECREF(pFunc);
+  Py_DECREF(pModule);
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // mangle a void pointer into a SWIG-style string
 char* vtkPythonUtil::ManglePointer(const void* ptr, const char* type)
 {
@@ -957,21 +1082,20 @@ char* vtkPythonUtil::ManglePointer(const void* ptr, const char* type)
   int ndigits = 2 * (int)sizeof(void*);
   union vtkPythonUtilConstPointerUnion u;
   u.p = ptr;
-  snprintf(ptrText, sizeof(ptrText), "_%*.*llx_%s", ndigits, ndigits,
-    static_cast<unsigned long long>(u.l), type);
+  auto result = vtk::format_to_n(
+    ptrText, sizeof(ptrText), "_{:0{}x}_{}", static_cast<unsigned long long>(u.l), ndigits, type);
+  *result.out = '\0';
 
   return ptrText;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // unmangle a void pointer from a SWIG-style string
 void* vtkPythonUtil::UnmanglePointer(char* ptrText, int* len, const char* type)
 {
   int i;
   union vtkPythonUtilPointerUnion u;
   char text[1024];
-  char typeCheck[1024];
-  typeCheck[0] = '\0';
 
   // Do some minimal checks that it might be a swig pointer.
   if (*len < 256 && *len > 4 && ptrText[0] == '_')
@@ -993,20 +1117,24 @@ void* vtkPythonUtil::UnmanglePointer(char* ptrText, int* len, const char* type)
     // If no null bytes, then do a full check for a swig pointer
     if (i == 0)
     {
-      unsigned long long l;
-      i = sscanf(text, "_%llx_%s", &l, typeCheck);
-      u.l = static_cast<uintptr_t>(l);
-
-      if (strcmp(type, typeCheck) == 0)
-      { // successfully unmangle
-        *len = 0;
-        return u.p;
-      }
-      else if (i == 2)
-      { // mangled pointer of wrong type
+      const std::string_view textView(text);
+      std::string_view typeCheck;
+      auto lAndTypeResult = vtk::scan<unsigned long long, std::string_view>(textView, "_{:x}_{:s}");
+      if (lAndTypeResult)
+      {
+        std::tie(u.l, typeCheck) = lAndTypeResult->values();
+        if (typeCheck == type)
+        { // successfully unmangle
+          *len = 0;
+          return u.p;
+        }
+        // mangled pointer of wrong type
         *len = -1;
         return nullptr;
       }
+      // mangled pointer of wrong type
+      *len = -1;
+      return nullptr;
     }
   }
 
@@ -1015,7 +1143,7 @@ void* vtkPythonUtil::UnmanglePointer(char* ptrText, int* len, const char* type)
   return (void*)ptrText;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 Py_hash_t vtkPythonUtil::VariantHash(const vtkVariant* v)
 {
   Py_hash_t h = -1;
@@ -1029,31 +1157,14 @@ Py_hash_t vtkPythonUtil::VariantHash(const vtkVariant* v)
   {
     case VTK_OBJECT:
     {
-      h = _Py_HashPointer(v->ToVTKObject());
+      h = PY_HASHPOINTER(v->ToVTKObject());
       break;
     }
 
-#ifdef Py_USING_UNICODE
-    case VTK_UNICODE_STRING:
-    {
-      vtkUnicodeString u = v->ToUnicodeString();
-      const char* s = u.utf8_str();
-      PyObject* tmp = PyUnicode_DecodeUTF8(s, strlen(s), "strict");
-      if (tmp == nullptr)
-      {
-        PyErr_Clear();
-        return 0;
-      }
-      h = PyObject_Hash(tmp);
-      Py_DECREF(tmp);
-      break;
-    }
-#endif
-
     default:
     {
-      vtkStdString s = v->ToString();
-      PyObject* tmp = PyString_FromString(s.c_str());
+      std::string s = v->ToString();
+      PyObject* tmp = PyUnicode_FromString(s.c_str());
       h = PyObject_Hash(tmp);
       Py_DECREF(tmp);
       break;
@@ -1063,7 +1174,7 @@ Py_hash_t vtkPythonUtil::VariantHash(const vtkVariant* v)
   return h;
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonVoidFunc(void* arg)
 {
   PyObject *arglist, *result;
@@ -1077,7 +1188,7 @@ void vtkPythonVoidFunc(void* arg)
     return;
   }
 
-#ifndef VTK_NO_PYTHON_THREADS
+#ifdef VTK_PYTHON_HAS_GIL
   vtkPythonScopeGilEnsurer gilEnsurer(true);
 #endif
 
@@ -1094,14 +1205,14 @@ void vtkPythonVoidFunc(void* arg)
   {
     if (PyErr_ExceptionMatches(PyExc_KeyboardInterrupt))
     {
-      cerr << "Caught a Ctrl-C within python, exiting program.\n";
+      std::cerr << "Caught a Ctrl-C within python, exiting program.\n";
       Py_Exit(1);
     }
     PyErr_Print();
   }
 }
 
-//--------------------------------------------------------------------
+//------------------------------------------------------------------------------
 void vtkPythonVoidFuncArgDelete(void* arg)
 {
   PyObject* func = (PyObject*)arg;
@@ -1114,7 +1225,7 @@ void vtkPythonVoidFuncArgDelete(void* arg)
     return;
   }
 
-#ifndef VTK_NO_PYTHON_THREADS
+#ifdef VTK_PYTHON_HAS_GIL
   vtkPythonScopeGilEnsurer gilEnsurer(true);
 #endif
 
@@ -1123,3 +1234,34 @@ void vtkPythonVoidFuncArgDelete(void* arg)
     Py_DECREF(func);
   }
 }
+
+//------------------------------------------------------------------------------
+PyGetSetDef* vtkPythonUtil::FindGetSetDescriptor(PyTypeObject* pytype, PyObject* key)
+{
+  // Check if tp_dict is present
+  if (pytype->tp_dict != nullptr && PyDict_Check(pytype->tp_dict))
+  {
+    // Check if the attribute is in the dictionary
+    PyObject* attr = PyDict_GetItem(pytype->tp_dict, key);
+    if (attr != nullptr)
+    {
+      PyDescrObject* descr = (PyDescrObject*)attr;
+      if (pytype == descr->d_type || PyType_IsSubtype(pytype, descr->d_type))
+      {
+        PyGetSetDescrObject* getsetDescr = (PyGetSetDescrObject*)descr;
+        if (getsetDescr->d_getset != nullptr)
+        {
+          return getsetDescr->d_getset;
+        }
+      }
+    }
+  }
+  // Recursively check in base types
+  if (pytype->tp_base != nullptr)
+  {
+    return vtkPythonUtil::FindGetSetDescriptor(pytype->tp_base, key);
+  }
+  // No matching getset descriptor found
+  return nullptr;
+}
+VTK_ABI_NAMESPACE_END
